
// *****************************************************************************
//
// source:    AC_LUMI.C
//
// type:      source code
//
// created:   13 Dec 2000
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_LUMI_init
//            AC_LUMI_evnt
//            acLumiEvnt
//
// called by: AC_init()
//            AC_evnt()
//            rcevnt_()
//
// purpose:   Data Quality Monitoring using rhp
//
// hints:     use ARTE_error() to publish error messages
//            steered by /AC/SUBD LUMI rec ON
//
// changes:
// 13 Dec 2000 (AG) creation
// 21 Dec 2000 (AS) addeded code to fill HITL
// 21 Dec 2000 (AG) introduced acLumiEvnt
// 21 Dec 2000 (AG) cured small bug
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "alca/AC.h"
#include "alca/AC_LUMI.hh"


// externals
extern int FADEB; // can be set by runargs=-debug in the Makefile

//
// function code
//
int
AC_LUMI_init()
{
  ARTE_mark( "AC_LUMI_init 0", 0 );
  cout << "AC_LUMI_init" << endl;
  
  //
  // put your code here
  //
  int rc = 0;
  
  // example
  rhp_hbook1( AC_ID( COMP_LUMI, LVL_REC, 1 ), "LUMI: test", 800, -0.5, 399.5 );
  
  //
  // return
  //
  ARTE_mark( "AC_LUMI_init 1", 0 );
  return rc; // return error code (0 = okay)
}

//
// function code
//
int
AC_LUMI_evnt()
{
  ARTE_mark( "AC_LUMI_evnt 0", 0 );
  
  //
  // put your code here
  //
  int rc = 0;
 
  // loop over HITL 
  float E_sum = 0;
  for( ArteTable<HITL>::iterator hitl = ArteTable<HITL>::begin();
       hitl != ArteTable<HITL>::end(); ++hitl ){

    // sum up cells with
    float energy = (*hitl)->ener;
    if( energy > 3.) E_sum += energy;
  }
  
  // example
  rhp_hf1( AC_ID( COMP_LUMI, LVL_REC, 1 ), E_sum, 1 );
  
  //
  // return
  //
  ARTE_mark( "AC_LUMI_evnt 1", 0 );
  return rc; // return error code (0 = okay)
}

//
// function code
//
void
acLumiEvnt()
{
  ARTE_mark( "acLumiEvnt 0", 0 );

  // check mode
  if( !AC_flag( COMP_LUMI ) ){
    ARTE_mark( "acLumiEvnt 1", 0 );
    return;
  }

  // loop over HITC and fill HITL
  for( ArteTable<HITC>::iterator hitc = ArteTable<HITC>::begin();
       hitc != ArteTable<HITC>::end();
       ++hitc ){
    
    // get cell id
    int fed = ArtePointer<ECHS>(((*hitc)->gcce).index())->fdid;
    int ncl = 0;
    if(fed <  0x64)               ncl = 5; // inner
    if(fed >= 0x64 && fed < 0xb4) ncl = 2; // middle
    if(fed >= 0xb4)               ncl = 1; // outer

    // check for inner ECAL
    if( ncl == 5 ){
      HITL hitl;  
	
      hitl.cell = ( (*hitc)->gcce ).index();
      hitl.ener = (*hitc)->q;
      ArtePointer<HITL>::append(hitl);
      //cout << "acLumiEvnt: HITL cell="  <<  (*hitc)->gcce << endl; 
      //cout << "acLumiEvnt: HITL ener="  <<  (*hitc)->q    << endl;
    }
  }  

  // return
  ARTE_mark( "acLumiEvnt 1", 0 );
  return;
}
