
// ****************************************************************************
//
// Routine:      acReadGlobalByKey
// Author:       T.Oest
//
// Description:  read global alignment constants from database
//               using the event key
//
// Called by:    acReadGlobal
//
// Hints:        KTstatus:
//               -2: no key or error
//               -1: no key or error
//               0 : = 2
//               1 : new keytable
//               2 : new keytable
//
// Modification:
// 16 Aug 2000 (AG) corrected include path bug
// 23 Nov 2000 (AG) added warnings
// 13 Aug 2001 (JH) printout commented out
//
// ****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>
#include <limits.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// ONLINE includes
#include "esp.h"
#include "table.h"
#include "keytable.h"

// globals
static int GlobalAlignServer = -1;
static int ACGAtable = -1;

// prptotypes
void ackuexec( char * KuipCommand );

//*****************************************************************
// function
//*****************************************************************

int
acReadGlobalByKey( int KTstatus, char * dbserver )
{
  // show up
  ARTE_mark( "acReadGlobalByKey 0", 0 );
  
  int ACGAupdate = 0;
  
  //==================================================================
  // get update information from key table
  //==================================================================
  
  switch( KTstatus )
    {     
      //
      // get new constants
      //
    case 2:
      
      // check server
      ARTE_mark( "KeyFindServer", 0 );
      GlobalAlignServer = KeyFindServer( dbserver );
      if( GlobalAlignServer == -1 ){
	ARTE_exit( GRR_SFATAL, "acReadGlobalByKey: Server access failed.", 0 );
	ARTE_mark( "acReadGlobalByKey 1", 0 );
	return -1;
      }
      
      // check table
      ARTE_mark( "KeyFindTable", 0 );
      ACGAtable = KeyFindTable( GlobalAlignServer, "ACGA" );
      if( ACGAtable == -1 ){
	ARTE_exit( GRR_SFATAL, "acReadGlobalByKey: ACGA access failed.", 0 );
	ARTE_mark( "acReadGlobalByKey 1", 0 );
	return -2;
      }
      
      //
      // check ACGA
      //
    case 1:
      
      if( !KeyTableNew( ACGAtable ) ){
	// ARTE_error( GRR_SINFO, "acReadGlobalByKey: ACGA not new.", 0 );
	ARTE_mark( "acReadGlobalByKey 1", 0 );
	return 0;
      }
      
      // major and minor
      int major = KeyTableMajor( ACGAtable );
      int minor = KeyTableMinor( ACGAtable );
      
      //==================================================================
      // read in new ACGA table if table not existing or update necessay
      //==================================================================
      
      char LunName[5] = { 'D', 'B', 'G', 'A', 0 };
      char * KuipCommand =  (char*) malloc( max(128, int(24+strlen(dbserver))) );
      
      sprintf ( KuipCommand,"dbopen %s ro",LunName );
      ackuexec( KuipCommand );
      
      ArteTable<ACGA>::clear();
      sprintf ( KuipCommand,"dbread %s ACGA %i %i eq",LunName,major,minor );
      ackuexec( KuipCommand );
      
      sprintf ( KuipCommand,"dbclose %s",LunName );
      ackuexec( KuipCommand );
      
      ACGAupdate = 1;
      
      break;     
    }
  
  //==================================================================
  // check if ACGA table exist
  //==================================================================
  if( ArteTable<ACGA>::size() == 0 ) ACGAupdate = -3;
  
  ARTE_mark( "acReadGlobalByKey 1", 0 );
  return ACGAupdate;
}
