*-- Author :    Ulrich Husemann  09/28/01

      subroutine ALCOUN(ngluons,ntracks,nstrings)

************************************************************************
*
* Description:  Routine to scan through the LUJETS common block and
*               count all gluons,strings, and charged tracks roughly within 
*               the acceptance of the HERA-B detector
*
*-- Author :    Ulrich Husemann  09/28/01
*
* Called from: ALPYTH
*
* External references: LUND, PYTHIA
*
* Arguments: returns number of gluons,strings, and charged tracks
*
************************************************************************

      implicit none

#include "jets/LUJETS.fi"
#include "jets/LUDAT1.fi"

      real tx,ty,txmin,txmax,tymin,tymax
      integer i,ngluons,ntracks,nstrings,LUCHGE
      external LUCHGE

      parameter(txmin = 0.01)
      parameter(txmax = 0.22)
      parameter(tymin = 0.01)
      parameter(tymax = 0.22)

      ngluons  = 0
      ntracks  = 0
      nstrings = 0

      do i = 1,N
c gluons have LUND code 21
         if (K(i,2).eq.21) ngluons = ngluons + 1
c search for stable charged tracks        
         if (K(i,1).eq.1.and.LUCHGE(K(i,2)).ne.0) then
c check roughly the acceptance (10-220 mrad)
            tx = abs(P(i,1)/P(i,3))
            ty = abs(P(i,2)/P(i,3))
            if((tx.gt.txmin.or.ty.gt.tymin).and.
     +           (tx.lt.txmax.and.ty.lt.tymax)) ntracks = ntracks + 1
         endif
c strings have LUND code 92
         if (K(i,2).eq.92) nstrings = nstrings + 1
      enddo

      end
