*-- Author : U. Schwanke   

      SUBROUTINE ALDYNLO (IFLAG)

************************************************************************
*
* Description: Selection of PYTHIA Drell-Yan leptons
*              as preparation of generation of underlying event (FRITIOF)
* 
*-- Authors :  U. Schwanke
*   
* Called from: ALPYTH
*
* External references: LUND, PYTHIA
*
* Arguments: IFLAG 
*   on exit, IFLAG =
*            1  if e+e- or mu+mu- pair generated
*            0  no such pair
* 
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "jets/LUDAT1.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "arte/luopts.fi"

      INTEGER IFLAG
      integer z,i,j,imu1,imu2,im,mag

#include "arte/Arte.ff"

C assume failure
      IFLAG = 0
      imu1 = -1
      imu2 = -1
      z = -1

C   find outgoing gamma^* or Z
      do i=7,8 
          if( K(MSTI(i),2).eq.22 .OR. K(MSTI(i),2).eq.23 ) then
             z = i
          endif
      enddo 

      if( z.ne.-1 ) then
         do i=1,N          
            if( K(i,1).eq.21 .and. K(i,3).eq.z ) then  ! comment line plus particle from Z/gamma
               mag = abs(K(i,2))                       ! magnitude of particle code
               if( mag.eq.11 .or. mag.eq.13 ) then     ! electron or muon
                  if( imu1.eq.-1 ) then
                     imu1 = i
                  else
                     imu2 = i
                  endif
               endif
            endif
         enddo
         if( imu2.ne.-1 .and. K(imu1,2).eq.-K(imu2,2) ) then   ! unlike sign leptons of the same flavour ?
            iflag = 1
         else
            imu1 = -1 
            imu2 = -1
         endif
      endif
      
c discard all particles but the drell-yan leptons
      do  j = 1, N
        if (K(j,1).ne.0.and.K(j,1).lt.21) then
          if (j.ne.imu1.and.j.ne.imu2) then
            im = j
 2          continue
            im = K(im,3)
            if (im.eq.imu1.or.im.eq.imu2) goto 3
            if (im.ne.0)                  goto 2
            K(j,1)=0
 3          continue
          endif
        endif
      enddo

c     remove lines with status flag =0. Update event history information.
      call luedit(12)      

c     remove comment lines. Update event history information.
      call luedit(15)      

 900  END

************************************************************************
* End of ALDY
************************************************************************


