/*-- Author :    Serguei Moukhine   26/04/98 */

#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include <arte/mecCount.h>
#include "arte/Arte.h"
#include "arte/ArteFlag.h"
#include "auto/ERRC.h"

extern void mmbook_(int *adr, int *var);
extern void chttoh_ (char *t, int *h, int *n);
extern void atstopcc_ (char *t);
extern void qnexte_ ();
extern void meallpc_ (int *buf);

int mecErrorCount   (int *id, char *text)
/************************************************************************
*
* Description: error message; count for final statistics
*
* Called from: anywhere (from C programs)
*
* External references: mecMessageCount
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does
*                  not matter: important is its address.
*  TEXT    in      message text
*
************************************************************************/
{
mecMessageCount(id,text,MEC_INFINITE_COUNT,1,0);
return(0);
}

int mecWarningCount (int *id, char *text)
/************************************************************************
*
* Description: warning; count number of occurencies
*
* Called from: anywhere
*
* External references: mecMessageCount
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does not
*                  matter: important is its address.
*  TEXT    in      message text
*
************************************************************************/
{
mecMessageCount(id,text,MEC_WARNING_COUNT,MEC_INFINITE_COUNT,0);
return(0);
}

int mecNextCount    (int *id, char *text)
/************************************************************************
*
* Description: warning; count number of occurencies; skip current event
*
* Called from: anywhere
*
* External references: mecMessageCount
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does not
*                  matter: important is its address.
*  TEXT    in      message text
*
************************************************************************/
{
mecMessageCount(id,text,MEC_WARNING_COUNT,MEC_INFINITE_COUNT,1);
return(0);
}

int mecMessageCount (int *id, char *text, int np, int nf, int lnext)
/************************************************************************
*
* Description: list, count, handle an error
*
* Called from: anywhere
*
* External references: mecAllPrint, mmbook_, atstopcc_, qnexte_
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does
*                  not matter: important is its address.
*  NP              print the text NP times.
*  NF              fatal error if it occurs more than NF times.
*  LNEXT           if 1 then skip current event.
*  TEXT    in      message text
*
************************************************************************/
{
int  k;
int  found;
int  var;
char tmpText[1000];
char what;

/*
** Negative NP and NF mean INFINITE
*/

if (np < 0)
   np = MEC_INFINITE_COUNT;

if (nf < 0)
   nf = MEC_INFINITE_COUNT;

/*
** Search of ID. Speed is not important because this function is called
** in case of errors only.
*/
found = 0;
for (k=1; k<=NERRC(); k++)
   {
   if (ERRC(k).id == (int)id)
      {
      found = 1;
      break;
      }
   }

/*
** Add new row
*/
if (!found)
   {
   var = 1;
   mmbook_(&artelink_.jerrc, &var);
   k = NERRC();
   ERRC(k).id  = (int)id;
   ERRC(k).cnt = 0;
   
   var = sizeof(ERRC(k).text)/sizeof(int);
   chttoh_ (text, (int *)(ERRC(k).text), &var);
   }
   
/*
** Count error
*/
ERRC(k).cnt  = ERRC(k).cnt + 1;
ERRC(k).prnt = np;
ERRC(k).ftl  = nf;

/*
** Fatal error
*/
if (ERRC(k).cnt >= nf)
   {
   arteflag_.progerr++;
   
   strcpy(tmpText,
   "*-----------------------------------------------------------------------------");
   mecAllPrint(tmpText);
   
   mecAllPrint("*");
   mecAllPrint("* Severe error condition");
   mecAllPrint("*");
   
   if (arteflag_.ibatch != 0)
      {
      atstopcc_(text);
      }
   
   sprintf(tmpText, "* === %s",text);
   mecAllPrint(tmpText);
   
   do
      {
      printf(" * Terminate program ? (<CR>=n) ");
      fgets(tmpText, 100, stdin);
      what = tmpText[0];
      if (what == '\n')
         what ='N';
      what = toupper ( what );
      }
   while ( what != 'Y' && what != 'N' );
   
   if (what == 'Y')
      {
      char *mess;
      
      mess = "Stop due to severe error condition";
      atstopcc_(mess);
      }
   
   mecAllPrint("* Continue on your own risk ...");
   mecAllPrint("*");
   strcpy(tmpText,
   "*-----------------------------------------------------------------------------");
   mecAllPrint(tmpText);

   arteflag_.neventl = 0;
   }

/*
** Warning
*/
if (ERRC(k).cnt <= np)
   {
   sprintf(tmpText, "=== %s",text);
   mecAllPrint(tmpText);

   if (lnext)
      mecAllPrint("=== ... Skip to next event!");

   if (ERRC(k).cnt == np)
      mecAllPrint("=== ... last message of this kind");
   }

/*
** Skip to next event
*/
if (lnext)
   {
   if (arteflag_.progstat == 0) /* Event processing */
      qnexte_();
   else
      {
      if (arteflag_.ibatch != 0) /* Not interactive */
         exit(0);
      }
   }


return(0);
}

int mecAllPrint (char *text)
/************************************************************************
*
* Description: print message
*
* Called from: mecMessageCount
*
* External references: meallpc_
*
* Argument in/out  description
*  TEXT    in      message text
*
************************************************************************/
{
int  var;
int  buf[20];

var = 20;
chttoh_ (text, buf, &var);
meallpc_(buf);
return(0);
}

