//                              -*- Mode: C++ -*-
//: ESEMSPos.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
//
// 20.02.01 cog on reduced cluster stored
//
#include "care/ESEMSPos.hh"
#include "arte/ArteMessage.hh"
#include "care/EMHitCell.hh"

//////////////////// ESEMSPOS Class Definitions ////////////////

ESEMSPos::ESEMSPos() : nearCells() {}
ESEMSPos::~ESEMSPos(){}

//--
//  Start a cluster analysis
void ESEMSPos::evaluatePos(const EMCluster* clus){

  peakSt->evaluatePos(clus);    // get the peak
  posData = peakSt->getData(); // the peak position is now local
 
  // define the 3x3 square where to evaluate the cluster position
  nearCells.of(posData.cell()->cellIdx(), ELNearCells::square);
 
  // get first the center of gravity
  ESCOGPos::evaluatePos(clus);
  xcog = posData.x();  
  ycog = posData.y();  
  // Now the correction can be evaluated
  ESCorCOGPos::correct();
}

//--
// HitCell Analyzer
void ESEMSPos::doAnalysis(const EMHitCell * hitT){
  // analyze only cells which are inside the confidence region
  if( hitT==peakSt->getData().cell() || nearCells.isNear(hitT->cellIdx()) ){
      ESCOGPos::doAnalysis(hitT);
  }
}




