using namespace std;

#include <arte/arte.hh>
#include <vector>
#include <map>
#include <algorithm>
#include <fstream.h>
#include "cats/ItrSPoint.hh"
#include "cats/hbook.h"
#include "arte/RSEG_cmp.hh"


extern "C"{
  float prob_(const float & ccchi2, const int & ndeg);
}
void ItrSmoothSP(pSPacepoint psp);


static const double Bmagnet =   0.65945;
static const double zmagnet = 450.0;

static const float      tan_max = 0.5; 
static const float f_x_dist_max = 3.0;
static const float f_y_dist_max = 3.0;

static const float     pinv_cut = 1.0;//0.2;

bool ItrHitbCompareX(const Hit a, const Hit b){
  return (a.x < b.x);
}

bool ItrTripletCompareDist(const Triplet a, const Triplet b){
  return (a.dist < b.dist);
}

bool ItrStereoCompareY(const Ystereo a, const Ystereo b){
  return (a.y < b.y);
}

void ItrSPoints(vector<Hit> *hits_bank_zf, vector<Hit> *hits_bank_sf, pSPVector vspacepoint, 
		int sl, const float geslzmin, const float geslzmax){

  pHit phit[4], sortp[4];
  
  Triplet tripltmp;
  vector<Triplet> utriplet;
  
  sort(hits_bank_zf->begin(),hits_bank_zf->end(), ItrHitbCompareX);

  for (vector<Hit>::iterator hitbIt1 = hits_bank_zf->begin(); 
       hitbIt1 != hits_bank_zf->end(); ++hitbIt1){
    float x1 = hitbIt1->x;
    float z1 = hitbIt1->z;
    phit[0]  = hitbIt1;

    vector<Hit>::iterator hitbIt1t = hitbIt1; hitbIt1t++;
    for (vector<Hit>::iterator hitbIt2 = hitbIt1t; 
	 hitbIt2 != hits_bank_zf->end(); ++hitbIt2){
      if (hitbIt2->lay == hitbIt1->lay) continue; // the same layer
      float x2 = hitbIt2->x;
      float z2 = hitbIt2->z;
      
      if (fabs((x2-x1)/(z2-z1)) > tan_max) break;
      phit[1]  = hitbIt2; 

      vector<Hit>::iterator hitbIt2t = hitbIt2; hitbIt2t++;
      for (vector<Hit>::iterator hitbIt3 = hitbIt2t; 
	   hitbIt3 != hits_bank_zf->end(); ++hitbIt3){
	if ((hitbIt3->lay == hitbIt1->lay) ||
	    (hitbIt3->lay == hitbIt2->lay)) continue;
	float x3 = hitbIt3->x;
	float z3 = hitbIt3->z; 
	
	if (fabs((x3-x2)/(z3-z2)) > tan_max) break;
	phit[2]  = hitbIt3;

	vector<Hit>::iterator hitbIt3t = hitbIt3; hitbIt3t++;
	for (vector<Hit>::iterator hitbIt4 = hitbIt3t; 
	     hitbIt4 != hits_bank_zf->end(); ++hitbIt4){
	  if ((hitbIt4->lay == hitbIt1->lay) ||
	      (hitbIt4->lay == hitbIt2->lay) || 
	      (hitbIt4->lay == hitbIt3->lay)) continue;
	  float x4 = hitbIt4->x;
	  float z4 = hitbIt4->z; 
	  
	  if (fabs((x4-x3)/(z4-z3)) > tan_max) break;
	  phit[3] = hitbIt4;

	  if (sl != 1){
	    for (int i = 0; i < 4; i++){
	      if (phit[i]->gede->lay == 1) sortp[0] = phit[i];
	      if (phit[i]->gede->lay == 2) sortp[1] = phit[i];
	      if (phit[i]->gede->lay == 5) sortp[2] = phit[i];
	      if (phit[i]->gede->lay == 8) sortp[3] = phit[i];
	    }
	  }
	  else{
	    ArtePointer<GEDE> Pgede; 
	    for(int i = 0; i < 4; i++){
	      Pgede = phit[i]->gede;
	      if ((Pgede->lay == 1) && (Pgede->gesl->nsl == 6)) sortp[0] = phit[i];
	      if ((Pgede->lay == 4) && (Pgede->gesl->nsl == 6)) sortp[1] = phit[i];
	      if ((Pgede->lay == 1) && (Pgede->gesl->nsl == 7)) sortp[2] = phit[i];
	      if ((Pgede->lay == 4) && (Pgede->gesl->nsl == 7)) sortp[3] = phit[i];
	    }
	  }
	  float tx = (sortp[3]->x - sortp[0]->x)/(sortp[3]->z - sortp[0]->z);
	  float distl = fabs(sortp[1]->x - (sortp[0]->x + tx*(sortp[1]->z - sortp[0]->z)));
	  float distr = fabs(sortp[2]->x - (sortp[0]->x + tx*(sortp[2]->z - sortp[0]->z)));
	  if ((distl/(sortp[1]->hitb)->r < f_x_dist_max) && 
	      (distr/(sortp[2]->hitb)->r < f_x_dist_max) ){ // store spacepoint
	    tripltmp.z  = geslzmin;    
	    tripltmp.dist = 0.5*(distl+distr);

	    float sumX,sumZ,Xmean,Zmean;
	    
	    sumX=0.0;sumZ=0.0;
	    for(int i=0;i<4;i++)
	      {
		sumX+=sortp[i]->x;
		sumZ+=sortp[i]->z- tripltmp.z;
	      }
	    Xmean=0.0;Zmean=0.0;
	    for(int i=0;i<4;i++)
	      {
		Xmean+=(sortp[i]->z-tripltmp.z)*(sortp[i]->x-0.25*sumX);
		Zmean+=(sortp[i]->z-tripltmp.z)*(sortp[i]->z-tripltmp.z-0.25*sumZ);
	      }
	    tripltmp.tx=Xmean/Zmean;
	    tripltmp.x =0.25*(sumX-tripltmp.tx*sumZ);

	    for(int i = 0; i < 4; i++){
	      tripltmp.Ptohit[i] = sortp[i];
	      //sortp[i].used = 1;
	    }
	    tripltmp.length = 4;
	    hitbIt1->used=1; hitbIt2->used=1; hitbIt3->used=1; hitbIt4->used=1;
	    utriplet.push_back(tripltmp);
	  }
	}
      }
    }
  }
  for (vector<Hit>::iterator hitbIt1 = hits_bank_zf->begin(); 
       hitbIt1 != hits_bank_zf->end(); ++hitbIt1){
    if (hitbIt1->used) continue;
    float x1 = hitbIt1->x;
    float z1 = hitbIt1->z;
    phit[0]  = hitbIt1;

    vector<Hit>::iterator hitbIt1t=hitbIt1; hitbIt1t++;
    for (vector<Hit>::iterator hitbIt2 = hitbIt1t; 
	 hitbIt2 != hits_bank_zf->end(); ++hitbIt2){
      if (hitbIt2->used) continue;
      if (hitbIt2->lay == hitbIt1->lay) continue; // the same layer
      float x2 = hitbIt2->x;
      float z2 = hitbIt2->z;
      
      if (fabs((x2-x1)/(z2-z1)) > tan_max) break;
      phit[1]  = hitbIt2;
      
      vector<Hit>::iterator hitbIt2t=hitbIt2; hitbIt2t++;
      for (vector<Hit>::iterator hitbIt3 = hitbIt2t; 
	   hitbIt3 != hits_bank_zf->end(); ++hitbIt3){
	if (hitbIt3->used) continue;
	if ((hitbIt3->lay == hitbIt1->lay) ||
	    (hitbIt3->lay == hitbIt2->lay)) continue;
	float x3 = hitbIt3->x;
	float z3 = hitbIt3->z; 
	
	if (fabs((x3-x2)/(z3-z2)) > tan_max) break;
	phit[2]  = hitbIt3;
	
	for(int i = 0; i < 4; i++) 
	  sortp[i] = 0;
	
	int first = -1, last = -1, middle = -1;

	if (sl != 1){
	  for (int i = 0; i < 3; i++){
	    if (phit[i]->gede->lay == 1) {sortp[0] = phit[i]; first = 0;}
	    if (phit[i]->gede->lay == 2) {sortp[1] = phit[i]; if (first !=  0) first = 1;} 
	    if (phit[i]->gede->lay == 5) {sortp[2] = phit[i]; if (last  == -1) last  = 2;}
	    if (phit[i]->gede->lay == 8) {sortp[3] = phit[i]; last  = 3;}
	  }
	}
	else{
	  ArtePointer<GEDE> Pgede; 
	  for (int i = 0; i < 3; i++){
	    Pgede=phit[i]->gede;
	    if ((Pgede->lay == 1) && (Pgede->gesl->nsl == 6)) {sortp[0] = phit[i]; first = 0;}
	    if ((Pgede->lay == 4) && (Pgede->gesl->nsl == 6)) {sortp[1] = phit[i]; if (first !=  0) first = 1;} 
	    if ((Pgede->lay == 1) && (Pgede->gesl->nsl == 7)) {sortp[2] = phit[i]; if (last  == -1) last  = 2;}
	    if ((Pgede->lay == 4) && (Pgede->gesl->nsl == 7)) {sortp[3] = phit[i]; last  = 3;}
	  }
	}
	if ((first == 0) && (sortp[1] != 0)) middle = 1;
	if ((first == 0) && (sortp[1] == 0) && (sortp[2] != 0) && (last == 3)) middle = 2;
	if (first == 1) middle = 2;
	
	float tx = (sortp[last]->x - sortp[first]->x)/(sortp[last]->z - sortp[first]->z);
	float distl = fabs(sortp[middle]->x - (sortp[first]->x + tx*(sortp[middle]->z - sortp[first]->z)));
	if (distl/(sortp[middle]->hitb)->r < f_x_dist_max){ // store sacepoint
	  tripltmp.z = geslzmin;

	  float dzml, dzrm, dzrl;
	  dzml = sortp[middle]->z - sortp[first]->z; dzrm = sortp[last]->z - sortp[middle]->z; 
	  dzrl = sortp[last]->z - sortp[first]->z;
	  tripltmp.tx = ((sortp[middle]->x - sortp[first]->x)*dzml  + 
			 (sortp[last]->x   - sortp[middle]->x)*dzrm +
			 (sortp[last]->x   - sortp[first]->x)*dzrl)/(dzml*dzml + dzrm*dzrm + dzrl*dzrl);
	  tripltmp.x = (sortp[first]->x + sortp[middle]->x + sortp[last]->x - 
		       tripltmp.tx*(sortp[first]->z + sortp[middle]->z + sortp[last]->z - 3.0*tripltmp.z))/3.0;

	  tripltmp.dist = 100.0+distl;

	  int j=0;
	  for (int i=0; i<4; i++){
	    if (sortp[i] != 0){
	      tripltmp.Ptohit[j] = sortp[i];  j++;
	      //sortp[i].used = 1;
	    }
	  }
	  hitbIt1->used=1; hitbIt2->used=1; hitbIt3->used=1;
	  tripltmp.Ptohit[3]=0;

	  tripltmp.length = 3;
	  utriplet.push_back(tripltmp);
	} 
      }
    }
  }  // end of loop collecting triplet
  //#ifdef XXX
  for (vector<Hit>::iterator hitbIt1 = hits_bank_zf->begin(); 
       hitbIt1 != hits_bank_zf->end(); ++hitbIt1){
    if (hitbIt1->used) continue;
    float x1 = hitbIt1->x;
    float z1 = hitbIt1->z; 
    phit[0]  = hitbIt1;

    vector<Hit>::iterator hitbIt1t=hitbIt1; hitbIt1t++;
    for (vector<Hit>::iterator hitbIt2 = hitbIt1t; 
	 hitbIt2 != hits_bank_zf->end(); ++hitbIt2){
      if (hitbIt2->used) continue;
      if (hitbIt2->lay == hitbIt1->lay) continue; // the same layer
      float x2 = hitbIt2->x;
      float z2 = hitbIt2->z;
      
      if (fabs((x2-x1)/(z2-z1)) > tan_max) break;
      phit[1]  = hitbIt2;
      
	
      sortp[0] = phit[0];
      sortp[1] = phit[1];
      
      int first = 0, last = 1;
      
      if (sortp[1]->z < sortp[0]->z){
	first = 1; last = 0;
      }
      
      float tx = (sortp[last]->x - sortp[first]->x)/(sortp[last]->z - sortp[first]->z);

      tripltmp.z = geslzmin;
	
      tripltmp.tx = tx;
      tripltmp.x  = sortp[first]->x + tx*(tripltmp.z-sortp[first]->z);
	
      tripltmp.dist = 200.0;
      tripltmp.Ptohit[0] = sortp[first];
      tripltmp.Ptohit[1] = sortp[last];
      tripltmp.Ptohit[2] = 0;
      tripltmp.Ptohit[3] = 0;
      hitbIt1->used=1; hitbIt2->used=1;
     
      tripltmp.length = 2;
      utriplet.push_back(tripltmp);
    }
  }  // end of loop collecting triplet
  //#endif
  sort(utriplet.begin(), utriplet.end(), ItrTripletCompareDist);


  vector<Ystereo> ystereo;
  SPacepoint sp;
  Ystereo stereotmp;

  for (vector<Triplet>::iterator triplIt = utriplet.begin(); 
       triplIt != utriplet.end(); ++triplIt){

    ystereo.clear();
    for (vector<Hit>::iterator hitbstIt = hits_bank_sf->begin(); 
	 hitbstIt != hits_bank_sf->end(); ++hitbstIt){
      //if (hitbstIt->used) continue;  
      int senarea = calc_y(triplIt->x, triplIt->tx, triplIt->z, hitbstIt); 
      if (senarea){
	float x = triplIt->x + triplIt->tx*(hitbstIt->z-triplIt->z);
	stereotmp.y  = (x*hitbstIt->cosa-hitbstIt->x )/hitbstIt->sina;
	stereotmp.ty = stereotmp.y/hitbstIt->z;
	stereotmp.z  = hitbstIt->z;
	stereotmp.used   = 0;
	stereotmp.Ptohit = hitbstIt;
	ystereo.push_back(stereotmp);
      }
    }

    if (distance(ystereo.begin(), ystereo.end()) == 0) continue;
    sort(ystereo.begin(), ystereo.end(), ItrStereoCompareY);

    int amountof_quadro = 0;
    for (vector<Ystereo>::iterator hitp1It = ystereo.begin();
	 hitp1It != ystereo.end(); ++hitp1It){ 
      pHit hitbP1 = hitp1It->Ptohit;
      float x1   = hitbP1->x;
      float z1   = hitbP1->z;
      phit[0]    = hitbP1;

      vector<Ystereo>::iterator phit2_begin = hitp1It; phit2_begin++; 
      for (vector<Ystereo>::iterator hitp2It = phit2_begin;
	   hitp2It != ystereo.end(); ++hitp2It){
	pHit hitbP2 = hitp2It->Ptohit;
	if (hitbP2->lay == hitbP1->lay) continue; // the same layer
	if (fabs((hitp2It->y - hitp1It->y)/(hitp2It->z - hitp1It->z)) > tan_max) continue;//???
	 
	 float z2 = hitbP2->z; 
	 float u  = hitbP2->x;
	 float x  = triplIt->x + triplIt->tx*(z2-triplIt->z);
	 float y  = (x*hitbP2->cosa - u)/hitbP2->sina;
	 float x2 = x*hitbP1->cosa - y*hitbP1->sina; // calculate to the same degree
	 
	 if (fabs((x2-x1)/(z2-z1)) > tan_max) break;

	 phit[1]  = hitbP2; 

	 vector<Ystereo>::iterator phit3_begin = hitp2It; phit3_begin++;
	 for (vector<Ystereo>::iterator hitp3It = phit3_begin;
	      hitp3It != ystereo.end(); ++hitp3It){
	   pHit hitbP3 = hitp3It->Ptohit;
	   if ((hitbP3->lay == hitbP1->lay) || 
	       (hitbP3->lay == hitbP2->lay)) continue; 
	   if (fabs((hitp3It->y - hitp2It->y)/(hitp3It->z - hitp2It->z)) > tan_max) continue;

	   float z3 = hitbP3->z; 
	   float u  = hitbP3->x;
	   float x  = triplIt->x + triplIt->tx*(z3-triplIt->z);
	   float y  = (x*hitbP3->cosa - u)/hitbP3->sina;
	   float x3 = x*hitbP1->cosa - y*hitbP1->sina;  
	   
	   if (fabs((x3-x2)/(z3-z2)) > tan_max) break;

	   phit[2]  = hitbP3;  

	   vector<Ystereo>::iterator phit4_begin = hitp3It; phit4_begin++;
	   for (vector<Ystereo>::iterator hitp4It = phit4_begin;
		hitp4It != ystereo.end(); ++hitp4It){
	     pHit hitbP4 = hitp4It->Ptohit;
	     if ((hitbP4->lay == hitbP1->lay) || 
		 (hitbP4->lay == hitbP2->lay) || 
		 (hitbP4->lay == hitbP3->lay)) continue; 
	     
	     if (fabs((hitp4It->y - hitp3It->y)/(hitp4It->z - hitp3It->z)) > tan_max) continue;
	     
	     float z4 = hitbP4->z; 
	     float u  = hitbP4->x;
	     float x  = triplIt->x + triplIt->tx*(z4-triplIt->z);
	     float y  = (x*hitbP4->cosa - u)/hitbP4->sina;
	     float x4 = x*hitbP1->cosa - y*hitbP1->sina;  
	     
	     if (fabs((x4-x3)/(z4-z3)) > tan_max) break;

	     phit[3]  = hitbP4;

	     if (sl != 1){
	       for (int i=0; i<4; i++){
		 if(phit[i]->gede->lay == 3) sortp[0] = phit[i];
		 if(phit[i]->gede->lay == 4) sortp[1] = phit[i];
		 if(phit[i]->gede->lay == 6) sortp[2] = phit[i];
		 if(phit[i]->gede->lay == 7) sortp[3] = phit[i];
	       }
	     }
	     else{
	       ArtePointer<GEDE> Pgede; 
	       for (int i=0; i<4; i++){
		 Pgede = phit[i]->gede;
		 if ((Pgede->lay == 2) && (Pgede->gesl->nsl == 6)) sortp[0] = phit[i];
		 if ((Pgede->lay == 3) && (Pgede->gesl->nsl == 6)) sortp[1] = phit[i];
		 if ((Pgede->lay == 2) && (Pgede->gesl->nsl == 7)) sortp[2] = phit[i];
		 if ((Pgede->lay == 3) && (Pgede->gesl->nsl == 7)) sortp[3] = phit[i];
	       }
	     }

	     float zl  = sortp[0]->z; 
	     float xl  = sortp[0]->x;
	     u         = sortp[0]->x;
	     x         = triplIt->x + triplIt->tx*(zl-triplIt->z);
	     float yl  = (x*sortp[0]->cosa - u)/sortp[0]->sina;

	     float zm  = sortp[1]->z; 
	     u         = sortp[1]->x;
	     x         = triplIt->x + triplIt->tx*(zm-triplIt->z);
	     float ym  = (x*sortp[1]->cosa - u)/sortp[1]->sina;
	     float xm  = x*sortp[0]->cosa - ym*sortp[0]->sina;  

	     float zr  = sortp[2]->z; 
	     u         = sortp[2]->x;
	     x         = triplIt->x + triplIt->tx*(zr-triplIt->z);
	     float yr  = (x*sortp[2]->cosa - u)/sortp[2]->sina;
	     float xr  = x*sortp[0]->cosa - yr*sortp[0]->sina; 

	     float zrr = sortp[3]->z; 
	     u         = sortp[3]->x;
	     x         = triplIt->x + triplIt->tx*(zrr-triplIt->z);
	     float yrr = (x*sortp[3]->cosa - u)/sortp[3]->sina;
	     float xrr = x*sortp[0]->cosa - yrr*sortp[0]->sina;

	     float tx     = triplIt->tx;
	     //tx = (xrr-xl)/(zrr-zl);
	     float distm  = fabs(xm  - (xl + tx*(zm  - zl)));
	     float distr  = fabs(xr  - (xl + tx*(zr  - zl)));
	     float distrr = fabs(xrr - (xl + tx*(zrr - zl)));

	     if ((distm /(sortp[1]->hitb)->r < f_y_dist_max) && 
		 (distr /(sortp[2]->hitb)->r < f_y_dist_max) && 
		 (distrr/(sortp[3]->hitb)->r < f_y_dist_max)){ //store SP

	       sp.zf  = triplIt->z;
	       sp.xf  = triplIt->x;    sp.txf = triplIt->tx;
	       sp.tyf = (yrr - yl)/(zrr - zl);
	       sp.yf  = yl + sp.tyf*(sp.zf - zl);

	       sp.ze  = geslzmax;
	       sp.xe  = sp.xf + sp.txf*(sp.ze - sp.zf);           
	       sp.txe = sp.txf;
	       sp.ye  = sp.yf + sp.tyf*(sp.ze - sp.zf);         sp.tye = sp.tyf; 
	       sp.used  = 0;  sp.level = 0;    sp.neighborindex = -1;

	       float pinv=1.0/(Bmagnet*zmagnet*
			       sqrt(1.0+sp.txf*sp.txf+sp.tyf*sp.tyf)/
			       ((sp.xf-sp.zf*sp.txf)*
				sqrt(1.0+sp.txf*sp.txf)));
	       pinv = fabs(pinv);
	       if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	       sp.pf=sp.pe=pinv;

	       if (pinv<pinv_cut){
		 sp.vhit.clear();
		 
		 for (int i=0; i<triplIt->length; i++)
		   if (triplIt->Ptohit[i] != 0)
		     sp.vhit.push_back((triplIt->Ptohit[i]));
		 
		 sp.vhit.push_back((hitp1It->Ptohit));
		 sp.vhit.push_back((hitp2It->Ptohit));
		 sp.vhit.push_back((hitp3It->Ptohit));
		 sp.vhit.push_back((hitp4It->Ptohit));
		 
		 ItrFitSPoint(sp,1);
		 
		 if (sp.prob > 0.05){
		   hitp1It->used=1; hitp2It->used=1; hitp3It->used=1; hitp4It->used=1; 
		   amountof_quadro++;
		   vspacepoint->push_back(sp);
		   //SP_to_rseg(sp);
		 }
	       }
	     }// dist
	   }//4
	 }
      } 
    }//quadro 
    
    if (amountof_quadro == 0){//amount of quadro
      for (vector<Ystereo>::iterator hitp1It = ystereo.begin();
	   hitp1It != ystereo.end(); ++hitp1It){
	if (hitp1It->used) continue; 
	pHit hitbP1 = hitp1It->Ptohit;
	float x1   = hitbP1->x;
	float z1   = hitbP1->z;
	phit[0]    = hitbP1;  
       	vector<Ystereo>::iterator phit2_begin = hitp1It; phit2_begin++; 
	for (vector<Ystereo>::iterator hitp2It = phit2_begin;
	     hitp2It != ystereo.end(); ++hitp2It){
	  if (hitp2It->used) continue;
	  pHit hitbP2 = hitp2It->Ptohit;
	  if (hitbP2->lay == hitbP1->lay) continue; // the same layer
	  if (fabs((hitp2It->y - hitp1It->y)/(hitp2It->z - hitp1It->z)) > tan_max) continue;
	  
	  float z2 = hitbP2->z; 
	  float u  = hitbP2->x;
	  float x  = triplIt->x + triplIt->tx*(z2-triplIt->z);
	  float y  = (x*hitbP2->cosa - u)/hitbP2->sina;
	  float x2 = x*hitbP1->cosa - y*hitbP1->sina; // calculate to the same degree
	  
	  if (fabs((x2-x1)/(z2-z1)) > tan_max) break;

	  phit[1]  = hitbP2; 
	  vector<Ystereo>::iterator phit3_begin = hitp2It; phit3_begin++;
	  for (vector<Ystereo>::iterator hitp3It = phit3_begin;
	       hitp3It != ystereo.end(); ++hitp3It){
	    if (hitp3It->used) continue;
	    pHit hitbP3 = hitp3It->Ptohit;
	    if ((hitbP3->lay == hitbP1->lay) || 
		(hitbP3->lay == hitbP2->lay)) continue; 
	    if (fabs((hitp3It->y - hitp2It->y)/(hitp3It->z - hitp2It->z)) > tan_max) continue;
	    
	    float z3 = hitbP3->z; 
	    float u  = hitbP3->x;
	    float x  = triplIt->x + triplIt->tx*(z3-triplIt->z);
	    float y  = (x*hitbP3->cosa - u)/hitbP3->sina;
	    float x3 = x*hitbP1->cosa - y*hitbP1->sina;  
	    
	    if (fabs((x3-x2)/(z3-z2)) > tan_max) break;
	       
	    phit[2]  = hitbP3;
	    
	    int first = -1, last = -1, middle = -1;
	    for (int i=0; i<4; i++) 
	      sortp[i]=0;

	    if (sl != 1){
	      for (int i=0; i<3; i++){
		if (phit[i]->gede->lay == 3) 
		  {
		    sortp[0] = phit[i]; 
		    first = 0;
		  }
		if (phit[i]->gede->lay == 4) { sortp[1] = phit[i]; if (first != 0) first = 1;} 
		if (phit[i]->gede->lay == 6) { sortp[2] = phit[i]; if (last  == -1) last  = 2;}
		if (phit[i]->gede->lay == 7) { sortp[3] = phit[i]; last  = 3;}
	      }
	    }
	    else{
	      ArtePointer<GEDE> Pgede; 
	      for (int i=0; i<3; i++){
		Pgede = phit[i]->gede;
		if ((Pgede->lay == 2) && (Pgede->gesl->nsl == 6)) {sortp[0] = phit[i]; first = 0;}
		if ((Pgede->lay == 3) && (Pgede->gesl->nsl == 6)) {sortp[1] = phit[i]; if (first !=  0) first = 1;} 
		if ((Pgede->lay == 2) && (Pgede->gesl->nsl == 7)) {sortp[2] = phit[i]; if (last  == -1) last  = 2;}
		if ((Pgede->lay == 3) && (Pgede->gesl->nsl == 7)) {sortp[3] = phit[i]; last  = 3;}
	      }
	    }
	    if ((first == 0) && (sortp[1] != 0)) middle = 1;
	    if ((first == 0) && (sortp[1] == 0) && (sortp[2] != 0) && (last == 3)) middle = 2;
	    if ( first == 1) middle = 2;
	    float zl  = sortp[first]->z; 
	    float xl  = sortp[first]->x;
	    u         = sortp[first]->x;
	    x         = triplIt->x + triplIt->tx*(zl-triplIt->z);
	    float yl  = (x*sortp[first]->cosa - u)/sortp[first]->sina;
	    float zm  = sortp[middle]->z; 
	    u         = sortp[middle]->x;
	    x         = triplIt->x + triplIt->tx*(zm-triplIt->z);
	    float ym  = (x*sortp[middle]->cosa - u)/sortp[middle]->sina;
	    float xm  = x*sortp[first]->cosa - ym*sortp[first]->sina;  
	    float zr  = sortp[last]->z; 
	    u         = sortp[last]->x;
	    x         = triplIt->x + triplIt->tx*(zr-triplIt->z);
	    float yr  = (x*sortp[last]->cosa - u)/sortp[last]->sina;
	    float xr  = x*sortp[first]->cosa - yr*sortp[first]->sina; // rec. to stereo layer of the first cluster 
	    float tx     = triplIt->tx;
	    float distm  = fabs(xm  - (xl + tx*(zm - zl)) );
	    float distr  = fabs(xr  - (xl + tx*(zr - zl)) );
	    if((distm/(sortp[middle]->hitb)->r < f_y_dist_max) && 
	       (distr/(sortp[last]->hitb)->r   < f_y_dist_max)){ //store SP
	      sp.zf  = triplIt->z;
	      sp.xf  = triplIt->x;    sp.txf = triplIt->tx;
	      sp.tyf = (yr - yl)/(zr - zl);
	      sp.yf  = yl + sp.tyf*(sp.zf - zl); 

	      sp.ze  = geslzmax;
	      sp.txe = sp.txf;  sp.tye = sp.tyf;
	      sp.xe  = sp.xf + sp.txf*(sp.ze-sp.zf);
	      sp.ye  = sp.yf + sp.tyf*(sp.ze-sp.zf);         
	      sp.used = 0; sp.level = 0; sp.neighborindex = -1;

	      float pinv=1.0/(Bmagnet*zmagnet*
			      sqrt(1.0+sp.txf*sp.txf+sp.tyf*sp.tyf)/
			      ((sp.xf-sp.zf*sp.txf)*
			       sqrt(1.0+sp.txf*sp.txf)));
	      pinv = fabs(pinv);
	      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	      sp.pf=sp.pe=pinv;

	      if (pinv<pinv_cut){
		sp.vhit.clear();
		
		for (int i=0; i<triplIt->length; i++)
		  if (triplIt->Ptohit[i] != 0)
		    sp.vhit.push_back((triplIt->Ptohit[i]));
		
		sp.vhit.push_back((hitp1It->Ptohit));
		sp.vhit.push_back((hitp2It->Ptohit));
		sp.vhit.push_back((hitp3It->Ptohit));
		ItrFitSPoint(sp,1);
		if (sp.prob > 0.05){
		  hitp1It->used=1; hitp2It->used=1; hitp3It->used=1; 
		  amountof_quadro++;
		  vspacepoint->push_back(sp);
		  //SP_to_rseg(sp);
		}
	      }
	    }
	  }
	} 
      }  // tripl beg
    }
    if (amountof_quadro == 0){//amount of quadro
      for (vector<Ystereo>::iterator hitp1It = ystereo.begin();
	   hitp1It != ystereo.end(); ++hitp1It){
	//if (hitp1It->used) continue; 
	pHit hitbP1 = hitp1It->Ptohit;
	float x1   = hitbP1->x;
	float z1   = hitbP1->z;
	phit[0]    = hitbP1;  

       	vector<Ystereo>::iterator phit2_begin = hitp1It; phit2_begin++; 
	for (vector<Ystereo>::iterator hitp2It = phit2_begin;
	     hitp2It != ystereo.end(); ++hitp2It){
	  //if (hitp2It->used) continue;
	  pHit hitbP2 = hitp2It->Ptohit;
	  if (hitbP2->lay == hitbP1->lay) continue; // the same layer
	  if (fabs((hitp2It->y - hitp1It->y)/(hitp2It->z - hitp1It->z)) > tan_max) continue;
	  
	  float z2 = hitbP2->z; 
	  float u  = hitbP2->x;
	  float x  = triplIt->x + triplIt->tx*(z2-triplIt->z);
	  float y  = (x*hitbP2->cosa - u)/hitbP2->sina;
	  float x2 = x*hitbP1->cosa - y*hitbP1->sina; // calculate to the same degree
	  
	  if (fabs((x2-x1)/(z2-z1)) > tan_max) break;

	  phit[1]  = hitbP2; 

	  for (int i=0; i<4; i++) 
	    sortp[i] = 0;

	  sortp[0] = phit[0];
	  sortp[1] = phit[1];
	  
	  int first = 0, last = 1;
	  
	  if (sortp[1]->z < sortp[0]->z){
	    first = 1; last = 0;
	  }
	  
	  float zl  = sortp[first]->z; 
	  //float xl  = sortp[first]->x;
	  u         = sortp[first]->x;
	  x         = triplIt->x + triplIt->tx*(zl-triplIt->z);
	  float yl  = (x*sortp[first]->cosa - u)/sortp[first]->sina;
	  
	  float zr  = sortp[last]->z; 
	  u         = sortp[last]->x;
	  x         = triplIt->x + triplIt->tx*(zr-triplIt->z);
	  float yr  = (x*sortp[last]->cosa - u)/sortp[last]->sina;
	  // float xr  = x*sortp[first]->cosa - yr*sortp[first]->sina; // rec. to stereo layer of the first cluster 
	    
	  sp.zf  = triplIt->z;
	  sp.xf  = triplIt->x;    sp.txf = triplIt->tx;
	  sp.tyf = (yr - yl)/(zr - zl);
	  sp.yf  = yl + sp.tyf*(sp.zf - zl);

	  sp.ze  = geslzmax;
	  sp.txe = sp.txf;  sp.tye = sp.tyf;
	  sp.xe  = sp.xf + sp.txf*(sp.ze-sp.zf);
	  sp.ye  = sp.yf + sp.tyf*(sp.ze-sp.zf);         
	  sp.used = 0; sp.level = 0; sp.neighborindex = -1;

	  float pinv=1.0/(Bmagnet*zmagnet*
			  sqrt(1.0+sp.txf*sp.txf+sp.tyf*sp.tyf)/
			  ((sp.xf-sp.zf*sp.txf)*
			   sqrt(1.0+sp.txf*sp.txf)));
	  pinv = fabs(pinv);
	  if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	  sp.pf=sp.pe=pinv;

	  if (pinv<pinv_cut){
	    sp.vhit.clear();
	    
	    for (int i=0; i<triplIt->length; i++)
	      if (triplIt->Ptohit[i] != 0)
		sp.vhit.push_back((triplIt->Ptohit[i]));
	    
	    sp.vhit.push_back((hitp1It->Ptohit));
	    sp.vhit.push_back((hitp2It->Ptohit));
	    
	    ItrFitSPoint(sp,1);
	    
	    if (sp.prob > 0.05){
	      hitp1It->used=1; hitp2It->used=1;
	      amountof_quadro++;
	      vspacepoint->push_back(sp);
	      //SP_to_rseg(sp);
	    }
	  }
	} 
      }  // tripl beg
    }


  } // end of ItrSPoint

}


//---------------------------------------------------------------------------------


int calc_y(double bx,double kx,double dist,vector<Hit>::iterator hitbsearchtg)
{
  double minv,maxv;
  minv=maxv=0.0;
  if(hitbsearchtg->gede->rot[1]<0){
    maxv=(hitbsearchtg->gede->symn)*hitbsearchtg->sina+(hitbsearchtg->x)*hitbsearchtg->cosa;
    minv=(hitbsearchtg->gede->symx)*hitbsearchtg->sina+(hitbsearchtg->x)*hitbsearchtg->cosa;
  }
  if(hitbsearchtg->gede->rot[1]>0){
    minv=(hitbsearchtg->gede->symn)*hitbsearchtg->sina+(hitbsearchtg->x)*hitbsearchtg->cosa;
    maxv=(hitbsearchtg->gede->symx)*hitbsearchtg->sina+(hitbsearchtg->x)*hitbsearchtg->cosa;
  }
  double shifted=kx*((hitbsearchtg->z)-dist)+bx;
  if( shifted<maxv && shifted>minv)
    return 1;
  
  return 0;
}

//-------------------------------------------------------------------------
void ItrFitSPoint(SPacepoint& sp,int propagate_midd){ 
  float Rk[4],Kk[4],Bk[4],Gk[4][4];
  float Z,Z0,sin2,cos2,Dk,Dk2,zeta,chi2=0.0,z00=0.0,sigma00=1.0,sigma2,dchi2,xe=0.0,xf=0.0;
  int nstereo_max;

  nstereo_max=(propagate_midd<2)?2000:0;

  nstereo_max=2000;
  
  float zf = sp.zf;
  float ze = sp.ze;

  Z0 = sp.zf; sp.zf = sp.ze; sp.ze = Z0;
  for (vector<pHit>::iterator cluIt = sp.vhit.begin();
       cluIt != sp.vhit.end(); ++cluIt){
    if ((*cluIt)->z < sp.zf) 
      {
	sp.zf = (*cluIt)->z;
	xf=(*cluIt)->x;
      }

    if ((*cluIt)->z > sp.ze) 
      {
	sp.ze = (*cluIt)->z;
	xe=(*cluIt)->x;
      }
  }
  sp.xf = sp.xf + sp.txf*(sp.zf - zf);
  sp.yf = sp.yf + sp.tyf*(sp.zf - zf);

  Z0 = sp.zf;
  for (int i=0; i<4; i++) 
    for (int j=0; j<=i; j++) 
      Gk[j][i] = 0.0;
  Gk[0][0] = 1.0; Gk[1][1] = 4.0; Gk[2][2] = 0.1; Gk[3][3] = 0.1; 
  // Rk[0] = sp.xf; Rk[1] = sp.yf; Rk[2] = sp.txf; Rk[3] = sp.tyf;  
  // new initialization .......

    Rk[0]=xf;Rk[1]=sp.yf;Rk[2]=(xe-xf)/(sp.ze-sp.zf);Rk[3]=0.0;
  

  int nstereo = 0;
  for (vector<pHit>::iterator cluIt = sp.vhit.begin();
       cluIt != sp.vhit.end(); ++cluIt){
    pHit pclu = (*cluIt);
    if (fabs(pclu->sina)>0.01) nstereo++;

    Dk = pclu->z-Z0; Dk2 = Dk*Dk;

    sigma2=(pclu->hitb)->r*(pclu->hitb)->r;
    sin2 = pclu->sina*pclu->sina; cos2 = pclu->cosa*pclu->cosa;
    Z = (cos2*(Gk[0][0] + Dk2*Gk[2][2] + 2.0*Dk*Gk[0][2]) +
	 sin2*(Gk[1][1] + Dk2*Gk[3][3] + 2.0*Dk*Gk[1][3]) -
	 2.0*pclu->cosa*pclu->sina*(Gk[0][1] + Dk*(Gk[0][3] + Gk[1][2]) + Dk2*Gk[2][3]) + sigma2);
    Z = 1.0/Z;
    Bk[0] = pclu->cosa*(Gk[0][0] + Dk*Gk[0][2]) - pclu->sina*(Gk[0][1] + Dk*Gk[0][3]);
    Bk[1] = pclu->cosa*(Gk[0][1] + Dk*Gk[1][2]) - pclu->sina*(Gk[1][1] + Dk*Gk[1][3]);
    Bk[2] = pclu->cosa*(Gk[0][2] + Dk*Gk[2][2]) - pclu->sina*(Gk[1][2] + Dk*Gk[2][3]);
    Bk[3] = pclu->cosa*(Gk[0][3] + Dk*Gk[2][3]) - pclu->sina*(Gk[1][3] + Dk*Gk[3][3]);
    zeta = pclu->x - Rk[0]*pclu->cosa + Rk[1]*pclu->sina - Rk[2]*Dk*pclu->cosa + Rk[3]*Dk*pclu->sina;
    for(int i=0;i<4;i++) Kk[i]=Bk[i]*Z;
    for (int i=0; i<4; i++){
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }

  if (nstereo <= nstereo_max){
    Dk=z00-Z0;Dk2=Dk*Dk;
    zeta=-Rk[1]-Rk[3]*Dk;
    sigma2=sigma00*sigma00;
    Z=Gk[1][1]+2.0*Dk*Gk[1][3]+Dk2*Gk[3][3]+sigma2;
    Z=1.0/Z;
    Bk[0]=Gk[0][1]+Dk*Gk[0][3]; Bk[1]=Gk[1][1]+Dk*Gk[1][3];
    Bk[2]=Gk[1][2]+Dk*Gk[2][3]; Bk[3]=Gk[1][3]+Dk*Gk[3][3];
    for (int i=0; i<4; i++){
      Kk[i]  = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }

  int nclu = 0;
  for (vector<pHit>::iterator cluIt = sp.vhit.begin();
       cluIt != sp.vhit.end(); ++cluIt){
    pHit pclu = (*cluIt);
    Dk = pclu->z-Z0; Dk2 = Dk*Dk;
   
    sigma2=(pclu->hitb)->r*(pclu->hitb)->r;
    //sigma2=((pclu->hitb)->r/(pclu->mask+0.01))*((pclu->hitb)->r/(pclu->mask+0.01));
    sin2 = pclu->sina*pclu->sina; cos2 = pclu->cosa*pclu->cosa;
    Z = (cos2*(Gk[0][0] + Dk2*Gk[2][2] + 2.0*Dk*Gk[0][2])+
	 sin2*(Gk[1][1] + Dk2*Gk[3][3] + 2.0*Dk*Gk[1][3])-
	 2.0*pclu->cosa*pclu->sina*(Gk[0][1] + Dk*(Gk[0][3] + Gk[1][2]) + Dk2*Gk[2][3]) + sigma2);
    zeta = pclu->x - Rk[0]*pclu->cosa + Rk[1]*pclu->sina - Rk[2]*Dk*pclu->cosa + Rk[3]*Dk*pclu->sina;

    dchi2= zeta*zeta/Z;
    chi2 += zeta*zeta/Z;
    nclu++;
  }

  if (nstereo <= nstereo_max){
    Dk=z00-Z0;Dk2=Dk*Dk;
    zeta=-Rk[1]-Rk[3]*Dk;
    sigma2=sigma00*sigma00;
    Z=Gk[1][1]+2.0*Dk*Gk[1][3]+Dk2*Gk[3][3]+sigma2;
    chi2+=zeta*zeta/Z;
  }

  sp.xf  = Rk[0]; sp.yf  = Rk[1];
  sp.txf = Rk[2]; sp.tyf = Rk[3];
  
  for (int i=0;i <4; i++) 
    for (int j=0; j<=i; j++){
      sp.cvf[j][i] = Gk[j][i];
      Gk[j][i] = 0.0; // clear for next steps
    }

  Z0 = sp.ze;

 
  Gk[0][0] = 1.0; Gk[1][1] = 4.0; Gk[2][2] = 0.1; Gk[3][3] = 0.1;

  //  Rk[0] = sp.xf+sp.txf*(sp.ze-sp.zf); Rk[1] = sp.yf+sp.tyf*(sp.ze-sp.zf); Rk[2] = sp.txf; Rk[3] = sp.tyf;  
  //  Rk[0] = sp.xe+sp.txe*(sp.ze-ze); Rk[1] = sp.ye+sp.tye*(sp.ze-ze); Rk[2] = sp.txe; Rk[3] = sp.tye;  

  // new initialization ......

    Rk[0]=xe;Rk[1]=sp.ye;Rk[2]=(xe-xf)/(sp.ze-sp.zf);Rk[3]=0.0;

  
  for (vector<pHit>::reverse_iterator cluIt = sp.vhit.rbegin();
       cluIt != sp.vhit.rend(); ++cluIt){
    pHit pclu = (*cluIt);
    Dk = pclu->z-Z0; Dk2 = Dk*Dk;

    sigma2=(pclu->hitb)->r*(pclu->hitb)->r;
    sin2 = pclu->sina*pclu->sina; cos2 = pclu->cosa*pclu->cosa;
    Z = (cos2*(Gk[0][0] + Dk2*Gk[2][2] + 2.0*Dk*Gk[0][2]) +
	 sin2*(Gk[1][1] + Dk2*Gk[3][3] + 2.0*Dk*Gk[1][3]) -
	 2.0*pclu->cosa*pclu->sina*(Gk[0][1] + Dk*(Gk[0][3] + Gk[1][2]) + Dk2*Gk[2][3]) + sigma2);
    Z = 1.0/Z;
    Bk[0] = pclu->cosa*(Gk[0][0] + Dk*Gk[0][2]) - pclu->sina*(Gk[0][1] + Dk*Gk[0][3]);
    Bk[1] = pclu->cosa*(Gk[0][1] + Dk*Gk[1][2]) - pclu->sina*(Gk[1][1] + Dk*Gk[1][3]);
    Bk[2] = pclu->cosa*(Gk[0][2] + Dk*Gk[2][2]) - pclu->sina*(Gk[1][2] + Dk*Gk[2][3]);
    Bk[3] = pclu->cosa*(Gk[0][3] + Dk*Gk[2][3]) - pclu->sina*(Gk[1][3] + Dk*Gk[3][3]);
    zeta = pclu->x - Rk[0]*pclu->cosa + Rk[1]*pclu->sina - Rk[2]*Dk*pclu->cosa + Rk[3]*Dk*pclu->sina;
    for (int i=0; i<4; i++){
      Kk[i]  = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }

  if (nstereo <= nstereo_max){
    Dk=z00-Z0;Dk2=Dk*Dk;
    zeta=-Rk[1]-Rk[3]*Dk;
    sigma2=sigma00*sigma00;
    Z=Gk[1][1]+2.0*Dk*Gk[1][3]+Dk2*Gk[3][3]+sigma2;
    Z=1.0/Z;
    Bk[0]=Gk[0][1]+Dk*Gk[0][3]; Bk[1]=Gk[1][1]+Dk*Gk[1][3];
    Bk[2]=Gk[1][2]+Dk*Gk[2][3]; Bk[3]=Gk[1][3]+Dk*Gk[3][3];
    for (int i=0; i<4; i++){
      Kk[i]  = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }

  sp.nclu = nclu;

  sp.xe  = Rk[0]; sp.ye  = Rk[1];
  sp.txe = Rk[2]; sp.tye = Rk[3];
  
  for (int i=0;i <4; i++) 
    for (int j=0; j<=i; j++){
      sp.cve[j][i] = Gk[j][i];
    }
  
  if ((sp.cvf[0][0]< 0.0)||(sp.cvf[1][1]< 0.0)||
      (sp.cvf[2][2]< 0.0)||(sp.cvf[3][3]< 0.0)){
    
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------> ItrCats bad cvf matrix ("<<nclu<<" hits) "
	<<sp.cvf[0][0]<<" "<<sp.cvf[1][1]<<" "<<sp.cvf[2][2]<<" "<<sp.cvf[3][3]<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
    
    sp.chi2 = 1000.0;
    sp.prob = 0.0; // will not be stored
  }
  else{
    if ((sp.cve[0][0]< 0.0)||(sp.cve[1][1]< 0.0)||
	(sp.cve[2][2]< 0.0)||(sp.cve[3][3]< 0.0)){
 
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------> ItrCats bad cve matrix ("<<nclu<<" hits) "
	  <<sp.cve[0][0]<<" "<<sp.cve[1][1]<<" "<<sp.cve[2][2]<<" "<<sp.cve[3][3]<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;

      sp.chi2 = 1000.0;
      sp.prob = 0.0; // will not be stored
    }
    else{
      if (chi2<0.0) chi2 = 0.0;
      if (nclu != 4){
	if (nstereo <= nstereo_max) nclu += 1; //ndof++;
	sp.chi2 = chi2/(nclu - 4.0);
	sp.prob = prob_(fabs(chi2),(nclu - 4));
      }
      else{
	sp.chi2 = 0.0;
	sp.prob = 0.5;
      }
    }
  }

  if (propagate_midd!=1) return;
  
  float dz = 0.5*(zf+ze) - sp.zf;
  sp.xm = sp.xf + sp.txf*dz; sp.txm = sp.txf;
  sp.ym = sp.yf + sp.tyf*dz; sp.tym = sp.tyf;
  sp.zm = 0.5*(zf+ze);
  
  sp.cvm[0][0] = sp.cvf[0][0] + dz * (dz*sp.cvf[2][2] + sp.cvf[0][2] + sp.cvf[0][2]);
  sp.cvm[0][1] = sp.cvf[0][1] + dz * (dz*sp.cvf[2][3] + sp.cvf[0][3] + sp.cvf[1][2]); //sp.cvm[1][0] = sp.cvm[0][1];
  sp.cvm[1][1] = sp.cvf[1][1] + dz * (dz*sp.cvf[3][3] + sp.cvf[1][3] + sp.cvf[1][3]);
  sp.cvm[0][2] = sp.cvf[0][2] + dz * sp.cvf[2][2];                                    //sp.cvm[2][0] = sp.cvm[0][2];  
  sp.cvm[1][2] = sp.cvf[1][2] + dz * sp.cvf[2][3];                                    //sp.cvm[2][1] = sp.cvm[1][2]; 
  sp.cvm[0][3] = sp.cvf[0][3] + dz * sp.cvf[2][3];                                    //sp.cvm[3][0] = sp.cvm[0][3];
  sp.cvm[1][3] = sp.cvf[1][3] + dz * sp.cvf[3][3];                                    //sp.cvm[3][1] = sp.cvm[1][3];
  sp.cvm[2][2] = sp.cvf[2][2];
  sp.cvm[3][3] = sp.cvf[3][3];
  sp.cvm[2][3] = sp.cvf[2][3];                                                        //sp.cvm[3][2] = sp.cvm[2][3];


  if ((sp.cvm[0][0]< 0.0)||(sp.cvm[1][1]< 0.0)||
      (sp.cvm[2][2]< 0.0)||(sp.cvm[3][3]< 0.0)){

    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------> ItrCats bad cvm matrix ("<<nclu<<" hits) "
	<<sp.cvm[0][0]<<" "<<sp.cvm[1][1]<<" "<<sp.cvm[2][2]<<" "<<sp.cvm[3][3]<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;

    sp.chi2 = 1000.0;
    sp.prob = 0.0; // will not be stored
  }


  return;
}
//----------------------------------------------------------------------------

void ItrSmoothSP(pSPacepoint psp){
  
  float ax = (psp->xe-psp->xf)/(psp->ze-psp->zf);
  float bx = psp->xf - ax*psp->zf;
  float errfx = 3.0*0.0200;
  float errex = 3.0*0.0200;
  
  float ay = (psp->ye-psp->yf)/(psp->ze-psp->zf);
  float by = psp->yf - ay*psp->zf;
  float errfy = 5.0*0.0200;
  float errey = 5.0*0.0200;
  
  int maxiter = 5;//3;//5;
  for (int iter = 0; iter < maxiter; ++iter){ // iter
    
    double S = 0.0, Sx = 0.0, Sy = 0.0, Sxx = 0.0, Sxy = 0.0;
    double s = 0.0, sx = 0.0, sy = 0.0, sxx = 0.0, sxy = 0.0; // y copy
    
    float zmin_zero   = 1000000.0, zmax_zero   = -1000000.0;
    float zmin_stereo = 1000000.0, zmax_stereo = -1000000.0;

    // attract to the measurements
    //loop over hits of the track

    for (vector<pHit>::iterator phitIt = psp->vhit.begin();
    	 phitIt != psp->vhit.end(); ++phitIt){ // phit
          
      float cosa = (*phitIt)->cosa;
      float sina = (*phitIt)->sina;
      
      float alpha = 180.0/3.1415926*atan2(sina, cosa); if (alpha<-10.0) alpha += 360.0;

      float hitbIt_x = (*phitIt)->x;
      float hitbIt_z = (*phitIt)->z;

      if ((alpha > -1.0)&&(alpha < 1.0)){ // zero projection
	
	//straight line
	float x = ax*hitbIt_z + bx;
	float cutx = errfx + ((errex-errfx)/(psp->ze-psp->zf))*
	  (hitbIt_z-psp->zf);

	float w = 0.0;
	float dist2 = (hitbIt_x-x)*(hitbIt_x-x);
	float cut2 = cutx*cutx;
	if (dist2<cut2){
	  w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey

	  if (hitbIt_z < zmin_zero) zmin_zero = hitbIt_z;
	  if (hitbIt_z > zmax_zero) zmax_zero = hitbIt_z;

	}
	(*phitIt)->mask = w;

	S   += w;
	Sx  += w*hitbIt_z;
	Sy  += w*hitbIt_x;
	Sxx += w*hitbIt_z*hitbIt_z;
	Sxy += w*hitbIt_x*hitbIt_z;
      } // zero projection
      else{
	//straight line
	float y = ay*hitbIt_z + by;
	float cuty = errfy + ((errey-errfy)/(psp->ze-psp->zf))*(hitbIt_z-psp->zf);

	float hitb_y = ((ax*hitbIt_z + bx)*cosa - hitbIt_x)/sina;
	float w = 0.0;
	float dist2 = (hitb_y-y)*(hitb_y-y);
	float cut2 = cuty*cuty;
	if (dist2<cut2){
	  w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey

	  if (hitbIt_z < zmin_stereo) zmin_stereo = hitbIt_z;
	  if (hitbIt_z > zmax_stereo) zmax_stereo = hitbIt_z;
	  
	}
	(*phitIt)->mask = w;

	s   += w;
	sx  += w*hitbIt_z;
	sy  += w*hitb_y;
	sxx += w*hitbIt_z*hitbIt_z;
	sxy += w*hitb_y*hitbIt_z;
      } // zero 
    } // phit
    

    if ( (zmax_zero-zmin_zero) > 3.0){ 
      double D = S*Sxx-Sx*Sx;
      ax = (S*Sxy-Sx*Sy)/D;
      bx = (Sxx*Sy-Sx*Sxy)/D;
    }
    else{
      errfx *= 2.0; errex *= 2.0;
    }
    
    if ( (zmax_stereo-zmin_stereo) > 3.0){ 
      double d = s*sxx-sx*sx;
      ay = (s*sxy-sx*sy)/d;
      by = (sxx*sy-sx*sxy)/d;
    }      
    else{
      errfy *= 2.0; errey *= 2.0;
    }
    
  } // iter
  
  psp->xf = ax*psp->zf+bx; psp->xe = ax*psp->ze+bx; 
  psp->txf = psp->txe = ax;
  psp->yf = ay*psp->zf+by; psp->ye = ay*psp->ze+by; 
  psp->tyf = psp->tye = ay;
}


//-------------------------
void SP_to_rseg(SPacepoint& sp){
  
  RSEG r;
  
  r.xf  = sp.xf;  r.yf  = sp.yf;  r.zf  = sp.zf; 
  r.txf = sp.txf; r.tyf = sp.tyf;
  r.xe  = sp.xe;  r.ye  = sp.ye;  r.ze  = sp.ze; 
  r.txe = sp.txe; r.tye = sp.tye;
  
  r.cvf[ 0] = sp.cvf[0][0];      r.cvf[ 1] = sp.cvf[0][1];
  r.cvf[ 2] = sp.cvf[1][1];      r.cvf[ 3] = sp.cvf[0][2];
  r.cvf[ 4] = sp.cvf[1][2];      r.cvf[ 5] = sp.cvf[2][2];
  r.cvf[ 6] = sp.cvf[0][3];      r.cvf[ 7] = sp.cvf[1][3];
  r.cvf[ 8] = sp.cvf[2][3];      r.cvf[ 9] = sp.cvf[3][3];
  
  r.cvf[10] = r.cvf[11] = r.cvf[12] = r.cvf[13] = 0.0;
  r.cvf[14] = 100.0;
  
  r.cve[ 0] = sp.cve[0][0];      r.cve[ 1] = sp.cve[0][1];
  r.cve[ 2] = sp.cve[1][1];      r.cve[ 3] = sp.cve[0][2];
  r.cve[ 4] = sp.cve[1][2];      r.cve[ 5] = sp.cve[2][2];
  r.cve[ 6] = sp.cve[0][3];      r.cve[ 7] = sp.cve[1][3];
  r.cve[ 8] = sp.cve[2][3];      r.cve[ 9] = sp.cve[3][3];
  
  r.cve[10] = r.cve[11] = r.cve[12] = r.cve[13] = 0.0;
  r.cve[14] = 100.0;
  
  r.pf = r.pe = sp.pf;
  
  r.time = 0.0; r.dtim = 0.0;
  r.chi  = sp.chi2;
  
  r.fit  = 0;
  r.cmp  = Rsegc::patt | Rsegc::bit16 | Rsegc::msg;
  
  ArteTable<RSEG>::push_back( r );
  ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );
		
  
  for (vector<pHit>::iterator pcluIt = sp.vhit.begin();
       pcluIt != sp.vhit.end(); ++pcluIt){
    createRelation( (*pcluIt)->hitb, rseg, &RSEG::hitb );
  }
  
}


