//==========================================================================//
//                                                                          //
//  Authors:  Silvia Masciocchi                     Created:  05-Feb-99     //
//            Ivan Kisel                            Modified: 07-Jun-99     //
//            Max-Planck-Institut fuer Physik                 01-Jul-99     //
//            Muenchen                                        19-Sep-99     //
//                                                            25-Oct-99     //
//                                                            11-Dec-99     //
//                                                            23-Jan-00     //
//                                                            11-Feb-00     //
//                                                            03-Sep-00     //
//                                                            27-Nov-00     //
//                                                            01-Feb-01     //
//                                                            26-Apr-01     //
//                                                            02-May-01     //
//                                                            10-May-01     //
//  Email:    sma@mppmu.mpg.de ikisel@mppmu.mpg.de  HERA-B Collaboration    //
//                                                                          //
// fill_banks.C                                                             //
// VDS reconstruction program - fitting tracks and filling banks            //
//                              after tracking program Cats.                //
// Called from CatsEvent().                                                  //
//                                                                          //
//==========================================================================//
 
using namespace std;

#include <vector>
#include <iostream.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <arte/arte.hh>
#include "arte/RSEG_cmp.hh"
#include "arte/Chrono.hh"

#include "cats/cats.h"
#include "cats/Class_MGEDE.hh"

//extern  Chrono zeit;

int acGlobalToLocal(float *, float *, int);
int acLocalToGlobal(float *, float *, int);

const MGede& LoadGede(void);

extern "C"{
  int  CreateVDSPattern(ArtePointer<RSEG>);
  void CatsLocalRefit(RSEG *rseg, int *nh_trk, int *hits); 
  void CatsRefit(ArtePointer<RSEG>); 
  void CatsInterpolate(ArtePointer<RSEG> , float , 
		       float& , float& , float& , float& , float *);
  void CatsExtrapolate(ArtePointer<RSEG>  , float , 
		       float& , float& , float& , float& , float *);
  void prima_love(void);
  void verona_event(void);
  void second_rove(ArtePointer<RSEG>, ArtePointer<RSEG>);
}
//extern void rc99Avertex_local(void);
extern ArtePointer<RTRA> rangerSeg2track(ArtePointer<RSEG> rseg);
 
#define NCHAMBERS  10
#define NPLANES   200
#define NGEWA    1500
extern int   npr[NPLANES];
extern float zpl[NPLANES], coeff[NPLANES];

extern int   Nsl_Sec[NCHAMBERS+1][5][5];

extern int   GEDE_cmp[NPLANES],  GEDE_nsl[NPLANES],
  GEDE_sec[NPLANES],  GEDE_lay[NPLANES];
extern float GEDE_sxmn[NPLANES], GEDE_sxmx[NPLANES];
extern float GEDE_symn[NPLANES], GEDE_symx[NPLANES];
extern float GEDE_cos[NPLANES],  GEDE_sin[NPLANES]; 
extern float GEDE_xfw[NPLANES],  GEDE_nwi_x_pit[NPLANES];
extern int   GEDE_gewa[NPLANES];

extern int   base_plane[5];

extern float GEWA_xmin[NGEWA], GEWA_xmax[NGEWA];
extern float GEWA_ymin[NGEWA], GEWA_ymax[NGEWA];
extern float GEWA_zmin[NGEWA], GEWA_zmax[NGEWA];
extern float GEWA_zmean[NGEWA];
extern int   GEWA_gewa[NGEWA];
extern float GEWA_u[NGEWA], GEWA_z[NGEWA], GEWA_ms2[NGEWA];
extern int   ngewa;

extern int rover_newgeometry;
bool fill_GEDE_array_called = false;

static int nh, itk, ih;

int new_geo_for_ms = 0;
static int old_geo = 1014;


extern "C" {

  int CreateVDSPattern(ArtePointer<RSEG> rseg){
    // create bit pattern of planes

    if (rseg->cmp  != (Rsegc::vxd | Rsegc::bit16)) return 0; // not CATS

    int pattern = 0;
    
    for( ArteRelationIterator<HITB> hitbIt = rseg->hitb.begin();
	 hitbIt != rseg->hitb.end(); ++hitbIt ){
#ifdef XXX
      cout<<"gede "  <<(*hitbIt)->gede.index()
	  <<" gesl " <<(*hitbIt)->gede->gesl.index()
	  <<" layer "<<(*hitbIt)->gede->lay<<endl;
#endif//XXX
      int index = ((*hitbIt)->gede->gesl.index()-8)*4 + (*hitbIt)->gede->lay - 1;
      pattern = pattern | vds_pattern[index];
    }

#ifdef XXX
    for( int index = 0; index <32; index++){
      if ((index/4)*4 == index) cout<<" ";
      if ((pattern | vds_pattern[index]) == pattern)
	cout<<"1";
      else
	cout<<"0";
    }
    cout<<endl<<endl;
    // output : 0000 1011 1011 1011 1111 0000 0000 0000
    //          SL 1 SL 2 SL 3 SL 4 SL 5 SL 6 SL 7 SL 8
#endif//XXX
 
   return pattern;
  }
  


  // bit pattern finished



  void CatsLocalRefit(RSEG *rseg, int *nh_trk, int *hits){
    /****************************************************************
     *                                                              *
     *     This routine provides a procedure for the fast refit of  *
     *     VDS RSEG parameters, using optimized Kalman filter       *
     *                                                              *
     *     (c) Dmitry Emeliyanov, 2000, DESY HH.                    *
     *                                                              *
     ***************************************************************/
    
    int   gewa[NGEWA];
    int   i, j, k, g, gedeP, nfound;
    float Rk[4], Kk[4], Gk[4][4];
    float Z, sina, cosa, zeta_k, D_z, u, coeff;
    float chi2, pinv, xp, yp, zp, l2sqrt, zprev, ms2, dz;
    float Ax, Bx, Ay, By;
    float sigma_m2 = 0.0012*0.0012;  /* measurement resolution */
    
    pinv = rseg->pf;
    coeff = (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
    if (!new_geo_for_ms)
      coeff*=2.0; // multiple scattering model was changed in the old geometry???

    chi2 = 0.0;
    
    Rk[0] = rseg->xf;           Rk[1] = rseg->yf;
    Rk[2] = rseg->txf;          Rk[3] = rseg->tyf;
    
    Gk[0][0] = rseg->cvf[0];    Gk[0][1] = rseg->cvf[1];
    Gk[1][1] = rseg->cvf[2];    Gk[0][2] = rseg->cvf[3];
    Gk[1][2] = rseg->cvf[4];    Gk[2][2] = rseg->cvf[5];
    Gk[0][3] = rseg->cvf[6];    Gk[1][3] = rseg->cvf[7];
    Gk[2][3] = rseg->cvf[8];    Gk[3][3] = rseg->cvf[9];
    
    int ngewaf=100000, ngewae=-20, ngewai;
    int nhitb = 0;

    for (ih=nh+nh_trk[itk]-1; ih>=nh; ih--){
      ArtePointer<HITB> hitb (hits[ih]);

      nhitb++;
      ngewai = GEDE_gewa[hitb->gede.index()];
      if (ngewaf>ngewai) ngewaf = ngewai;
      if (ngewae<ngewai) ngewae = ngewai;

      if (GEWA_gewa[ngewai]>0)
	GEWA_gewa[ngewai] = -GEWA_gewa[ngewai];
      GEWA_u[ngewai] = hitb->x;
      GEWA_z[ngewai] = hitb->z;
    }

    Ax=(rseg->xe-rseg->xf)/(rseg->ze-rseg->zf);
    Bx=rseg->xf-Ax*rseg->zf;
    Ay=(rseg->ye-rseg->yf)/(rseg->ze-rseg->zf);
    By=rseg->yf-Ay*rseg->zf;

    l2sqrt = 1.0+Ax*Ax+Ay*Ay; l2sqrt = l2sqrt*sqrt(l2sqrt);
    
    nfound = 0;
    for(i=ngewaf; i<=ngewae; i++) {
      zp = 0.5*(GEWA_zmin[i]+GEWA_zmax[i]);
      xp = Ax*zp+Bx;
      yp = Ay*zp+By;

      if ((xp>=GEWA_xmin[i])&&(xp<=GEWA_xmax[i])&&
	  (yp>=GEWA_ymin[i])&&(yp<=GEWA_ymax[i]))
	gewa[nfound++]=i;
      else
	if (GEWA_gewa[i]<0)
	  gewa[nfound++]=i;
    }

    /********************* Fit for the last hit *********************/

    zprev=rseg->zf; // start from the first hit

    for(k=0; k<nfound; k++) {
      g  = gewa[k];
      dz = GEWA_zmax[g]-GEWA_zmin[g];
      ms2= GEWA_ms2[g];
      
      /***** Filtering step if there is hit *****/
      
      if (GEWA_gewa[g]<0){
	u     = GEWA_u[g];
	gedeP =-GEWA_gewa[g];
	sina  = GEDE_sin[gedeP]; cosa = GEDE_cos[gedeP];
	D_z   = GEWA_z[g]-zprev;
	zprev = GEWA_z[g];
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
	Z=sigma_m2+
	  cosa*cosa*Gk[0][0]-2.0*sina*cosa*Gk[0][1]+sina*sina*Gk[1][1];
	zeta_k=(u-cosa*Rk[0]+sina*Rk[1]);
	chi2 += zeta_k*zeta_k/Z;
	Kk[0]=cosa*Gk[0][0]-sina*Gk[0][1];
	Kk[1]=cosa*Gk[0][1]-sina*Gk[1][1];
	Kk[2]=cosa*Gk[0][2]-sina*Gk[1][2];
	Kk[3]=cosa*Gk[0][3]-sina*Gk[1][3];
	zeta_k=zeta_k/Z;
	for(j=0;j<4;j++){
	  Rk[j]+=Kk[j]*zeta_k;
	  for(i=0;i<=j;i++)
	    Gk[i][j]-=Kk[i]*Kk[j]/Z;
	}
      }
      
      /***** Prediction step : from zprev to zmax *****/
      
      D_z   = GEWA_zmax[g]-zprev;
      zprev = GEWA_zmax[g];
      Rk[0]+=D_z*Rk[2];
      Rk[1]+=D_z*Rk[3];
      Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
      Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
      Gk[0][2]+=D_z*Gk[2][2];
      Gk[0][3]+=D_z*Gk[2][3];
      Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
      Gk[1][2]+=D_z*Gk[2][3];
      Gk[1][3]+=D_z*Gk[3][3];

      /***** Multiple scattering corrections ****/

      Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
      Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
      Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;

    }

    /***** Prediction step : from zprev to ze *****/
    D_z=rseg->ze-zprev;
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];


    rseg->xe  = Rk[0];          rseg->ye  = Rk[1];
    rseg->txe = Rk[2];          rseg->tye = Rk[3];
    
    rseg->cve[0] = Gk[0][0];    rseg->cve[1] = Gk[0][1];
    rseg->cve[2] = Gk[1][1];    rseg->cve[3] = Gk[0][2];
    rseg->cve[4] = Gk[1][2];    rseg->cve[5] = Gk[2][2];
    rseg->cve[6] = Gk[0][3];    rseg->cve[7] = Gk[1][3];
    rseg->cve[8] = Gk[2][3];    rseg->cve[9] = Gk[3][3];


    /********************* Fit for the first point *********************/

    zprev=rseg->ze;

    for(k=nfound-1; k>-1; k--) {
      g  = gewa[k];
      dz = GEWA_zmin[g]-GEWA_zmax[g];
      ms2= GEWA_ms2[g];

      /***** Multiple scattering corrections ****/

      Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
      Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
      Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;

      /***** Filtering step if there is hit *****/

      if (GEWA_gewa[g]<0){
	u     = GEWA_u[g];
	gedeP =-GEWA_gewa[g];
	sina=GEDE_sin[gedeP];cosa=GEDE_cos[gedeP];
	D_z=GEWA_z[g]-zprev;
	zprev=GEWA_z[g];
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
	Z=sigma_m2+
	  cosa*cosa*Gk[0][0]-2.0*sina*cosa*Gk[0][1]+sina*sina*Gk[1][1];
	zeta_k=(u-cosa*Rk[0]+sina*Rk[1]);
	chi2 += zeta_k*zeta_k/Z;
	Kk[0]=cosa*Gk[0][0]-sina*Gk[0][1];
	Kk[1]=cosa*Gk[0][1]-sina*Gk[1][1];
	Kk[2]=cosa*Gk[0][2]-sina*Gk[1][2];
	Kk[3]=cosa*Gk[0][3]-sina*Gk[1][3];
	zeta_k=zeta_k/Z;
	for(j=0;j<4;j++){
	  Rk[j]+=Kk[j]*zeta_k;
	  for(i=0;i<=j;i++)
	    Gk[i][j]-=Kk[i]*Kk[j]/Z;
	}
      }
      /***** Prediction step: from zprev to zmin *****/
      D_z=GEWA_zmin[g]-zprev;
      zprev=GEWA_zmin[g];
      Rk[0]+=D_z*Rk[2];
      Rk[1]+=D_z*Rk[3];
      Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
      Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
      Gk[0][2]+=D_z*Gk[2][2];
      Gk[0][3]+=D_z*Gk[2][3];
      Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
      Gk[1][2]+=D_z*Gk[2][3];
      Gk[1][3]+=D_z*Gk[3][3];
    }

    /***** Prediction step : from zprev to zf *****/

    D_z=rseg->zf-zprev;
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];

    rseg->xf  = Rk[0];          rseg->yf  = Rk[1];
    rseg->txf = Rk[2];          rseg->tyf = Rk[3];
    
    rseg->cvf[0] = Gk[0][0];    rseg->cvf[1] = Gk[0][1];
    rseg->cvf[2] = Gk[1][1];    rseg->cvf[3] = Gk[0][2];
    rseg->cvf[4] = Gk[1][2];    rseg->cvf[5] = Gk[2][2];
    rseg->cvf[6] = Gk[0][3];    rseg->cvf[7] = Gk[1][3];
    rseg->cvf[8] = Gk[2][3];    rseg->cvf[9] = Gk[3][3];


    if ((rseg->cvf[0]< 0.0)||(rseg->cvf[2]< 0.0)||(rseg->cvf[5]< 0.0)||(rseg->cvf[9]< 0.0)){
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------> CATS bad cov matrix "<<rseg->cvf[0]<<" "<<rseg->cvf[2]<<" "<<rseg->cvf[5]<<" "<<rseg->cvf[9]<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
    }
    
    rseg->chi = chi2/(2.0*(nhitb-4));

    for (ih=nh+nh_trk[itk]-1; ih>=nh; ih--){
      ArtePointer<HITB> hitb (hits[ih]);

      ngewai = GEDE_gewa[hitb->gede.index()];
      if (GEWA_gewa[ngewai]<0)
	GEWA_gewa[ngewai] = -GEWA_gewa[ngewai];
      GEWA_u[ngewai]    = 0.0;
      GEWA_z[ngewai]    = 0.0;
    }
    
    return;   
  }

  
  //=========================================================================================
  
  void CatsRefit(ArtePointer<RSEG>  rseg){
    /****************************************************************
     *                                                              *
     *     This routine provides a procedure for the fast refit of  *
     *     VDS RSEG parameters, using optimized Kalman filter       *
     *                                                              *
     *     (c) Dmitry Emeliyanov, 2000, DESY HH.                    *
     *                                                              *
     ***************************************************************/
    
    int   gewa[NGEWA];
    int   i, j, k, g, gedeP, nfound;
    float Rk[4], Kk[4], Gk[4][4];
    float Z, sina, cosa, zeta_k, D_z, u, coeff;
    float chi2, pinv, xp, yp, zp, l2sqrt, zprev, ms2, dz;
    float Ax, Bx, Ay, By;
    float sigma_m2 = 0.0012*0.0012;  /* measurement resolution */
    
    pinv = rseg->pf;
    coeff = (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
    if (!new_geo_for_ms)
      coeff*=2.0; // multiple scattering model was changed in the old geometry???

    chi2 = 0.0;
    
    Rk[0] = rseg->xf;           Rk[1] = rseg->yf;
    Rk[2] = rseg->txf;          Rk[3] = rseg->tyf;
    
    Gk[0][0] = rseg->cvf[0];    Gk[0][1] = rseg->cvf[1];
    Gk[1][1] = rseg->cvf[2];    Gk[0][2] = rseg->cvf[3];
    Gk[1][2] = rseg->cvf[4];    Gk[2][2] = rseg->cvf[5];
    Gk[0][3] = rseg->cvf[6];    Gk[1][3] = rseg->cvf[7];
    Gk[2][3] = rseg->cvf[8];    Gk[3][3] = rseg->cvf[9];
    
    int ngewaf=100000, ngewae=-20, ngewai;
    int nhitb = 0;
    for (ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	  hitb != rseg->hitb.end(); ++hitb ) {
      nhitb++;
      ngewai = GEDE_gewa[(*hitb)->gede.index()];
      if (ngewaf>ngewai) ngewaf = ngewai;
      if (ngewae<ngewai) ngewae = ngewai;

      if (GEWA_gewa[ngewai]>0)
	GEWA_gewa[ngewai] = -GEWA_gewa[ngewai];
      GEWA_u[ngewai] = (*hitb)->x;
      GEWA_z[ngewai] = (*hitb)->z;
    }

    Ax=(rseg->xe-rseg->xf)/(rseg->ze-rseg->zf);
    Bx=rseg->xf-Ax*rseg->zf;
    Ay=(rseg->ye-rseg->yf)/(rseg->ze-rseg->zf);
    By=rseg->yf-Ay*rseg->zf;

    l2sqrt = 1.0+Ax*Ax+Ay*Ay; l2sqrt = l2sqrt*sqrt(l2sqrt);
    
    nfound = 0;
    for(i=ngewaf; i<=ngewae; i++) {
      zp = 0.5*(GEWA_zmin[i]+GEWA_zmax[i]);
      xp = Ax*zp+Bx;
      yp = Ay*zp+By;

      if ((xp>=GEWA_xmin[i])&&(xp<=GEWA_xmax[i])&&
	  (yp>=GEWA_ymin[i])&&(yp<=GEWA_ymax[i]))
	gewa[nfound++]=i;
      else
	if (GEWA_gewa[i]<0)
	  gewa[nfound++]=i;
    }

    /********************* Fit for the last hit *********************/

    zprev=rseg->zf; // start from the first hit

    for(k=0; k<nfound; k++) {
      g  = gewa[k];
      dz = GEWA_zmax[g]-GEWA_zmin[g];
      ms2= GEWA_ms2[g];
      
      /***** Filtering step if there is hit *****/
      
      if (GEWA_gewa[g]<0){
	u     = GEWA_u[g];
	gedeP =-GEWA_gewa[g];
	sina  = GEDE_sin[gedeP]; cosa = GEDE_cos[gedeP];
	D_z   = GEWA_z[g]-zprev;
	zprev = GEWA_z[g];
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
	Z=sigma_m2+
	  cosa*cosa*Gk[0][0]-2.0*sina*cosa*Gk[0][1]+sina*sina*Gk[1][1];
	zeta_k=(u-cosa*Rk[0]+sina*Rk[1]);
	chi2 += zeta_k*zeta_k/Z;
	Kk[0]=cosa*Gk[0][0]-sina*Gk[0][1];
	Kk[1]=cosa*Gk[0][1]-sina*Gk[1][1];
	Kk[2]=cosa*Gk[0][2]-sina*Gk[1][2];
	Kk[3]=cosa*Gk[0][3]-sina*Gk[1][3];
	zeta_k=zeta_k/Z;
	for(j=0;j<4;j++){
	  Rk[j]+=Kk[j]*zeta_k;
	  for(i=0;i<=j;i++)
	    Gk[i][j]-=Kk[i]*Kk[j]/Z;
	}
      }
      
      /***** Prediction step : from zprev to zmax *****/
      
      D_z   = GEWA_zmax[g]-zprev;
      zprev = GEWA_zmax[g];
      Rk[0]+=D_z*Rk[2];
      Rk[1]+=D_z*Rk[3];
      Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
      Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
      Gk[0][2]+=D_z*Gk[2][2];
      Gk[0][3]+=D_z*Gk[2][3];
      Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
      Gk[1][2]+=D_z*Gk[2][3];
      Gk[1][3]+=D_z*Gk[3][3];

      /***** Multiple scattering corrections ****/

      Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
      Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
      Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;

    }

    /***** Prediction step : from zprev to ze *****/
    D_z=rseg->ze-zprev;
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];


    rseg->xe  = Rk[0];          rseg->ye  = Rk[1];
    rseg->txe = Rk[2];          rseg->tye = Rk[3];
    
    rseg->cve[0] = Gk[0][0];    rseg->cve[1] = Gk[0][1];
    rseg->cve[2] = Gk[1][1];    rseg->cve[3] = Gk[0][2];
    rseg->cve[4] = Gk[1][2];    rseg->cve[5] = Gk[2][2];
    rseg->cve[6] = Gk[0][3];    rseg->cve[7] = Gk[1][3];
    rseg->cve[8] = Gk[2][3];    rseg->cve[9] = Gk[3][3];

    Rk[0] = rseg->xe;           Rk[1] = rseg->ye;
    Rk[2] = rseg->txe;          Rk[3] = rseg->tye;
    
    Gk[0][0] = rseg->cvf[0];    Gk[0][1] = rseg->cvf[1];
    Gk[1][1] = rseg->cvf[2];    Gk[0][2] = rseg->cvf[3];
    Gk[1][2] = rseg->cvf[4];    Gk[2][2] = rseg->cvf[5];
    Gk[0][3] = rseg->cvf[6];    Gk[1][3] = rseg->cvf[7];
    Gk[2][3] = rseg->cvf[8];    Gk[3][3] = rseg->cvf[9];

    /********************* Fit for the first point *********************/

    zprev=rseg->ze;

    for(k=nfound-1; k>-1; k--) {
      g  = gewa[k];
      dz = GEWA_zmin[g]-GEWA_zmax[g];
      ms2= GEWA_ms2[g];

      /***** Multiple scattering corrections ****/

      Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
      Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
      Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;

      /***** Filtering step if there is hit *****/

      if (GEWA_gewa[g]<0){
	u     = GEWA_u[g];
	gedeP =-GEWA_gewa[g];
	sina=GEDE_sin[gedeP];cosa=GEDE_cos[gedeP];
	D_z=GEWA_z[g]-zprev;
	zprev=GEWA_z[g];
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
	Z=sigma_m2+
	  cosa*cosa*Gk[0][0]-2.0*sina*cosa*Gk[0][1]+sina*sina*Gk[1][1];
	zeta_k=(u-cosa*Rk[0]+sina*Rk[1]);
	chi2 += zeta_k*zeta_k/Z;
	Kk[0]=cosa*Gk[0][0]-sina*Gk[0][1];
	Kk[1]=cosa*Gk[0][1]-sina*Gk[1][1];
	Kk[2]=cosa*Gk[0][2]-sina*Gk[1][2];
	Kk[3]=cosa*Gk[0][3]-sina*Gk[1][3];
	zeta_k=zeta_k/Z;
	for(j=0;j<4;j++){
	  Rk[j]+=Kk[j]*zeta_k;
	  for(i=0;i<=j;i++)
	    Gk[i][j]-=Kk[i]*Kk[j]/Z;
	}
      }
      /***** Prediction step: from zprev to zmin *****/
      D_z=GEWA_zmin[g]-zprev;
      zprev=GEWA_zmin[g];
      Rk[0]+=D_z*Rk[2];
      Rk[1]+=D_z*Rk[3];
      Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
      Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
      Gk[0][2]+=D_z*Gk[2][2];
      Gk[0][3]+=D_z*Gk[2][3];
      Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
      Gk[1][2]+=D_z*Gk[2][3];
      Gk[1][3]+=D_z*Gk[3][3];
    }

    /***** Prediction step : from zprev to zf *****/

    D_z=rseg->zf-zprev;
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];

    rseg->xf  = Rk[0];          rseg->yf  = Rk[1];
    rseg->txf = Rk[2];          rseg->tyf = Rk[3];
    
    rseg->cvf[0] = Gk[0][0];    rseg->cvf[1] = Gk[0][1];
    rseg->cvf[2] = Gk[1][1];    rseg->cvf[3] = Gk[0][2];
    rseg->cvf[4] = Gk[1][2];    rseg->cvf[5] = Gk[2][2];
    rseg->cvf[6] = Gk[0][3];    rseg->cvf[7] = Gk[1][3];
    rseg->cvf[8] = Gk[2][3];    rseg->cvf[9] = Gk[3][3];


    if ((rseg->cvf[0]< 0.0)||(rseg->cvf[2]< 0.0)||(rseg->cvf[5]< 0.0)||(rseg->cvf[9]< 0.0)){
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------> CATS bad cov matrix "<<rseg->cvf[0]<<" "<<rseg->cvf[2]<<" "<<rseg->cvf[5]<<" "<<rseg->cvf[9]<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
    }
    
    rseg->chi = chi2/(2.0*(nhitb-4));

    for (ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	 hitb != rseg->hitb.end(); ++hitb ) 
      {
	ngewai = GEDE_gewa[(*hitb)->gede.index()];
	if (GEWA_gewa[ngewai]<0)
	  GEWA_gewa[ngewai] = -GEWA_gewa[ngewai];
	GEWA_u[ngewai]    = 0.0;
	GEWA_z[ngewai]    = 0.0;
      }
    
    return;   
  }

  
  //=========================================================================================
  
  void CatsExtrapolate(ArtePointer<RSEG>  rseg, float z0, 
		     float& x0, float& y0, float& tx, float& ty, float *cov){
    /****************************************************************
     *                                                              *
     *     This routine provides a procedure for the track          *
     *     parameters extrapolation to a vertex point               *
     *                                                              *
     *     (c) Dmitry Emeliyanov, 2000, DESY HH.                    *
     *                                                              *
     ***************************************************************/
    
    int   gewa[NGEWA];
    int   i, k, g, nfound;
    float Rk[4], Gk[4][4];
    float coeff, pinv, xp, yp, zp, l2sqrt, zprev, ms2, D_z;
    float Ax, Bx, Ay, By;

    if ((z0>=rseg->zf)&&(z0<=rseg->ze)){
      CatsInterpolate(rseg, z0, x0, y0, tx, ty, cov);
      return;
    }

    Ax=(rseg->xe-rseg->xf)/(rseg->ze-rseg->zf);
    Bx=rseg->xf-Ax*rseg->zf;
    Ay=(rseg->ye-rseg->yf)/(rseg->ze-rseg->zf);
    By=rseg->yf-Ay*rseg->zf;
    
    l2sqrt = 1.0+Ax*Ax+Ay*Ay; l2sqrt = l2sqrt*sqrt(l2sqrt);
    
    if (z0<rseg->zf){
      pinv = rseg->pf;
      coeff = (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
      if (!new_geo_for_ms)
	coeff*=2.0;      

      
      Rk[0] = rseg->xf;           Rk[1] = rseg->yf;
      Rk[2] = rseg->txf;          Rk[3] = rseg->tyf;
      
      Gk[0][0] = rseg->cvf[0];    Gk[0][1] = rseg->cvf[1];
      Gk[1][1] = rseg->cvf[2];    Gk[0][2] = rseg->cvf[3];
      Gk[1][2] = rseg->cvf[4];    Gk[2][2] = rseg->cvf[5];
      Gk[0][3] = rseg->cvf[6];    Gk[1][3] = rseg->cvf[7];
      Gk[2][3] = rseg->cvf[8];    Gk[3][3] = rseg->cvf[9];
      
      ArtePointer<HITB> hitb = rseg->hitb.begin();
      int ngewaf = GEDE_gewa[hitb->gede.index()];
      
      nfound = 0;
      for(i=0; i<ngewaf; i++) {
	if (GEWA_zmax[i]<z0) continue;
	zp = 0.5*(GEWA_zmin[i]+GEWA_zmax[i]);
	xp = Ax*zp+Bx;
	yp = Ay*zp+By;
	
	if ((xp>=GEWA_xmin[i])&&(xp<=GEWA_xmax[i])&&
	    (yp>=GEWA_ymin[i])&&(yp<=GEWA_ymax[i]))
	  gewa[nfound++]=i;
      }
      
      /********************* Fit for the point at z0 *********************/
      
      zprev=rseg->zf;
      
      for(k=nfound-1; k>=0; k--) {
	g  = gewa[k];
	ms2= GEWA_ms2[g];
	
	/***** Multiple scattering corrections ****/
	
	Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
	Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
	Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;
	
	/***** Prediction step: from zprev to zmin *****/
	D_z=GEWA_zmin[g]-zprev; 
	zprev=GEWA_zmin[g];
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
      }
      
      /***** Prediction step : from zprev to z0 *****/
      
      D_z=z0-zprev;
      Rk[0]+=D_z*Rk[2];
      Rk[1]+=D_z*Rk[3];
      Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
      Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
      Gk[0][2]+=D_z*Gk[2][2];
      Gk[0][3]+=D_z*Gk[2][3];
      Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
      Gk[1][2]+=D_z*Gk[2][3];
      Gk[1][3]+=D_z*Gk[3][3];
    }
    else{
      pinv = rseg->pe;
      coeff = (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
      if (!new_geo_for_ms)
	coeff*=2.0; // multiple scattering model was changed in the new geometry???

      
      Rk[0] = rseg->xe;           Rk[1] = rseg->ye;
      Rk[2] = rseg->txe;          Rk[3] = rseg->tye;
      
      Gk[0][0] = rseg->cve[0];    Gk[0][1] = rseg->cve[1];
      Gk[1][1] = rseg->cve[2];    Gk[0][2] = rseg->cve[3];
      Gk[1][2] = rseg->cve[4];    Gk[2][2] = rseg->cve[5];
      Gk[0][3] = rseg->cve[6];    Gk[1][3] = rseg->cve[7];
      Gk[2][3] = rseg->cve[8];    Gk[3][3] = rseg->cve[9];
      
      int ngewaf=0;
      for (ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	   hitb != rseg->hitb.end(); ++hitb ) {

	ngewaf = GEDE_gewa[(*hitb)->gede.index()];
      }
      
      nfound = 0;
      for(i=ngewaf; i<ngewa; i++) {
	if (GEWA_zmin[i]>z0) break;
	zp = 0.5*(GEWA_zmin[i]+GEWA_zmax[i]);
	xp = Ax*zp+Bx;
	yp = Ay*zp+By;
	
	if ((xp>=GEWA_xmin[i])&&(xp<=GEWA_xmax[i])&&
	    (yp>=GEWA_ymin[i])&&(yp<=GEWA_ymax[i]))
	  gewa[nfound++]=i;
      }
      
      /********************* Fit for the point at z0 *********************/
      
      zprev=rseg->ze;
      
      for(k=0; k<nfound; k++) {
	g  = gewa[k];
	ms2= GEWA_ms2[g];
	
	/***** Multiple scattering corrections ****/
	
	Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
	Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
	Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;
	
	/***** Prediction step: from zprev to zmax *****/
	D_z=GEWA_zmax[g]-zprev; 
	zprev=GEWA_zmax[g];
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
      }
      
      /***** Prediction step : from zprev to z0 *****/
      
      D_z=z0-zprev;
      Rk[0]+=D_z*Rk[2];
      Rk[1]+=D_z*Rk[3];
      Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
      Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
      Gk[0][2]+=D_z*Gk[2][2];
      Gk[0][3]+=D_z*Gk[2][3];
      Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
      Gk[1][2]+=D_z*Gk[2][3];
      Gk[1][3]+=D_z*Gk[3][3];
    }

    x0 = Rk[0];           y0  = Rk[1];
    tx = Rk[2];           ty  = Rk[3];
    
    cov[0] = Gk[0][0];    cov[1] = Gk[0][1];
    cov[2] = Gk[1][1];    cov[3] = Gk[0][2];
    cov[4] = Gk[1][2];    cov[5] = Gk[2][2];
    cov[6] = Gk[0][3];    cov[7] = Gk[1][3];
    cov[8] = Gk[2][3];    cov[9] = Gk[3][3];
    
    return;   
  }


  //=======================================================================

  void CatsInterpolate(ArtePointer<RSEG>  rseg, float z0, 
		       float& x0, float& y0, float& tx, float& ty, float *cov){
    /****************************************************************
     *                                                              *
     *     This routine provides a procedure for the fast fit of    *
     *     VDS track parameters in the intermediate point, using    * 
     *     Kalman parameter estimator                               *
     *                                                              *
     *     (c) Dmitry Emeliyanov, 2000, DESY HH.                    *
     *                                                              *
     ***************************************************************/
    
    int   gewa[NGEWA];
    int   i,j,k,g,gi,gedeP,nfound,rfound;
    float Rk[4], Kk[4], Gk[4][4];
    float Z,Z1,Z2,sina,cosa,zeta_k,D_z,u,pinv,coeff,D_z2;
    float Ax, Bx, Ay, By, sigma_m_s2;
    float xp, yp, zp, l2sqrt, zhit,dz21,dz212,dz102,dz202,sum;
    float sigma_m_s = 0.0020;         /* mult scatt  */
    float sigma_m2  = 0.0013*0.0013;  /* measurement resolution */


    if ((z0<rseg->zf)||(z0>rseg->ze)){
      CatsExtrapolate(rseg, z0, x0, y0, tx, ty, cov);
      return;
    }

    Ax=(rseg->xe-rseg->xf)/(rseg->ze-rseg->zf);
    Bx=rseg->xf-Ax*rseg->zf;
    Ay=(rseg->ye-rseg->yf)/(rseg->ze-rseg->zf);
    By=rseg->yf-Ay*rseg->zf;
    
    l2sqrt = 1.0+Ax*Ax+Ay*Ay; l2sqrt = l2sqrt*l2sqrt*sqrt(l2sqrt);
    
    pinv = rseg->pf;
    coeff = (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
    if (!new_geo_for_ms)
      coeff*=2.0; // multiple scattering model was changed in the new geometry???

    sigma_m_s2=2.0*sigma_m_s*sigma_m_s*coeff*l2sqrt;

    Z1 = rseg->zf;Z2 = rseg->ze;dz21=(Z2-Z1);    
    Rk[0]=rseg->xf+(rseg->xe-rseg->xf)*(z0-Z1)/dz21;
    Rk[1]=rseg->yf+(rseg->ye-rseg->yf)*(z0-Z1)/dz21;
    Rk[2]=(rseg->xe-rseg->xf)/dz21;
    Rk[3]=(rseg->ye-rseg->yf)/dz21;
      
    dz212=(Z2-Z1)*(Z2-Z1); dz102=(Z1-z0)*(Z1-z0); dz202=(Z2-z0)*(Z2-z0);
    
    Gk[0][0]=Gk[1][1]=sigma_m2*(dz202+dz102)/dz212+2.0*sigma_m_s2*dz212;
    Gk[2][2]=Gk[3][3]=2.0*sigma_m2/dz212+2.0*sigma_m_s2;
    Gk[0][2]=Gk[1][3]= 0.0;
    Gk[0][1]=Gk[0][3]=Gk[1][2]=Gk[2][3]=0.0;

    int ngewaf=100000, ngewae=-20, ngewai;
    for (ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	 hitb != rseg->hitb.end(); ++hitb ) {
      
      ngewai = GEDE_gewa[(*hitb)->gede.index()];
      if (ngewaf>ngewai)    ngewaf = ngewai;
      if (ngewae<ngewai)    ngewae = ngewai;
      
      if (GEWA_gewa[ngewai]>0)
	GEWA_gewa[ngewai] = -GEWA_gewa[ngewai];
      GEWA_u[ngewai] = (*hitb)->x;
      GEWA_z[ngewai] = (*hitb)->z;
    }
    
    nfound = 0;rfound = -10000;
    for(i=ngewaf; i<=ngewae; i++) {
      zp = 0.5*(GEWA_zmin[i]+GEWA_zmax[i]);
      xp = Ax*zp+Bx;
      yp = Ay*zp+By;

      if ((xp>=GEWA_xmin[i])&&(xp<=GEWA_xmax[i])&&
	  (yp>=GEWA_ymin[i])&&(yp<=GEWA_ymax[i])){
	gewa[nfound++]=i;
	if ((zp>z0)&&(rfound<0)) rfound = nfound-1;
      }
      else{
	if (GEWA_gewa[i]<0){
	  gewa[nfound++]=i;
	  if ((zp>z0)&&(rfound<0)) rfound = nfound-1;
	}
      }
    }
    if (rfound<0) rfound = nfound;
    for(k=0; k<nfound; k++) {
      g  = gewa[k];
      if (GEWA_gewa[g]<0){
	u     = GEWA_u[g];
	gedeP =-GEWA_gewa[g];
	sina  = GEDE_sin[gedeP]; cosa = GEDE_cos[gedeP];
	D_z   = GEWA_z[g]-z0;D_z2=D_z*D_z;
	zhit = GEWA_z[g];
	sum=0.0;
	if(zhit<z0)
	  {
	    for(i=k;i<rfound;i++)
	      {
		gi=gewa[i];
		sum+=GEWA_ms2[gi]*(zhit-GEWA_zmin[gi])*(zhit-GEWA_zmin[gi]);
	      }
	  }
	else
	  {
	    for(i=rfound;i<=k;i++)
	      {
		gi=gewa[i];
		sum+=GEWA_ms2[gi]*(zhit-GEWA_zmin[gi])*(zhit-GEWA_zmin[gi]);
	      }
	  }

	Z=sum*l2sqrt*coeff+sigma_m2+
	  cosa*cosa*(Gk[0][0]+2.*Gk[0][2]*D_z+Gk[2][2]*D_z2)+
	  sina*sina*(Gk[1][1]+2.*Gk[1][3]*D_z+Gk[3][3]*D_z2)-
	  2.*sina*cosa*(Gk[0][1]+(Gk[0][3]+Gk[1][2])*D_z+Gk[2][3]*D_z2);
	zeta_k=(u-cosa*(Rk[0]+Rk[2]*D_z)+sina*(Rk[1]+Rk[3]*D_z))/Z;
	Kk[0]=cosa*(Gk[0][0]+Gk[0][2]*D_z)-sina*(Gk[0][1]+Gk[0][3]*D_z);
	Kk[1]=cosa*(Gk[0][1]+Gk[1][2]*D_z)-sina*(Gk[1][1]+Gk[1][3]*D_z);
	Kk[2]=cosa*(Gk[0][2]+Gk[2][2]*D_z)-sina*(Gk[1][2]+Gk[2][3]*D_z);
	Kk[3]=cosa*(Gk[0][3]+Gk[2][3]*D_z)-sina*(Gk[1][3]+Gk[3][3]*D_z);
	for(j=0;j<4;j++){
	  Rk[j]+=Kk[j]*zeta_k;
	  for(i=0;i<=j;i++){
	    Gk[i][j]-=Kk[i]*Kk[j]/Z;
	  }
	}
      }
    }
    
    for (ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	 hitb != rseg->hitb.end(); ++hitb ) {
      ngewai = GEDE_gewa[(*hitb)->gede.index()];
      if (GEWA_gewa[ngewai]<0)
	GEWA_gewa[ngewai] = -GEWA_gewa[ngewai];
      GEWA_u[ngewai]    = 0.0;
      GEWA_z[ngewai]    = 0.0;
    }
    
    x0=Rk[0];               y0=Rk[1];
    tx=Rk[2];               ty=Rk[3];
    
    cov[0] = Gk[0][0];      cov[1] = Gk[0][1];
    cov[2] = Gk[1][1];      cov[3] = Gk[0][2];
    cov[4] = Gk[1][2];      cov[5] = Gk[2][2];
    cov[6] = Gk[0][3];      cov[7] = Gk[1][3];
    cov[8] = Gk[2][3];      cov[9] = Gk[3][3];
    
    return;   
  }


  //=======================================================================

  
  /*
  void time_start(){
    zeit.start();
  }


  void time_stop(){
    zeit.stop();
  }
  */

  //=====================================================================

  int acglobaltolocal(float * rLocal, float * rGlobal, int cmp){

    return  acGlobalToLocal(rLocal, rGlobal, cmp);
  }


  //======================================================================

  int acGTARinGlobalCoordinates(void){
    int flag = -1;

    if (ArteTable<ACGA>::size() == 0 ) return flag;
    
    float local[5], global[5];
    local[3] = 0.; local[4] = 0.; /* transform wires as space points */

    // loop over all wires
    
    for( ArteTable<GTAR>::iterator gtar = ArteTable<GTAR>::begin();
         gtar != ArteTable<GTAR>::end(); ++gtar ) {

      local[0] = (*gtar)->x; 
      local[1] = (*gtar)->y; 
      local[2] = (*gtar)->z; 
      //cout<<"Local  "<<(*gtar)->x<<" "<<(*gtar)->y<<" "<<(*gtar)->z<<endl;

      flag = acLocalToGlobal(local, global, 1);
      (*gtar)->x=global[0]; 
      (*gtar)->y=global[1]; 
      (*gtar)->z=global[2]; 

      //cout<<"Global "<<(*gtar)->x<<" "<<(*gtar)->y<<" "<<(*gtar)->z<<endl;
      
    };

    return flag;
  }


  //========================================================================

  void fill_GEDE_array(void) {
    int   ipl;
    float alpha;

    MGede gedeMap = LoadGede();

    float Xmin = 1000.0,    Xmax =-1000.0;
    float Ymin = 1000.0,    Ymax =-1000.0;
    float sxmn, sxmx, symn, symx, xmin, xmax, ymin, ymax;
    float zmin, zmax, zmean, u, z, ms2, tmp;
    int   ok_gewa=(1==1);
    fill_GEDE_array_called = true;
    ngewa = 0;

    //check the new geometry
    ArtePointer<NGHD> nhd(1); 
    int Major = int(nhd->geom+1.e-5);
    int Minor = int(10000.*(nhd->geom+1.e-5 - float(Major)));     
    if (Minor > old_geo) new_geo_for_ms = 1;

    rover_newgeometry = 1; // to initialize gtar arrays

    /* use MGedeIterator */
    for(MGedeIterator pos = gedeMap.begin(); pos!=gedeMap.end(); ++pos) {

      ipl = pos->gede();
      
      GEDE_xfw[ipl] = pos->dmin(); 
      zpl[ipl] = pos->z();
      
      
      GEDE_cos[ipl]  = pos->c();         GEDE_sin[ipl]  = pos->s();
      GEDE_sxmn[ipl] = pos->sxmn()-0.1;  GEDE_sxmx[ipl] = pos->sxmx()+0.1;
      GEDE_symn[ipl] = pos->symn()-0.1;  GEDE_symx[ipl] = pos->symx()+0.1;
      
      alpha  = pos->alpha();
      npr[ipl] = 0;
      
      if(  80.0 < alpha && alpha <   90.0 ){
	npr[ipl]   =  1;	coeff[ipl] =  1.0;	base_plane[1] = ipl;
      }
      if( 260.0 < alpha && alpha <  270.0 ){
	npr[ipl]   =  1;	coeff[ipl] = -1.0;
	}
      if(  90.0 < alpha && alpha <  100.0 ){
	npr[ipl]   =  2;	coeff[ipl] =  1.0;	base_plane[2] = ipl;
      }
      if( 270.0 < alpha && alpha <  280.0 ){
	npr[ipl]   =  2;	coeff[ipl] = -1.0;
      }
      if( 170.0 < alpha && alpha <  180.0 ){
	npr[ipl]   =  3;	coeff[ipl] =  1.0;	base_plane[3] = ipl;
      }
      if( 350.0 < alpha && alpha <  360.0 ){
	npr[ipl]   =  3;	coeff[ipl] = -1.0;
      }
      if( -10.0 < alpha && alpha <    0.0 ){
	npr[ipl]   =  3;	coeff[ipl] = -1.0;
      }
      if( 180.0 < alpha && alpha <  190.0 ){
	npr[ipl]   =  4;	coeff[ipl] =  1.0;	base_plane[4] = ipl;
      }
      if(   0.0 < alpha && alpha <   10.0 ){
	npr[ipl]   =  4;	coeff[ipl] = -1.0;
      }
      
      if (npr[ipl]==0)
	cout<<"*** CATS: Wrong angle for the plane "<<ipl<<": "<<alpha<<endl; 
      
      Nsl_Sec[GEDE_nsl[ipl]][GEDE_sec[ipl]][npr[ipl]] = ipl;
      //#ifdef XXX
      ArtePointer<GEDE> gede(pos->gede());
	  
      sxmn = gede->sxmn*gede->rot[0]+gede->symn*gede->rot[1];
      sxmx = gede->sxmx*gede->rot[0]+gede->symx*gede->rot[1];
      symn = gede->sxmn*gede->rot[3]+gede->symn*gede->rot[4];
      symx = gede->sxmx*gede->rot[3]+gede->symx*gede->rot[4];
	  
      if( sxmn > sxmx ){
	tmp  = sxmx; sxmx = sxmn; sxmn = tmp;
      }
      if( symn > symx ){
	tmp  = symx; symx = symn; symn = tmp;
      }

      xmin = gede->xmin*gede->rot[0]+gede->ymin*gede->rot[1];
      xmax = gede->xmax*gede->rot[0]+gede->ymax*gede->rot[1];
      ymin = gede->xmin*gede->rot[3]+gede->ymin*gede->rot[4];
      ymax = gede->xmax*gede->rot[3]+gede->ymax*gede->rot[4];
	  
      if( xmin > xmax ){
	tmp  = xmax; xmax = xmin; xmin = tmp;
      }
      if( ymin > ymax ){
	tmp  = ymax; ymax = ymin; ymin = tmp;
      }


      GEWA_xmin[ngewa] = pos->sxmn()+(xmin-sxmn);
      GEWA_xmax[ngewa] = pos->sxmx()+(xmax-sxmx);
      GEWA_ymin[ngewa] = pos->symn()+(ymin-symn);
      GEWA_ymax[ngewa] = pos->symx()+(ymax-symx);
      GEWA_zmean[ngewa]= pos->z();
      GEWA_zmin[ngewa] = pos->z()+(gede->zmin-gede->zfw[0]);
      GEWA_zmax[ngewa] = pos->z()+(gede->zmax-gede->zfw[0]);
      GEWA_gewa[ngewa] = -pos->gede();
      GEDE_gewa[pos->gede()] = ngewa;
      GEWA_u[ngewa]    = 0.0; 
      GEWA_z[ngewa]    = 0.0; 
      GEWA_ms2[ngewa]  = 1.1*gede->ms;

      if (GEWA_xmin[ngewa]<Xmin) Xmin=GEWA_xmin[ngewa];
      if (GEWA_xmax[ngewa]>Xmax) Xmax=GEWA_xmax[ngewa];
      if (GEWA_ymin[ngewa]<Ymin) Ymin=GEWA_ymin[ngewa];
      if (GEWA_ymax[ngewa]>Ymax) Ymax=GEWA_ymax[ngewa];

      if (++ngewa>=NGEWA){
	ngewa--;
	if (ok_gewa){
	  cout << "--->" << endl;
	  cout << "---> CATS : Failed to initialize GEWA table" << endl;
	  cout << "--->" << endl;
	  ok_gewa=(1==0);
	}
      }
      //#endif 
    }


    for ( ArteTable<GESL>::iterator gesl = ArteTable<GESL>::begin();
	  gesl != ArteTable<GESL>::end(); ++gesl ) {
      if ((*gesl)->cmp == 1){


	for ( ArteRelationIterator<GWAL> gwal = (*gesl)->gwal.begin();
	      gwal != (*gesl)->gwal.end(); ++gwal ) {
	  
	  xmin = (*gwal)->xmin*(*gwal)->rot[0]+(*gwal)->ymin*(*gwal)->rot[1];
	  xmax = (*gwal)->xmax*(*gwal)->rot[0]+(*gwal)->ymax*(*gwal)->rot[1];
	  ymin = (*gwal)->xmin*(*gwal)->rot[3]+(*gwal)->ymin*(*gwal)->rot[4];
	  ymax = (*gwal)->xmax*(*gwal)->rot[3]+(*gwal)->ymax*(*gwal)->rot[4];
	  
	  if( xmin > xmax ){
	    tmp  = xmax; xmax = xmin; xmin = tmp;
	  }
	  if( ymin > ymax ){
	    tmp  = ymax; ymax = ymin; ymin = tmp;
	  }
	  
	  if ((xmin>Xmax)||(xmax<Xmin)||(ymin>Ymax)||(ymax<Ymin)){
	  }
	  else{
	    GEWA_xmin[ngewa] = xmin;
	    GEWA_xmax[ngewa] = xmax;
	    GEWA_ymin[ngewa] = ymin;
	    GEWA_ymax[ngewa] = ymax;
	    GEWA_zmin[ngewa] = (*gwal)->zmin;
	    GEWA_zmax[ngewa] = (*gwal)->zmax;
	    GEWA_zmean[ngewa]= 0.5*((*gwal)->zmin+(*gwal)->zmax);
	    GEWA_gewa[ngewa] = gwal.index();
	    GEWA_u[ngewa]    = 0.0; 
	    GEWA_z[ngewa]    = 0.0; 
	    GEWA_ms2[ngewa]  = (*gwal)->ms;
	    
	    if (++ngewa>=NGEWA){
	      ngewa--;
	      if (ok_gewa){
		cout << "--->" << endl;
		cout << "---> CATS : Failed to initialize GEWA table" << endl;
		cout << "--->" << endl;
		ok_gewa=(1==0);
	      }
	    }
	  }
	}
      }
    }


    // sort gewa
    int gewa_i;
    for (int i=0; i<ngewa; i++){ 
      float zmean_i = GEWA_zmean[i];
      for (int j=i+1; j<ngewa; j++){ 
	if (zmean_i>0.5*(GEWA_zmin[j]+GEWA_zmax[j])){
	  // exchange i<->j
	  xmin   = GEWA_xmin[i];	  xmax   = GEWA_xmax[i];
	  ymin   = GEWA_ymin[i];	  ymax   = GEWA_ymax[i];
	  zmin   = GEWA_zmin[i];	  zmax   = GEWA_zmax[i];
	  u      = GEWA_u[i];    	  z      = GEWA_z[i];
	  ms2    = GEWA_ms2[i];  	  gewa_i = GEWA_gewa[i]; 
	  zmean  = GEWA_zmean[i];
	  if (GEWA_gewa[i]<0) GEDE_gewa[-GEWA_gewa[i]]=j;

	  GEWA_xmin[i] = GEWA_xmin[j];	  GEWA_xmax[i] = GEWA_xmax[j];
	  GEWA_ymin[i] = GEWA_ymin[j];	  GEWA_ymax[i] = GEWA_ymax[j];
	  GEWA_zmin[i] = GEWA_zmin[j];	  GEWA_zmax[i] = GEWA_zmax[j];
	  GEWA_u[i]    = GEWA_u[j];	  GEWA_z[i]    = GEWA_z[j];
	  GEWA_ms2[i]  = GEWA_ms2[j];	  GEWA_gewa[i] = GEWA_gewa[j];
	  GEWA_zmean[i]= GEWA_zmean[j];
	  if (GEWA_gewa[j]<0) GEDE_gewa[-GEWA_gewa[j]]=i;

	  GEWA_xmin[j] = xmin;	          GEWA_xmax[j] = xmax;
	  GEWA_ymin[j] = ymin;	          GEWA_ymax[j] = ymax;
	  GEWA_zmin[j] = zmin;	          GEWA_zmax[j] = zmax;
	  GEWA_gewa[j] = gewa_i;	  GEWA_u[j]    = u;
	  GEWA_z[j]    = z;	          GEWA_ms2[j]  = ms2;
	  GEWA_zmean[j]= zmean;

	  zmean_i = GEWA_zmean[i];
	}
      }
      if (GEWA_gewa[i]<0) GEWA_gewa[i]=-GEWA_gewa[i];
    }

  }
  

  //======================================================================

  int fill_mimp_points(int n_mtra, 
		  float *mimp_x, float *mimp_y, float *mimp_z){
    int nmimps;

    nmimps = 0;
    ArteTable<MTRA>::iterator mtra = (ArteTable<MTRA>::iterator) n_mtra;
    for( ArteRelationIterator<MIMP> mimp = mtra->mimp.begin();
	 mimp != mtra->mimp.end(); ++mimp ){
      if ((*mimp)->gede->cmp != 1 ) continue;
      nmimps++;
      
      mimp_x[nmimps] = (*mimp)->x;
      mimp_y[nmimps] = (*mimp)->y;
      mimp_z[nmimps] = (*mimp)->z;
    }

    return nmimps;
  }

   //=======================================================================

  void fill_banks(int Ntk, int *nh_trk, 
		  float *Xp_trk, float *X0_trk, float *Yp_trk, float *Y0_trk, 
		  float *Z1_trk, float *Z2_trk, float *Chi2_trk, int *hits){
    
    //    int nh, itk, ih;
    float dz212, dz10f, dz20f, dz10e, dz20e, txy2, pinv;
    float coeff, sigma_m_s2;
    RSEG r;
    RHIT rh;
    float sigma_m_s = 0.0020;         /* mult scatt  */
    float sigma_m2  = 0.0013*0.0013;  /* measurement */
   
    /*fill the RSEG table*/

    nh = 0;
    for (itk=0;itk<Ntk;itk++){ // loop over 3D tracks

      // a track is parametrized as a line y(or x)=A*z+B in projections
      // for x proj. A = Xp_trk[itk]; B = X0_trk[itk];   
      // for y proj. A = Yp_trk[itk]; B = Y0_trk[itk];


      r.zf  = ArtePointer<HITB>(hits[nh])->z;
      r.xf  = Xp_trk[itk]*r.zf + X0_trk[itk];
      r.yf  = Yp_trk[itk]*r.zf + Y0_trk[itk];
      r.txf = Xp_trk[itk];
      r.tyf = Yp_trk[itk];

      r.ze  = ArtePointer<HITB>(hits[nh+nh_trk[itk]-1])->z;
      r.xe  = Xp_trk[itk]*r.ze + X0_trk[itk];
      r.ye  = Yp_trk[itk]*r.ze + Y0_trk[itk];
      r.txe = Xp_trk[itk];
      r.tye = Yp_trk[itk];

      txy2 = Xp_trk[itk]*Xp_trk[itk]+Yp_trk[itk]*Yp_trk[itk];
      pinv = 4.0*sqrt(txy2);

      coeff = (1.0+txy2)*(1.0+txy2)*(1.0+0.01*pinv*pinv)*pinv*pinv;
      if (!new_geo_for_ms)
	coeff*=2.0; // multiple scattering model was changed in the new geometry???

      sigma_m_s2=sigma_m_s*sigma_m_s*coeff;

      dz212=(Z2_trk[itk]-Z1_trk[itk])*(Z2_trk[itk]-Z1_trk[itk]); 
      dz10f=(Z1_trk[itk]-r.zf);       dz20f=(Z2_trk[itk]-r.zf); 
      dz10e=(Z1_trk[itk]-r.ze);       dz20e=(Z2_trk[itk]-r.ze); 

      r.cvf[0]=r.cvf[2]= sigma_m2*(dz20f*dz20f+dz10f*dz10f)/dz212+
	2.0*sigma_m_s2*dz212;
      r.cvf[5]=r.cvf[9]= 2.0*sigma_m2/dz212+2.0*sigma_m_s2;
      r.cvf[3]=r.cvf[7]= sigma_m2*(dz20f+dz10f)/dz212;
      r.cvf[ 1]=r.cvf[ 6]=r.cvf[ 4]=r.cvf[ 8]=0.0;
      r.cvf[10]=r.cvf[11]=r.cvf[12]=r.cvf[13]=0.0;
      r.cvf[14] = 100.0;
      
      r.cve[0]=r.cve[2]= sigma_m2*(dz20e*dz20e+dz10e*dz10e)/dz212+
	2.0*sigma_m_s2*dz212;
      r.cve[5]=r.cve[9]= 2.0*sigma_m2/dz212+2.0*sigma_m_s2;
      r.cve[3]=r.cve[7]=-sigma_m2*(dz20e+dz10e)/dz212;
      r.cve[ 1]=r.cve[ 6]=r.cve[ 4]=r.cve[ 8]=0.0;
      r.cve[10]=r.cve[11]=r.cve[12]=r.cve[13]=0.0;
      r.cve[14] = 100.0;
      
      r.pf  = r.pe  = pinv;
      
      r.time = 0.0; // see below
      r.dtim = 0.0; // ---"---
      r.chi  = Chi2_trk[itk]; 

      r.fit  = 0;
      r.cmp  = Rsegc::vxd | Rsegc::bit16;

      RPNT rpf, rpe;

      //rpf.rseg = rseg;
      rpf.xf  = r.xf;  rpf.yf  = r.yf;  rpf.zf = r.zf; 
      rpf.txf = r.txf; rpf.tyf = r.tyf; rpf.pf = r.pf;
      for (int i=0; i<15; i++) rpf.cvf[i] = r.cvf[i];
      
      //rpe.rseg = rseg;
      rpe.xf  = r.xe;  rpe.yf  = r.ye;  rpe.zf = r.ze; 
      rpe.txf = r.txe; rpe.tyf = r.tye; rpe.pf = r.pe;
      for (int i=0; i<15; i++) rpe.cvf[i] = r.cve[i];
      
      CatsLocalRefit(&r,nh_trk,hits);


      //      if ((r.chi>100.0)&&(nh_trk[itk]<=6)){
      //	cout<<"Short RSEG with big chi found : "<<r.chi<<" -> rejected"<<endl;
      //      }

      if ((r.chi<=100.0)||(nh_trk[itk]>6)){
	
	ArteTable<RSEG>::push_back( r );
	ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );

	ArtePointer<RPNT> rpntf = ArteTable<RPNT>::push_back( rpf );
	createRelation( rseg, rpntf, &RSEG::rpnt );
      
	ArtePointer<RPNT> rpnte = ArteTable<RPNT>::push_back( rpe );
	createRelation( rseg, rpnte, &RSEG::rpnt ); 

	for (ih=nh+nh_trk[itk]-1; ih>=nh; ih--){
	  ArtePointer<HITB> hitb (hits[ih]);
	  createRelation( hitb, rseg, &RSEG::hitb );
	}
	
	
	int nN=0,nP=0,layer=0;
	int number_of_rhits = 0;
	for( ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	     hitb != rseg->hitb.end(); ++hitb ) {
	  
	  layer=(*hitb)->gede->lay;
	  if (layer==1 || layer==4) nP++;
	  if (layer==2 || layer==3) nN++;
	  
	  rh.tx = (rseg->xe-rseg->xf)/(rseg->ze-rseg->zf);
	  rh.ty = (rseg->ye-rseg->yf)/(rseg->ze-rseg->zf);
	  rh.x  = rseg->xf+((*hitb)->z-rseg->zf)*rh.tx;
	  rh.y  = rseg->yf+((*hitb)->z-rseg->zf)*rh.ty;
	  rh.p  = rseg->pf;
	  rh.q  = 0;
	  ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back( rh );
	  createRelation( *hitb, rhit, &HITB::rhit );
	  number_of_rhits++;
	}
	
	rseg->time = nP;
	rseg->dtim = nN;
	rseg->nhit = nP+nN; //!!!
	
	for (ih=number_of_rhits; ih>=1; ih--){
	  int hitb_num = 0;
	  for( ArteRelationIterator<HITB> hitbIt = rseg->hitb.begin();
	       hitbIt != rseg->hitb.end(); ++hitbIt ) {
	    hitb_num++;
	    if (hitb_num == ih){
	      for( ArteRelationIterator<RHIT> rhitIt = (*hitbIt)->rhit.begin();
		   rhitIt != (*hitbIt)->rhit.end(); ++rhitIt ) {
		ArtePointer<RHIT> rhit(*rhitIt);
		createRelation( rseg, rhit, &RSEG::rhit );	  
	      }
	    }
	  }
	}
	
	
      }
      nh=nh+nh_trk[itk]+1; /* skip (-1): hits of diff. tracks are */
                           /* separated by (-1)                   */

    }

    //cout<<"Cats found "<<Ntk<<" tracks"<<endl;


    return;
  }
  
}
