// *****************************************************************************
//
// source:    daq_mpix_if.C
//
// type:      source code
//
// created:   08 Oct. 2001
//
// author:    Jing-Ge Shiu, Lars Soezueer 
//            email: soezueer@mail.desy.de
//
// purpose:   functions to deliver FED IDs and contents to the class
//            DAQ_MPIX_IF
//            called by DAQ_MC_IF::buildFED and DAQ_UNSPARSE_IF::setSLBs
//
// changes:   
//
// *****************************************************************************
#include "daqif/daq_mu_if.hh"
#include "daqif/daq_mpix_if.hh"

unsigned int DAQ_MPIX_IF::getFED_IDs(unsigned short *buf, unsigned int maxIDs)
{
  unsigned int nfeds = 0;

  if (! ArteTable<DMUX>::size()) {
    MCHitb2Digi();
  }

  nfeds = jsGetDMUFED(3,buf,maxIDs);

  return nfeds;
}

int DAQ_MPIX_IF::getFED(unsigned short fedid, FED_HDR *buf, unsigned int maxSize)
{
  int rc = 0; 
  short cmpid=0;
  LWord dmupos=0;

  if (! ArteTable<DMUX>::size()) {
    MCHitb2Digi();
  }

  cmpid = jsMuFindCmp(fedid, dmupos);

  if (cmpid!=COMP_ID_MPIX) {
    arteMsg << "There is no such FEDid : " << hex << fedid << dec
            << " for MPIX readout." ;
    arteMsg.warn();
    return 1;
  }

  buf->length = 128 + sizeof(FED_HDR) + sizeof(FED_TAIL);
  buf->recid  = RID_NORMAL_COOKED;
  buf->fltevt = 0;
  buf->bxid   = 0;
  buf->compid = cmpid;
  buf->fedid  = fedid;
  
  LWord *p = (unsigned int *)(buf+1);
  LWord *lbuf=0, word=0, nbpos, first=dmupos;
  
  for (int di=0;di<32;di++,p++) {
     nbpos = first+2+di;
     lbuf=(LWord*)(ArtePointer<DMUP>(nbpos)->bits);
     if (jsGetEndian()) word=ESP_SWAP_INT(*lbuf);
     else word = *lbuf;                 
     *p = *lbuf; 
  }

  FED_TAIL *tail= (FED_TAIL*)p;
  tail->fltevt = 0;
  ESP_put_fed_hdr(buf, buf);
  ESP_put_fed_tail(tail, tail);

  return rc;
}

