/***********************************************************************
*
* DGSIL
* written by Siegmund Nowak
*
* This routines transforms the MIMPs in the silicon into the digit
* and hit structures DIGB / HITB foreseen in the reconstruction program
*
*       called from DGEVNT
*
*  Modification:
*	06.04.2001 - added call to silarte_updateGEDEeff() (fg sciacca)
*
***********************************************************************/
#include <math.h>
#include <vector.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "digi/zgdig.hh"
#include "digi/dgnoise.h"
#include "silarte/silarte.hh"

extern "C" {
  void dgsil_(int &);
  void grndm_(float [],int *);
  void granor_(float *, float*);
  void gpoiss_(float*, int*, const int&);
  void sortzv_(int[], int[], int*, int*, int*, int*);
  void mmbook_(int &, const int &);
  void memesc_(int *, int &, const int &, char *, const int);  
}
      
//    check whether there are mimps and if so give out the region in JMIMP

void dgsil_(int &ipl) {   

   static bool firsttime = true;

   static float probex[4] = {0.23, 0.5, 0.22, .05};
   static int   nmx_merg=20; 
   int i0 = 0, i1 = 1, mi1 = -1, nmim = 0, ifmimp =0, noise=0, nsides =0; 
   int *dg_srt = NULL, *hb_srt = NULL, *ind =0, *ind1 = 0; 
   float x_wireplane[3], x_rot[3], rndm[1];

   int ngede = ArteTable<GEDE>::size(); 
   if(ipl == ngede) nsides =1;
   else { 
     if(ArtePointer<GEDE>(ipl)->gcop==ArtePointer<GEDE>(ipl+1)->gcop) nsides=2; 
     else nsides = 1;
   }

   if(firsttime) {
     silarte_updateGEDEeff();
     firsttime = false;
   }

   for (int ip = 0; ip < nsides; ip++) { 
      ArtePointer<GEDE> gd(ipl);
      int nwirpl= gd->nwi[0];           // number of wires in that plane
      zgdig ZGDIG(ipl);
      if(!ip)  nmim = ZGDIG.Get_nmimp(ifmimp);
//    determine the noise
      
      float noise_mean = nwirpl*dgnoise_.noise[gd->cmp-1];
      if(noise_mean < 0.)      noise = int(-noise_mean); 
      else if(noise_mean > 0.) gpoiss_(&noise_mean,&noise,i1);
      if(nmim + noise <= 0) {
         ZGDIG.Set_hit_for_nomimp();
         if(ip+1 < nsides) ipl=ipl+1;
         continue; 
      }
  
      float pit = gd->pit;              // pitch of the chambers
 
//    check whether sorting of hits should be increasing or decreasing
      int lbigr = 0;

      if(gd->lay ==1 || gd->lay == 3) 
                           if(fabs(gd->sxmx) <= fabs(gd->sxmn)) lbigr = 1;
      else if(gd->lay == 2 || gd->lay == 4) 
                           if(fabs(gd->symx) <= fabs(gd->symn)) lbigr = 1;
      
      float *dg_chg = new float[nwirpl]; 
      vector<digx>  *dg_sil = new vector<digx>;
      vector<hitx>  *hb_sil = new vector<hitx>;

      for(int i=0; i<nwirpl; i++ ) dg_chg[i] =0.;
      hitx silhb;
      digx sildg;
      float pr13 = probex[0] + probex[2];

//    loop over the mimps - check whether the mimps are really
//    inside the elementary detector, transform  coordinates
      for(int i=0; i<nmim+noise; i++) {
          int ih = 0, itra = 0;
          float xt =0.; 
          if(i<nmim) {
             ih = i+ifmimp;
             ArtePointer<MIMP> mp(ih);
             float deltaz = gd->zfw[0] - mp->z;
             x_wireplane[0] = mp->x + mp->tx * deltaz;      //swim into the
             x_wireplane[1] = mp->y + mp->ty * deltaz;      //wire plane
             x_wireplane[2] = 0;          

             if(x_wireplane[0] < gd->sxmn || x_wireplane[0] > gd->sxmx ||
                x_wireplane[1] < gd->symn || x_wireplane[1] > gd->symx) {
                int storenr = 8976014 ;   char txt[22];  
                strcpy(txt,"mimp from SVD outside");  
                memesc_(&storenr,i0,AR_ILARGEST, txt,21);
                continue;
             }  
             ZGDIG.Rotate2plane_Z(x_wireplane,x_rot);
             silhb.x = gd->cos*x_rot[0] - gd->sin*x_rot[1]; 
             silhb.mimp = ih;
             grndm_(rndm,&i1);
             if(rndm[0] > gd->eff) continue;
             ArtePointer<MTRA> mt = mp->mtra;
             itra = mt.index();                             // track number
             xt = silhb.x - (gd->xfw[0] - 0.5*pit);
           }
	  else {
             grndm_(rndm,&i1);
             xt = rndm[0]*float(nwirpl*pit); 
	     silhb.x = xt +  (gd->xfw[0] - 0.5*pit);
             silhb.mimp = 0;             
          }
//           find out into which strip interval the mimp is going
          int nt = int(xt/pit);
          if(nt < 0 || nt >= nwirpl) {
             int storenr = 8976014 ;   char txt[22];  
             strcpy(txt,"MIMP from SVD outside");  
             memesc_(&storenr,i0,AR_ILARGEST, txt,21);
             continue;
	  }

//        distribute the charge over neighbouring strips intervals
//        here an interval ranges from -pit/2 to pit/2 around strip position

          int multi=0, istrip=nt;
          float chg = 1., padh[4], sig=0.;
          float xhit = xt - (nt+.5)*pit;  //mimp position in the strip interval
          grndm_(rndm,&i1);
          if(fabs(xhit) < 0.5*gd->pit*pr13) {
//           mimp is likely to lie around the center of a strip interval
//           decide whether the hit multiplicity will be 1 or 3
             if(rndm[0] < probex[0]/pr13) {
                multi = 1;
                padh[0] = chg;       // full charge in this interval
                istrip = nt;
             }
             else {
               multi = 3;
               istrip = nt - 1;
               sig = .424 * pit;
             }
          }
          else {
//           mimp is likely to lie around the boundary of a strip interval
//           decide whether the multiplicity will be 2 or 4
             if(rndm[0] < probex[1]/(1.-pr13)) {
                multi = 2;
                if(xhit < 0.) {
                   istrip = nt-1;
                   padh[1] = 0.5 * chg * (2.-pr13- 2.*fabs(xhit)/pit)/(1.-pr13);
                   padh[0] = chg - padh[1];   //charge is divided between 1/2
                }
                else {
                   istrip = nt;
                   padh[0] = 0.5 * chg * (2.-pr13- 2.*fabs(xhit)/pit)/(1.-pr13);
                   padh[1] = chg - padh[0];
                }
	     }
             else {
                multi = 4;
                istrip = nt-1;
                if(xhit < 0.) istrip = nt-2;
                sig = .566 * pit;
             }
          }

//        now distribute the charge really
          int found = 0;
          for( int n=istrip; n<istrip+multi;n++) {   // loop over the strips
             if(n < 0 || n >= nwirpl) continue;
             if(multi > 2) {
                float expr = 12.649;
                float temp = ((n-nt+.5)*pit-xhit)/sig;
                if(temp < expr) expr = temp;    
                padh[n-istrip] = 0.4*chg*pit/sig * exp(-0.5*expr*expr);
             }
 
             sildg.strip = n;                    // strip number
             sildg.charge = padh[n-istrip];      // charge
             sildg.mimp = ih;                    // mimp number
             sildg.track = itra;        // track number
             dg_sil->push_back(sildg);
             found =1;
             dg_chg[n] = dg_chg[n] + padh[n-istrip]; // add up charge of strip
         }
      if(found) hb_sil->push_back(silhb);
      }

      int ndigde = 0, nhitde = 0,  iflag = 0; 

      ndigde = dg_sil->size();      
      if(ndigde <= 0) goto L99;       

      dg_srt = new int[ndigde];
      ind    = new int[ndigde];
      for(int i=0; i<ndigde; i++) {
	 vector<digx>::iterator mu = dg_sil->begin() + i;
         dg_srt[i] = mu->strip;
      }    

//    sort the digit matrix idigx due to increasing x
      if(lbigr) sortzv_(dg_srt,ind,&ndigde,&mi1,&i0,&i0);
      else      sortzv_(dg_srt,ind,&ndigde,&mi1,&i1,&i0);

//    the next loop cleans the digits and fills digb bank and relations

      for(int ih = 0; ih < ndigde; ih++) {
         vector<digx>::iterator dg = dg_sil->begin() + ind[ih] - 1;
         int n = dg->strip; 
         mmbook_(ArtePointer<DIGB>::getLink().getRef(),1);
	 int nn1 =ArteTable<DIGB>::size() ;
         ArtePointer<DIGB> dgb(nn1);
         if(iflag == 0) {        // first digit in this detector
            gd->dig1 = nn1;
            iflag = 1;
         }
         dgb->id  = 10000*(n+1) + ipl;
         dgb->q   = dg_chg[n];
         if(gd->xfw[2] < 0) dgb->q  = dg_chg[n] ;
         if(dg->mimp)
            createRelation(ArtePointer<MIMP>(dg->mimp),dgb,&MIMP::digb);
         gd->digl = nn1;
      } 
      delete dg_sil;     dg_sil = NULL;
      delete [] dg_srt;  dg_srt = NULL;
      delete [] dg_chg;  dg_chg = NULL;
      delete [] ind;     ind    = NULL;

//    now should be included a code which makes clustering of digits to hits
//    since we dont have it at the moment the hits will be obtained directly
//    from the mimps

//    this loop creates hits from xmimp
       
      nhitde = hb_sil->size();
      if(nhitde <= 0) goto L99;       

      hb_srt = new int[nhitde];
      ind    = new int[nhitde];  
      ind1   = new int[nhitde];
      for(int i=0; i<nhitde;i++) {
	 vector<hitx>::iterator mh = hb_sil->begin() + i;
         hb_srt[i] = int(10000*mh->x);
      }        
      for(int i=0; i<nhitde;i++) ind[i]=0; 
      for(int i=0; i<nhitde;i++) ind1[i]=0;

//    sort the mimp matrix hb_sil due to increasing x
      if(lbigr) sortzv_(hb_srt,ind,&nhitde,&mi1,&i0,&i0);
      else      sortzv_(hb_srt,ind,&nhitde,&mi1,&i1,&i0);
      

      iflag = 0;
      float f, xx;
      for(int ih=0; ih<nhitde; ih++) {
          if(ind1[ih] < 0) continue;
          vector<hitx>::iterator hb = hb_sil->begin() + ind[ih] - 1;  

          mmbook_(ArtePointer<HITB>::getLink().getRef(),1);
          int nn = ArteTable<HITB>::size();
          ArtePointer<HITB> htb(nn);
          if(!ih) gd->hit1 = nn;
          gd->hitl = nn;
          htb->gede = ipl;
          htb->z  = gd->zfw[0];
          int max = 0;           

//        except for the last hit check if next hits are far enough away,
//        if not define the number of hits to be merged and mark them
          if(ih != nhitde-1) {
             for(int j = 0 ; j < nhitde; j++) {
                if(ind[j] < 0) continue;
                vector<hitx>::iterator hbb = hb_sil->begin() + ind[j] - 1;
                if(j != ih) {
                   if(fabs(hbb->x-hb->x) < gd->dod) {
                      max = max + 1;
                      ind1[j] = -ind[ih];
                   }
                }
             }
	  }

//        last mimp or next mimp far enough, take it and translate into a hit
          if(!max) {
             htb->q = 1.;
             if(gd->xfw[2] <0 ) htb->q =-1. ;
             htb->r = gd->acc;
             granor_(&f,&xx);
             htb->x = hb->x + gd->acc*f;
             if(hb->mimp)
                createRelation (ArtePointer<MIMP>(hb->mimp),htb,&MIMP::hitb);
	   }

          else {
             if(max+1 > nmx_merg) continue;
             max = max + 1;
             ind1[ih] = -ind[ih];
//           you have to merge mimps from i to i+max
             float sum = 0.,  sum2 = 0.;
             for(int j = 0; j<nhitde; j++){
                if(ind1[j] == -ind[ih]) {
                   vector<hitx>::iterator hbb = hb_sil->begin() + ind[j] -1;
                   sum = sum + hbb->x;
                   sum2 = sum2 + hbb->x * hbb->x;
                   if(hbb->mimp) 
                   createRelation(ArtePointer<MIMP>(hbb->mimp),htb,&MIMP::hitb);
                }   
             }
             sum = sum /max;
             sum2 = sum2/max;
             htb->q = float(max); 
             if(gd->xfw[2] < 0.) htb->q = - float(max);
             htb->r = sqrt((sum2 - sum*sum)*(sum2 - sum*sum) + gd->acc*gd->acc);
             granor_(&f,&xx);
             htb->x = sum + htb->r*f;
          }
	}

      delete hb_sil;     hb_sil = NULL;
      delete [] hb_srt;  hb_srt = NULL;
      delete [] ind;     ind    = NULL;
      delete [] ind1;    ind1   = NULL; 
      if(ip+1 < nsides ) ipl=ipl+1;
      continue;

L99:  ZGDIG.Set_hit_for_nomimp();
      if(hb_sil != NULL)  {delete hb_sil;      hb_sil = NULL;}
      if(dg_sil != NULL)  {delete dg_sil;      dg_sil = NULL;}
      if(hb_srt != NULL)  {delete [] hb_srt;   hb_srt = NULL;}
      if(ind != NULL)     {delete [] ind;      ind    = NULL;}
      if(ind1 != NULL)    {delete [] ind1;     ind1   = NULL;}
      if(dg_srt != NULL)  {delete [] dg_srt;   dg_srt = NULL;}
      if(dg_chg != NULL)  {delete [] dg_chg;   dg_chg = NULL;}
      if(ip+1 < nsides) ipl=ipl+1;    
    }

}







