
// ******************************************************************
//
// r. spighi january 2000
//
// Filling of histograms
//
// changes:
// 18 May 2000 (AG) removed ECAL_ihists_event() (S. Shuvalov)
// 26 Jul 2000 (AG) corrected warning
//
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ECAL.h"
#include "care/care.hh"
#include "care/EMHitCell.hh"
#include "care/EMClusterTree.hh"
#include "dq/DQ_ClusParticle.hh"
#include "dq/DQ_VecUtil.hh"
#include "arte/ArteUtil.h"
#include "care/ECALDetector.hh"

#include <CLHEP/Alist/AList.h>
#include <CLHEP/Combination/Combiner.h>
#include <vector>
#include <math.h>

//
// function code
//
ClusParticle* bremat(HepAList<ClusParticle> & clusters, unsigned int ind,
		     float xbr, float yc);


int
DQ_ECAL_rec_evnt()
{

  static int n_mon = 0;
  static int num_ev = 0;
  int r_trig = 0, p_trig = 0; 
  int typecal = 0;
  int i;
  int NrOfClusters=0, NrOfTracks=0, NrOfClustersPtGT03=0;
  double TotalClusterEnergy[4] = {0.,0.,0.,0.};
  static double TotalCLUSEnergyave=0;

  double ThisEt;

  float targetrate = faDTARia();       // target rate
  int tcode = faDDAQFCS_tcode();       // tcode
  int trigflag = 0;
  if( tcode > 0 ) trigflag = 900; // fill histos for tcode =0 and #0
  if (trigflag == 900) r_trig += 1;
  else p_trig += 1;

  num_ev++;   

  double EtMax[3] = {0.,0.,0.};
  double XEtMax=0., YEtMax=0.;

  NrOfClusters=ArtePointer<RCCL>::nrow();
  NrOfTracks=ArtePointer<RSEG>::nrow();

  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1000),NrOfClusters,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1005),NrOfTracks,1);

  
  const vector<const EMCluster*> signals = ECALDetector::instance()->signals();
  
  HepAList<ClusParticle>  gammas;
  HepAList<ClusParticle>  clusters;

  float vx = 0.0, vy = 0.0, vz = -5.0;
  
  Hep3Vector vertex(vx,vy,vz);
  
  int neutrals = 0;
  int charged = 0, tracks = 0;
  
  for(ArteTableIterator<RCCL> rccl; rccl.valid(); rccl.next()){
    Hep3Vector shower(rccl->x, rccl->y, rccl->z);
    HepLorentzVector gamma = eval4vec(shower, rccl->esum, vertex);
    ClusParticle* thisclus = new ClusParticle(Particle(gamma), rccl);
    clusters.append(thisclus);

    // add a gamma to the list
    if( gamma.perp() > 0.1) {  // the Pt
      gammas.append(thisclus);
    }
    
    ThisEt = 
      rccl->esum*sqrt(rccl->x*rccl->x + rccl->y*rccl->y)/
      sqrt(rccl->x*rccl->x + rccl->y*rccl->y + rccl->z*rccl->z);
    
      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1015+trigflag),rccl->esum,1); //En distr
      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1016+trigflag),ThisEt,1); //Et distr

    if(ThisEt > 0.3) NrOfClustersPtGT03++;
    
    if((rccl->flag)%16 == 5) typecal = 1;   // Inner
    if((rccl->flag)%16 == 2) typecal = 2;   // Middle
    if((rccl->flag)%16 == 1) typecal = 3;   // Outer

    TotalClusterEnergy[0]+= rccl->esum;
    if(typecal != 0) TotalClusterEnergy[typecal]+= rccl->esum;
    TotalCLUSEnergyave+=rccl->esum;

    // search for ThisET max, second and third
      if(ThisEt>EtMax[0])
        {
          EtMax[2]=EtMax[1];
          EtMax[1]=EtMax[0];
          EtMax[0]=ThisEt;
          XEtMax=rccl->x;
          YEtMax=rccl->y;
        }
      else if(ThisEt>EtMax[1])
        {
          EtMax[2]=EtMax[1];
          EtMax[1]=ThisEt;
        }
      else if(ThisEt>EtMax[2])
        {
          EtMax[2]=ThisEt;
        }

      tracks = 0;
      for(ArteRelationIterator<RSEG> relIter((rccl->rseg).begin()); 
	  relIter!=(rccl->rseg).end(); relIter++)     tracks++;

      if(tracks == 0) 	neutrals++;
      else      	charged++;
      

    // occupancy and energy density

      float weight = (rccl->x*rccl->x+rccl->y*rccl->y)/targetrate;

      rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1100+typecal),rccl->x,rccl->y,weight);
      rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1110+typecal),
		rccl->x,rccl->y,rccl->esum);

      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1002),rccl->wid,1); // nr cell

  }  // end loop on RCCL
  

  // Statistic in the first bin of histogram  

  rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1101),-78.,-44.,1.);
  rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1102),-223.,-133.,1.);
  rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1103),-312.,-234.,1.);

  rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1111),-78.,-44.,1.);
  rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1112),-223.,-133.,1.);
  rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,1113),-312.,-234.,1.);

  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1001),NrOfClustersPtGT03,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1003),neutrals,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1004),charged,1);


  for(i=0;i<4;i++){
    rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1010+trigflag+i),TotalClusterEnergy[i],1);
  }  

  for(i=0;i<3;i++){
    rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1021+i),EtMax[i],1);
  }  

  if (num_ev>0 && num_ev%100 == 0) {
    n_mon += 1;
    if(n_mon > 999) n_mon = 0;
    rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1009+trigflag),
	    float(n_mon),TotalCLUSEnergyave/100.);
    TotalCLUSEnergyave = 0;	
  }



  // invariant mass of p0
  int len = gammas.length();
  if( len>=2){ 
    HepCombiner<ClusParticle> TwoGammas(&gammas, &gammas);
    HepAList<ClusParticle>   Pions;
    ClusParticle * Pi0;

    Pi0 = TwoGammas();
    while ( Pi0!=0 ) {
      ClusParticle * gamma1 = Pi0->child(0);
      ClusParticle * gamma2 = Pi0->child(1);
      int typecal_g[2] = {0,0};
      if(gamma1->ifed() < 100)      typecal_g[0] = 1;
      else if(gamma1->ifed() < 180) typecal_g[0] = 2;
      else if(gamma1->ifed() > 179) typecal_g[0] = 3;

      if(gamma2->ifed() < 100)      typecal_g[1] = 1;
      else if(gamma2->ifed() < 180) typecal_g[1] = 2;
      else if(gamma2->ifed() > 179) typecal_g[1] = 3;

      if( gamma1->ptc() > 0.3 && gamma2->ptc() > 0.3) {
	rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1210),Pi0->mag(),1);
	if(typecal_g[0] == typecal_g[1] && typecal_g[0] != 0)
	  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1210+typecal_g[0]),Pi0->mag(),1);
	
      }
      if(gamma1->ptc() > 0.5 && gamma2->ptc() > 0.5) {
	rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1220),Pi0->mag(),1);
	if(typecal_g[0] == typecal_g[1] && typecal_g[0] != 0)
	  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1220+typecal_g[0]),
		  Pi0->mag(),1);
      }

      Pi0 = TwoGammas();
    }
  }

  // Charged Invariant Mass

  float br_pos  = 585.;

  HepAList<ClusParticle>  allelectrons; // 1 or 2 clusters (double matches)
  HepAList<ClusParticle>  allpositrons; // 1 or 2 clusters (double matches)

  float eng =0.0;

  // Loop over the RCCL clusters:
  // 1) Look for BR position ( +/- X side) and get e+/e- directions
  // 2) fill the e+/e- Particle list
  // 3) add the BR energy (if present) to e+/e- momentum
  // 4) fill the e+/e- Corrected Particle list
  // 5) Perform the ivariant mass calculation
  //
 
  float xpbr = 0.0;
  float xmbr = 0.0;
  float yc = 0.0;
  unsigned int ind = 0;

  HepAListIterator<ClusParticle> clusiter(gammas);  //gammas
  ClusParticle* aclus, *newele,*newpos, *newpart;

  while ( (aclus = clusiter()) ){
    if( aclus->perp()>0.5 ){
      ArtePointer<RCCL> rccl = aclus->whichBank();
      eng = aclus->t();
      ind = rccl.index();
      //
      // Calculation of the BR impact point according to the P/T algorithm
      //
      //  POSITIVE
      xpbr = rccl->x - br_pos/eng;
      yc = rccl->y;
      Hep3Vector showerp(xpbr, yc, rccl->z);
      HepLorentzVector positron = eval4vec(showerp, eng, vertex);
      // create an e+ particle
      newpos = new ClusParticle(Particle(positron), rccl);
      // append the original cluster as a child
      newpos->addChild(aclus);
      allpositrons.append(newpos);
      
      // NEGATIVE
      xmbr = rccl->x + br_pos/eng;
      Hep3Vector showern(xmbr, yc, rccl->z);
      HepLorentzVector electron = eval4vec(showern, eng, vertex);
      // create an e- particle
      newele = new ClusParticle(Particle(electron), rccl);
      // append the original cluster as a child
      newele->addChild(aclus);
      allelectrons.append(newele);
      
      //
      // Look for BR clusters positive side
      //
      
      ClusParticle* BRclus = bremat(clusters, ind, xpbr, yc); //gammas
      
      if( BRclus!=0){
	float engt =  eng + BRclus->t();
	HepLorentzVector cor_positron = eval4vec(showerp, engt, vertex);
	// add a e+ to the list
	newpart = new ClusParticle(Particle(cor_positron), rccl);
	// append the original cluster as a child
	newpart->addChild(aclus);
	// Find BR cluster in the ClusParticle cluster list
	newpart->addChild(BRclus);
	allpositrons.append(newpart);
      } 
      
      //
      // Look for BR clusters negative side
      //
      
      BRclus = bremat(clusters, ind, xmbr, yc);
      
      if( BRclus!=0){
	float engt =  eng + BRclus->t();      
	HepLorentzVector cor_electron = eval4vec(showern, engt, vertex);
	// add a e+ to the list
	newpart = new ClusParticle(Particle(cor_electron), rccl);
	// append the original cluster as a child
	newpart->addChild(aclus);
	// Find BR cluster in the ClusParticle cluster list
	newpart->addChild(BRclus);
	allelectrons.append(newpart);
      } 
    }
  }

  HepCombiner<ClusParticle> JPsis(&allelectrons, &allpositrons);
  ClusParticle * JPsi;
  JPsi = JPsis();
  while ( JPsi!=0 ) {
    ClusParticle* ele = JPsi->child(0);
    ClusParticle* pos = JPsi->child(1);
 
    int typecal_g[2] = {0,0};
    if(ele->ifed() < 100)      typecal_g[0] = 1;
    else if(ele->ifed() < 180) typecal_g[0] = 2;
    else if(ele->ifed() > 179) typecal_g[0] = 3;

    if(pos->ifed() < 100)      typecal_g[1] = 1;
    else if(pos->ifed() < 180) typecal_g[1] = 2;
    else if(pos->ifed() > 179) typecal_g[1] = 3;

    if(ele->ptc() > 0.7 && pos->ptc() > 0.7) {
      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1200),JPsi->mag(),1);
      
      if(typecal_g[0] != 0 && typecal_g[1] != 0){
	if(typecal_g[0] == 3 || typecal_g[1] == 3) 
	  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1203),JPsi->mag(),1);
	else if(typecal_g[0] == 2 || typecal_g[1] == 2) 
	  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1202),JPsi->mag(),1);
	else 
	  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1201),JPsi->mag(),1);
      }
    }
    
    JPsi = JPsis(); // Take the next combination

  }


  HepAListDeleteAll(allelectrons);
  HepAListDeleteAll(allpositrons);
  HepAListDeleteAll(clusters);

  return 0; // return error code (0 = okay)
}



ClusParticle* bremat(HepAList<ClusParticle> & clusters, unsigned int ind, 
		     float xbr, float yc){
  ClusParticle* brem=0;
  EMHitCell* hitcell = 
    ECALDetector::instance()->TopCluster().whichHitCell(xbr, yc);
  if( hitcell!=0){
    ArtePointer<HITC> hitc = hitcell->whichBank();
    ArteRelationIterator<RCCL> relrccl((hitc->rccl).begin());
    if(relrccl!=(hitc->rccl).end()){
      float dmin = 100000;
      ArtePointer<RCCL> rcclhit(0);
      for( ; relrccl!=(hitc->rccl).end(); ++relrccl){
	ArtePointer<RCCL> rccli(relrccl);
	float dist= sqrt( (rccli->x-xbr)*(rccli->x-xbr)+
			  (rccli->y-yc)*(rccli->y-yc));
	if( dist<dmin && rccli.index()!=ind ){
	  dmin = dist;
	  rcclhit = rccli;
	}
      }
      if( rcclhit.index()>0 && dmin<50.0){	  
	int ind_br = rcclhit.index();
	if(ind_br != int(ind) ) {
	  brem = clusters[rcclhit.index()-1];
	} 
      }
    }
  }
  return brem;
}




