// ****************************************************************
// source:    DQ_ITR_rec_evnt.C
// author:    Sonja Wunderlich
// called by: DQ_evnt()
//
// changes:
// 26 Jun 2000 (AG) creation
// 11/24 count itr segments, vds&itr segments, relation vds&itr/itr
// 01/16  count hits on rsegs per half station
// 01/16 (SW) residuals between ITR segments and ITR hits per HSL
// 01/25 (SW) change histogramnames (tracks = RSEGS)
// 01/30 (SW) if (targetrate < 1.0) targetrate = 1.0; realy dirty!!!!
// 2/5   (SW) 2 dim residual plot with large window
// 2/5   (SW) residuals per SL plot now with small window
// 2/8   (SW) change numbering of SLindex now 0 = +1; 1 = -1 ...
// ******************************************************************

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <map>

#include "rhp.h"
#include "hbook.h"
#include "event.h"

#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "auto/auto.hh"                   //what is these? 
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ITR.h"

extern "C" {
 void rkmain_(const int    & icovar,
              const double & zout,
              const double & fstep,
              const double * parameters,
              const double * prec,
              const double * p_track,
              const double * derivatives,
              const int    & ierror);
};  

int DQ_ITR_TraceTRACK(float z, float *pin, float *p);
void DQ_ITR_residuals();

// require at least HIT_CUT InnerTracker hits on a track to count.
// otherwise we mainly count otr tracks.

const int  HIT_CUT  = 12;                  
static int ITR_VDS_rtrasegs;  
static float ITRhitsonrsegSL[20];
static float ITRhitsonrsegQU[40];
static  int ITRhitsonrseg;  
static int event_count = 0;

int DQ_ITR_rec_evnt(){

#if defined (DQ_ITR_COMPILE) 
if(event_count == 1)
  cout<< "EEEEEEEEEEEEEEEEEEEEEEEEE DQ_ITR_rec_event.C EEEEEEEEEEEEEEEEEE"<<endl;
#endif

  ARTE_mark( "DQ_ITR_hit_evnt 0", 0 );

  int ITR_longrsegs;

  int ITR_rtrasegs, VDS_rtrasegs;
  int ITR_notmatched = 0, ITR_allsegs = 0;

  float targetrate;                    // target rate
  float norm;                          //normalize on target rate

  int   itrseg_idx;
  int SLindex,SLindex2;
  map<int, int> matchedITRsegs;
  
  event_count ++;                        // event counter 
  
  targetrate  = faDTARia();            // get target rate from ARTE
  
  if (targetrate > 1.0)
    norm = 1./(float)targetrate;  
  else 
    norm = 1.0;
  
   // initialize counters 
  ITR_VDS_rtrasegs  = 0;
  ITR_longrsegs = 0;

  // count RTRAs which have a match between ITR and VDS segments
  for (ArteTableIterator<RTRA> rtrait; rtrait.valid(); rtrait.next()) {
    ArtePointer<RTRA> rtra(*rtrait);
    
    ITR_rtrasegs = 0;
    VDS_rtrasegs = 0;
    
    for (ArteRelationIterator<RSEG> rsegit = rtra->rseg.begin(); 
	 rsegit!=rtra->rseg.end(); ++rsegit) {
      ArtePointer<RSEG> rseg (*rsegit);
      
      if((rseg->fit==0) && (rseg->cmp & (Rsegc::vxd | Rsegc::sil))) {
	VDS_rtrasegs++;
      }
      
      if((rseg->fit == 0) && (rseg -> cmp & Rsegc::msg)) {
	ITR_rtrasegs++;	
	itrseg_idx = rseg.index();
      } 
    }
    
    if (ITR_rtrasegs > 0 && VDS_rtrasegs > 0) {
      ITR_VDS_rtrasegs ++;
      matchedITRsegs [itrseg_idx] = 1;
      
#if defined (DQ_ITR_DEBUG)
      cout << "VDS-ITR RTRA " << rtra.index() 
	   << "\tzf = "   << rtra->zf 
	   << "\tp  = "   << (rtra->pf != 0.0 ? 1./rtra->pf : 0.) 
	   << "\thsi= "   << rtra->hsi
	   << "\thin= "   << rtra->hin 
	   << "\tchi2 = " << rtra->chi 
	   << endl ;
#endif
    }
  }
  
  // count long track segments (RSEGs) with more than HIT_CUT ITR hits in them
  for (ArteTableIterator<RSEG> rsegit; rsegit.valid(); rsegit.next())  { 
    ArtePointer<RSEG> rseg (*rsegit);  
    
    if ( (rseg->fit == 0) && (rseg->cmp & Rsegc:: msg)) { 
 
      ITRhitsonrseg = 0;
      ITR_allsegs ++;      
      
      // count ITR hits on this segment
      for (SLindex = 0; SLindex < k_numHalfStation; SLindex++)
	ITRhitsonrsegSL[SLindex] = 0.;
      for(SLindex2 = 0; SLindex2 < 2 * k_numHalfStation; SLindex2++)
       	ITRhitsonrsegQU[SLindex2] = 0.;

      for( ArteRelationIterator<HITB> hitbit = rseg->hitb.begin(); 
	   hitbit != rseg->hitb.end(); hitbit++ ) {
	ArtePointer<HITB> hitb(*hitbit);
	ArtePointer<GEDE> gede = hitb->gede;
	ArtePointer<GESL> gesl(gede->gesl);
	
	if( gesl->cmp == 3 ) {       	  
	 SLindex = 2*(gede->gesl->nsl - 1);            // we have 20 half-stations   
	if(gede->sec > 4)                         // +x gets the odd index       
	  SLindex++;
	  
	ITRhitsonrseg ++;
	ITRhitsonrsegSL[SLindex]++;
	
	SLindex2 = 4*(gede->gesl->nsl - 1);        // we have 40 sectors
	if (gede ->sec == 2 || gede -> sec == 1 )               // index : ++ 0  4 ...
	  SLindex2 +=0;								   
	else if(gede -> sec == 4 || gede -> sec ==3)            //         +- 1  5 
	  SLindex2 += 1; 								   
	else if (gede -> sec == 5 || gede -> sec ==6)           //         -- 2  6 
	  SLindex2 += 2;								   
	else if (gede -> sec == 7 || gede -> sec ==8)           //         -+ 3  7   
	  SLindex2 += 3;
	else 
	  cout<<"ITR_REC: Errow :segment not found "<< endl;

	ITRhitsonrsegQU[SLindex2]++;
	
	}
      }
      
      if(ITRhitsonrseg >= HIT_CUT) 
	ITR_longrsegs ++; 

      // did we NOT see this segment before as part of a matched RTRA?
      if(matchedITRsegs.find(rseg.index()) == matchedITRsegs.end()) 
	ITR_notmatched ++;
    }
  }
  
#if defined (DQ_ITR_DEBUG)
  cout << " === DQ_ITR_rec_evnt " << endl
       << "     Total Number of ITR Segments :\t" << ITR_allsegs << endl
       << "     Number of matched ITRsegs    :\t" << matchedITRsegs.size() << endl
       << "     Number of UNmatched ITRsegs  :\t" << ITR_notmatched << endl
       << "     Number of ITRsegs > HIT_CUT  :\t" << ITR_longrsegs << endl 
       << "     Number of ITR-VDS matches    :\t" << ITR_VDS_rtrasegs << endl  ;
#endif
  // 1. long ITR segs per event and target rate >hit_cut
  DQ_ITR_my_hf1_fill(h_RSEGPerEvent, LVL_REC,
		     ((float)ITR_longrsegs *(float)norm), 1.);
  // 2. ITR-hits on ITR segments per event and target rate  
  DQ_ITR_my_hf1_fill(h_HitsOnRSEGPerEvent, LVL_REC,
		     (float) ITRhitsonrseg, 1.);
  
  for( SLindex = 0; SLindex < 2 * k_numHalfStation; SLindex++)
    DQ_ITR_my_hf2_fill (h_HitsOnRSEGQU_all, LVL_REC,SLindex, 
			(float)ITRhitsonrsegQU[SLindex], 1.);
  
  DQ_ITR_my_hf1_fill(h_ITR_numRSEG, LVL_REC,
	 	     (float) (ITR_allsegs*(float)norm), 1.);
  
   // no double counting of RSEGs which are multiply matched !
  DQ_ITR_my_hf1_fill(h_ITR_matchedRSEG,LVL_REC,
	 	     (float)((float)matchedITRsegs.size()*(float)norm), 1.);
  
  // ratio of matched segments:  if no segments at all were found,
  // populate histogram bin < 0
  
  float matchRatio = -0.01;
  if (ITR_allsegs > 0) 
    matchRatio = (float)matchedITRsegs.size() / (float) ITR_allsegs;
  
  DQ_ITR_my_hf1_fill (h_ITR_matchRatio, LVL_REC,(float) matchRatio, 1.);
  
  // plotting residual histograms between ITR segments and hits
  
  DQ_ITR_residuals();
  
  ARTE_mark( "DQ_ITR_rec_evnt 1", 0 );
  return GRR_OK; 
}

//         Residuals between ITR segments and ITR hits
//                       T.Z. 16/01/2001

void DQ_ITR_residuals(){
  
  for (ArteTableIterator<RSEG> rseg1it; rseg1it.valid(); rseg1it.next()) {
    ArtePointer<RSEG> rseg1 (*rseg1it);
    
    if ( (rseg1->fit) || (rseg1->cmp & Rsegc::bitsign) )
      continue;
    
    //reseg with vds, otr and itr
    if (rseg1->cmp & Rsegc::patt || rseg1->cmp & Rsegc::magt 
	|| rseg1->cmp & Rsegc::msg) {
      
      int itrhit=0;
      
      for (ArteRelationIterator<RHIT> rhitit = rseg1->rhit.begin(); 
	   rhitit!=rseg1->rhit.end(); rhitit++) {
	ArtePointer<RHIT> rhit(*rhitit);
        ArtePointer<GEDE> gede(rhit->hitb->gede);

	if (gede->cmp==3 ) 
	  itrhit++;
      }
      if ( itrhit < 10 )                        // use only RSEGs with >= 10 ITR hits
	continue;
      
      for ( ArteTableIterator<HITB> hitbit; hitbit.valid(); hitbit.next() ) {
	ArtePointer<HITB> hitb(*hitbit);
	ArtePointer<GEDE> gede(hitb->gede);
	ArtePointer<GESL> gesl(gede->gesl); 
	
	if (hitb->z < rseg1->zf || hitb->z > rseg1->ze ) 
	  continue;
	
	if (gede->cmp == 3 && gesl -> nsl>0 && gesl->nsl<11) {
	  
	  float gasgap=(gede->zmax-gede->zmin)/2;
	  float trackx, tracky;
	  float pinrtra[6], poutrtra[6];
	  
	  // calculate trace track through magnetic field (zf<650)
	  if ( rseg1->zf < 650 ){
	    
	    pinrtra[0] = rseg1->zf;
	    pinrtra[1] = rseg1->xf;
	    pinrtra[2] = rseg1->yf;
	    pinrtra[3] = rseg1->txf;
	    pinrtra[4] = rseg1->tyf;
	    pinrtra[5] = rseg1->pf;
	    
	    //int rc = 
	      DQ_ITR_TraceTRACK(gede->zmin+gasgap, pinrtra, poutrtra);
	    
	    trackx = poutrtra[1];
	    tracky = poutrtra[2];
	  }
	  else {
	    // first point of rseg outside magnetic field linear extrapolation
            trackx = rseg1->xf+rseg1->txf*(gede->zmin+gasgap-rseg1->zf);
	    tracky = rseg1->yf+rseg1->tyf*(gede->zmin+gasgap-rseg1->zf);
	  }
	  
	  float uhitx, uhity;
	  uhitx = trackx*gede->rot[0]-tracky*gede->rot[1];
	  uhity = trackx*gede->rot[1]+tracky*gede->rot[0];
	  
	  if ( uhitx > gede->sxmn && uhitx < gede->sxmx && 
	       uhity > gede->symn && uhity < gede->symx) {
	    
#if defined(DQ_ITR_CONTROL)
	    int hi_id, chambernr;
	    hi_id=100*gesl->nsl+10*gede->lay;
	    
	    if (gede->sec == 1 || gede->sec == 2) hi_id = hi_id + 2;
	    else if (gede->sec == 3 || gede->sec == 4) 
	      hi_id = hi_id + 4;
	    else if (gede->sec == 5 || gede->sec == 6) 
	      hi_id = hi_id + 5;
	    else if (gede->sec == 7 || gede->sec == 8) 
	      hi_id = hi_id + 7;
	    else
	      cout<<"ITR_rec_event: segment not found \t"<< gede->sec<<endl;
	    
	    if( DQ_ITR_isKnown(hi_id)!= 0)
	      chambernr = (DQ_ITR_chamberIndex(hi_id) - 1); 
	    else
	      cout<<" chambernr not known  Chamber: "<<hi_id<<endl;

	    hf1((h_ResidualCham+hi_id),uhitx-hitb->x,1.);
	    
#endif	    
	    
	    int SLindex;
	    SLindex = 2*(gede->gesl->nsl - 1);       /* we have 10 stations         */
	    if (gede->sec > 4)                         
	      SLindex++;

	    DQ_ITR_my_hf2_fill (h_Residual, LVL_REC, SLindex, uhitx - hitb->x,1.);	    
	    DQ_ITR_my_hf2_fill (h_Residualfine, LVL_REC, SLindex, uhitx - hitb->x,1.);	    

	  }
	} 
      }
    }
  }
}

int DQ_ITR_TraceTRACK(float z, float * pin, float * p) {
  
  double       fstep;
  const int    icovar=2;
  int          ierror;
  double       prec[4];
  double       derivatives[5][5]={};
  double       pTrack[6], pOut[6];
  double       zout = double(z);
  
  pTrack[0] = double(pin[0]);
  pTrack[1] = double(pin[1]);
  pTrack[2] = double(pin[2]);
  pTrack[3] = double(pin[3]);
  pTrack[4] = double(pin[4]);
  pTrack[5] = double(pin[5]);
  
  prec[0]   = 0.001/fabs(zout-pTrack[0]);
  prec[1]   = prec[0];
  prec[2]   = 0.0001/fabs(zout-pTrack[0]);
  prec[3]   = prec[2];
  
  if (zout > pTrack[0] ) 
    fstep =  50.;
  else 
    fstep = -50.;
  
  if (zout != pTrack[0]){
    
    rkmain_(icovar, zout, fstep, pTrack, prec, pOut,
	    & derivatives[0][0], ierror);

    for (int i=0; i<6 ;i++)  
      p[i] = float(pOut[i]);
    
    
  } else {
    for (int i=0; i<6 ;i++)  
      p[i] = pin[i];
      
    ierror = 1;
  }
  return ierror;
}       
