//#include <list.h>
//#include <CLHEP/SymMatrixD.h>
//#include <CLHEP/VectorD.h>


#define maxSegments 3
#define maxTracks 5000
#define Nu 5
#define Na 1

const int indexARTE[5][5] = { {0, 1,  3, 6,10},
                              {1, 2,  4, 7,11},
                              {3, 4,  5, 8,12},
                              {6, 7,  8, 9,13},
                              {10,11,12,13,14}};
const int indexMATRIX[5][5] = { {0,1, 2, 3, 4},
                                {1,5, 6, 7, 8},
                                {2,6, 9,10,11},
                                {3,7,10,12,13},
                                {4,8,11,13,14}};

//extern double AlignParameter[Nu];
//extern double AlignBackground[Na];
//double AlignParameter[Nu]  = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
//double AlignBackground[Na] = {0.0};
//extern double AlignParameter[maxSegments][6]; // x, y, tx, ty, phi, z0

//-------------------------------------------------------------------------
//
//  class GlobalAlignTrack
//
//-------------------------------------------------------------------------


class GlobalAlignTrack{


public:
                GlobalAlignTrack();
                GlobalAlignTrack(ArtePointer<RTRA>);
                GlobalAlignTrack(ArtePointer<RSEG>);
                GlobalAlignTrack(float *);
   int          AddECAL(ArtePointer<RCCL>);
   int          AddRSEG(ArtePointer<RSEG>);
   int          AddVDS (ArtePointer<RSEG>);
   int          AddRICH(ArtePointer<RSEG>);
   void         FitStep();
   void         print();

   int          AddSegment(const int , const double , const double *, const double *);

   void         dp(int, int, double *);

   double       ndL(int);
   double       ndL2(int,int);

   int          fitSegment;                      // segment for the fit 
   int          SegFilled  [maxSegments];

   double       momentum();
   double       zvalue(int);
   double       ztrack();
   double       xtrack();
   double       ytrack();
   double       txtrack();
   double       tytrack();
   double       ptrack();

   double       Likelihood;
   double       LikelihoodSegments;
   double       LikelihoodBackground;


   double       dqSeg      [4];     // qSegTrans - qTrack
   double       WSeg       [maxSegments][4][4];  // 

   double       dlnLdu     [Nu];     // d -lnL / du
   double       dlnLda     [Na];     // d -lnL / da
   double       dlnLduu    [Nu][Nu]; // d -lnL / du du
   double       dlnLdua    [Nu][Na]; // d -lnL / du da
   double       dlnLdaa    [Na][Na]; // d -lnL / da da

private:


                         // z, q, W

   void         AlignmentTransformation();
   void         calculateLikelihood();
   void         calculateDerivatives();


   double       qSeg       [maxSegments][5];     // x, y, tx, ty, z
   double       qSegTrans               [5];     // x, y, tx, ty, z
   double       WSegTrans               [4][4];  // 
   double       detWSeg    [maxSegments];        // 
   double       qTrack     [maxSegments][4];     // x, y, tx, ty
   double       pTrack[6];                       // z, x, y, tx, ty, q/p
   double       dqW                     [4];     // W * dq


};

//-------------------------------------------------------------------------
//
//  class GlobalAlignment
//
//-------------------------------------------------------------------------


class GlobalAlignment{

friend class GlobalAlignTrack;

public:
                GlobalAlignment();
   void         AddTrack(ArtePointer<RTRA>);
   void         ReadTrack(GlobalAlignTrack);
   void         Fit(int);
   void         Print();
   void         Clear();
   double       error(const int,const int);


   int              nTracks;
   GlobalAlignTrack TrackList[maxTracks];

private:

   int              fitSegment;

   void             Search();
   int              FitIteration();
   void             calculateLikelihood();
   void             SumDerivatives();
   int              newParameters();


   double           lnLikelihood;
   double           Norm;
   double           parameter[Na+Nu];
   double           dlnLdp [Na+Nu];
   double           dlnLdpp[Na+Nu][Na+Nu];

   int              SegCoorFilled[maxSegments][4];

};

