/***********************************************************************
* all action routines called from KUIP 
* written by Siegmund Nowak 01/98
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "gean/gctmed.h"
#include "gean/zgbeam.hh"
#include "gean/zgpict.hh"
#include "gean/ZGDETV.h"
#include "gean/ZGESL.h"
#include "gean/ZGTRDE.h"

extern "C" {
  void kagean();
  void kabeam();
  void kamagn();
  void kaspirl();
  void katrace();
  void katurb();
  void kawire();
  void kapict();
  void kageov();
  char * ku_getc();
  double ku_getr(); 
  char* ku_path();
  void hginit_(int *);
  int  matnum_(char*,int);
  void iuwk_(int &, int &);
  void  picotr_();  
  void  picric_();  void  pictrd_();   void  piccal_();   void  picmuo_();
}


void kagean(){   
/* Description: Switch Geant on  
              - command GEAN         - normal GEAN run 
              - command GEOM_INSTALL - GEOMETRY installation with 
                                       output of ARTE and GEANT banks
              - Command GEAN READ    - GEOMETRY is taken from ARTE
                                       and GEAN                   */
   extern int Gean_Status;
   char chpatl[32], ch[5];
   strncpy(chpatl,strrchr(ku_path(),'/')+1,4); 
   for (int i =0; i<4; i++) chpatl[i] = toupper(chpatl[i]); 
   if(!strncmp(chpatl,"GEOM",4)) Gean_Status = 1; 
   else if(!strncmp(chpatl,"GEAN",4)) { 
      Gean_Status = 2;
      strncpy(ch,ku_getc(),4);    
      if(!strncmp(ch,"READ",4)) Gean_Status = 3;
   } 
   if (!arteflag_.geaninit)  hginit_(&Gean_Status);
   arteflag_.geaninit = 1;
}



// This routines switches on the generation time smearing
void kabeam() {
  char type[6];
  zgbeam ZGB;
  strncpy(type,ku_getc(),5);  
  if(!strncmp(type,"TIME",4)) {     
     if(!strncmp(ku_getc(),"ON",2)) {
        float sigb = ku_getr() *1.E-9;
        float meanb =  ku_getr() *1.E-9;
        ZGB.Set_time(1, sigb , meanb); 
        arteMsg << "time generation is switched ON";
        arteMsg.warn();
     }
  }
  else if(!strncmp(type,"SHIFT",5)) {
     if(!strncmp(ku_getc(),"OFF",2)){
        ZGB.Set_hdir(0);
        arteMsg << "beam shift is switched OFF";
        arteMsg.warn();
     }         
  }
          
}
/***********************************************************************/

// This routines switches on/off the Magnet
void kamagn() {
  char magnfl [4];
  strncpy(magnfl,ku_getc(),3);
  if(!strncmp(magnfl,"ON",2)) {
    zgdetv_.DETV[LGESL_FLD-1] = 'L';
    strncpy(magnfl,ku_getc(),3);
    if(!strncmp(magnfl,"REV",3))  zgdetv_.DETV[LGESL_FLD-1] = 'R';   
  }
  else if(!strncmp(magnfl,"OFF",3))  zgdetv_.DETV[LGESL_FLD-1] = 'S';
  else {   
    arteMsg << "check card magn with command "  <<  magnfl; 
    arteMsg.warn();  
  }
}

/***********************************************************************/ 

// This routine reads in the maximal spiraling rotation angle   
void kaspirl() {
  extern float nspirl_angle;
  nspirl_angle = float(ku_getr()); 
}

/***************************************************************/

// This routine selects the correct tracing for photons from RICH and TRD
void katrace() {
  char buffer[13],  patl[13];
  strcpy(patl, strrchr(ku_path(),'/') +1);
  strcpy(buffer, ku_getc());
    
  if(!strncmp(buffer,"ON",2) || !strncmp(buffer,"1",1)) {

    if(!strncmp(patl,"TRD",3)) {
      if(zgdetv_.DETV[LGESL_TRSL-1] != 'L') {
         arteMsg << "katrace: you want to have TRD ON, but GEOM is short";
         arteMsg.warn();
         arteMsg << "therefore only standard hits will be produced in TRD"; 
         arteMsg.warn();
         trdsw_.itrswt = 0;
      }
      else trdsw_.itrswt =1;
    }
    else if(!strncmp(patl,"CKOV",4)) gctlit_.itckov = 1;
  }            

  else if (!strncmp("OF",buffer,2) || !strncmp(buffer,"0",1)) {
    if     (!strncmp(patl,"TRD",3))   trdsw_.itrswt = 0;
    else if(!strncmp(patl,"CKOV",4))  gctlit_.itckov = 0; 
  }
  else {   
    arteMsg << "katrace: check command " << buffer ;
    arteMsg.error();
  }
}

/***************************************************************/

//  This routine reads in the switch for fast muon tracing
void katurb() {
   extern int muon_turbfl; 
   if(strncmp(ku_getc(),"ON",2))muon_turbfl =1;
}
/***************************************************************/

void kawire() {
  char buffer[4], tag[13], mat[12];
  strcpy(buffer, ku_getc());
  if(!strncmp(buffer,"MAT",2)) {
    memset(mat,' ',12);
    strcpy(tag, ku_getc());
    char * ptr =  strtok(tag,"/");
    if(!strcmp(ptr,"ALUMINIUM")      || !strcmp(ptr,"AL")) 
                                         strcpy(mat,"ALUMINIUM");
    else if(!strcmp(ptr,"CARBON")    || !strcmp(ptr,"C"))  
                                         strcpy(mat,"CARBON");
    else if(!strcmp(ptr,"COPPER")    || !strcmp(ptr,"CU")) 
                                         strcpy(mat,"COPPER");
    else if(!strncmp(ptr,"TITAN",5)  || !strcmp(ptr,"TI"))     
                                         strcpy(mat,"TITANIUM");
    else if(!strcmp(ptr,"TUNGSTEN")  || !strcmp(ptr,"W"))
                                         strcpy(mat,"TUNGSTEN");
    else if(!strcmp(ptr,"BERYLLIUM") || !strcmp(ptr,"BE"))
                                         strcpy(mat,"BERYLLIUM");
    else if(!strcmp(ptr,"PALLADIUM") || !strcmp(ptr,"PD"))
                                         strcpy(mat,"PALLADIUM");
    else {
      arteMsg << "kawire: target material " << ptr << " not foreseen";
      arteMsg.error();
    }
    int imat = matnum_(mat,strlen(mat)); 
    ArteTableIterator<GTAR> gt;
    for(; gt.valid(); gt.next()){
       if(tag[0] != '/') { 
         if(gt->cmp != imat) gt->eff = 0.;
       }
       else {
         if(gt->cmp == imat) gt->eff = 0.;
       }
    }	   
  }            
  else if (!strncmp("ALL",&buffer[1],3)) {
    float  eff = atof(ku_getc());
    ArteTableIterator<GTAR> gt;
    for(; gt.valid(); gt.next()){ 
       if(!strncmp(buffer,gt->name,1)) gt->eff = eff;
    }
  }
  else {   
    ArteTableIterator<GTAR> gt;
    int found = 0; 
    for(; gt.valid(); gt.next()){ 
      if(!strncmp(buffer,gt->name,2)) {
         found = 1;  break;
      }  
    }
    if(!found) {
       arteMsg << "kawire: check command " << buffer ;
       arteMsg.error();
    }
    else gt->eff = atof(ku_getc());
  }
}
	 
/****************************************************************/

// action routine for HBGEAN grafic commands                *
void kapict() {
  char subdet[4];
  zgpict ZGP;
  ZGP.picini();
  strncpy(subdet,ku_getc(),4);
  if      (!strncmp(subdet,"DET" ,3)) ZGP.picdet("MOTH");
  else if (!strncmp(subdet,"MAGN",4)) ZGP.picdet("MAGN");
  else if (!strncmp(subdet,"RPOT",4)) ZGP.picdet("RPOT");
  else if (!strncmp(subdet,"SVD" ,3)) ZGP.picsil();
  else if (!strncmp(subdet,"ITR" ,3)) ZGP.picitr();
  else if (!strncmp(subdet,"OTR" ,3)) picotr_();
  else if (!strncmp(subdet,"PT"  ,2)) {}  //        call picptc
  else if (!strncmp(subdet,"RIC" ,3)) picric_();
  else if (!strncmp(subdet,"TRD" ,3)) pictrd_(); 
  else if (!strncmp(subdet,"ECA" ,3)) piccal_();
  else if (!strncmp(subdet,"MUON",4)) picmuo_();
  else {
    arteMsg << "valid names: det otr itr pt svd ric trd eca muon";
    arteMsg.warn();
  }

  int i0=0, i1=1;
  iuwk_(i0,i1);
}
 
/**********************************************************************/

// reads in the geometry version from the kumac file
void kageov() {
  arteflag_.geomvers = float(ku_getr());
}








