
/***********************************************************************
* CRROTM
* This routines creates a rotation matrix on the next free place
* written by Siegmund Nowak 96/12/09
* partly on base of gsrotm
* updated by Alexander Lanyov to check whether the wanted matrix already exists
*
* called from different geometric routines
*
* Arguments same as in GSROTM (see GEANT-Manual GEOM200-1)
***********************************************************************/

#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcnum.h"

extern "C" {
  void crrotm_(int &,float &,float &,float &,float &,float &,float &);
  void mzbook_(int*, int*, int*, int*, char*, int*, int*, int*, int*, int*, int);
  void mzpush_(int*, int*, int*, int*, char*,const int);
  void vunit_(float *, float *, int*);
}

void crrotm_(int &irot,float &th1,float &ph1,float &th2,float &ph2,float &th3,float &ph3) {  
  int i0=0, i1=1, i3=3, i16=16, i50=50;
  int jrot=0;
  float *pjx; 

  if(gclink_.jrotm<=0) mzbook_(&gcbank_.ixcons,&gclink_.jrotm,&gclink_.jrotm,
                        &i1,"ROTM",&gcnum_.nrotm,&gcnum_.nrotm,&i0,&i3,&i0,4);

// check first the current rotation matrix
  if (irot >= 1  &&  irot<=gcnum_.nrotm) {
    int jrot = *(gcbank_.lq + gclink_.jrotm -1 - irot);
    if(jrot) {
      int *pji = gcbank_.iq + jrot-1;
      pjx = (float*)pji;
      if (fabs(*(pjx+11) - th1)  <  1.e-5  &&  fabs(*(pjx+12) - ph1)  <  1.e-5
       && fabs(*(pjx+13) - th2)  <  1.e-5  &&  fabs(*(pjx+14) - ph2)  <  1.e-5
       && fabs(*(pjx+15) - th3)  <  1.e-5  &&  fabs(*(pjx+16) - ph3)  <  1.e-5)
                                                                        return;
    }
  }

  for (irot=1;irot<=gcnum_.nrotm;irot++) {
    int jrot = *(gcbank_.lq + gclink_.jrotm -1 - irot);
    if(jrot) {
      int *pji = gcbank_.iq + jrot-1;
      pjx = (float*)pji;
      if (fabs(*(pjx+11) - th1)  <  1.e-5  &&  fabs(*(pjx+12) - ph1)  <  1.e-5
       && fabs(*(pjx+13) - th2)  <  1.e-5  &&  fabs(*(pjx+14) - ph2)  <  1.e-5
       && fabs(*(pjx+15) - th3)  <  1.e-5  &&  fabs(*(pjx+16) - ph3)  <  1.e-5)
                                                                        return;
    }

    else goto L200;
  }
  irot = gcnum_.nrotm+1;
  mzpush_(&gcbank_.ixcons,&gclink_.jrotm,&i50,&i0,"i",1);
  gcnum_.nrotm=*(gcbank_.iq + gclink_.jrotm-3);
 

L200: 
  int  mrot = -irot;
  mzbook_(&gcbank_.ixcons,&jrot,&gclink_.jrotm,&mrot,"ROTM",&i0,&i0,&i16,&i3,&i0,4);

  int *pji = gcbank_.iq + jrot-1;
  pjx = (float*)pji;
  *(pjx + 11) = th1;   *(pjx + 12) = ph1;   *(pjx + 13) = th2;
  *(pjx + 14) = ph2;   *(pjx + 15) = th3;   *(pjx + 16) = ph3;

  for(int n=0; n<3; n++) {
     float therad = *(pjx+11+2*n)*AR_DEGRAD;
     float phirad = *(pjx+12+2*n)*AR_DEGRAD;
     float sinthe = sin(therad);
     *(pjx+3*n+1) = sinthe * cos(phirad);
     *(pjx+3*n+2) = sinthe * sin(phirad);
     *(pjx+3*n+3) = cos(therad);
     if( fabs(*(pjx+3*n+1))  <  1.e-7) *(pjx+3*n+1) = 0.;
     if( fabs(*(pjx+3*n+2))  <  1.e-7) *(pjx+3*n+2) = 0.;
     if( fabs(*(pjx+3*n+3))  <  1.e-7) *(pjx+3*n+3) = 0.;
     vunit_(pjx+3*n+1,pjx+3*n+1,&i3);
  }

//       test orthonormality
  int oflag = 0;
  double rotmat[9], prod1 = 0., prod2 =0.,  prod3=0., hmod =0.;
  for (int n=0; n<9; n++)  rotmat[n] = *(pjx+n+1);

//               x - y
  prod1=rotmat[0]*rotmat[3]+rotmat[1]*rotmat[4]+rotmat[2]*rotmat[5];
  if(fabs(prod1) > 1.e-5) oflag = 1;

//               x - z
  prod2=rotmat[0]*rotmat[6]+rotmat[1]*rotmat[7]+rotmat[2]*rotmat[8];
  if(fabs(prod2) > 1.e-5) oflag =1;

//               y - z
  prod3=rotmat[6]*rotmat[3]+rotmat[7]*rotmat[4]+rotmat[8]*rotmat[5];
  if(fabs(prod3) > 1.e-5) oflag =1;;


//     orthonormalization needed
//          assume x correct
  if(oflag ==1) { 
      for(int n=3; n<6; n++){
        rotmat[n]=rotmat[n]-rotmat[n-3]*prod1;
        hmod=hmod+rotmat[n]*rotmat[n];
      } 
      hmod=1./sqrt(hmod);
      for (int n=3; n<6; n++) rotmat[n]=rotmat[n]*hmod;

//          y done, do z
      rotmat[6] = rotmat[1]*rotmat[5] - rotmat[2]*rotmat[4];
      rotmat[7] = rotmat[2]*rotmat[3] - rotmat[0]*rotmat[5];
      rotmat[8] = rotmat[0]*rotmat[4] - rotmat[1]*rotmat[3];
      hmod = rotmat[6]*rotmat[6] + rotmat[7]*rotmat[7] + rotmat[8]*rotmat[8];
      hmod = 1./sqrt(hmod);
      for(int n=6; n<9; n++) rotmat[n] = rotmat[n]*hmod;

//          put back the matrix in place
      for (int n=0; n<9; n++) *(pjx+n+1)=rotmat[n];

//          now recompute the angles
      float angles[6];
      for(int n=0; n<3; n++ ) {
        double arg = 1.;
	if (rotmat[n*3+2] < 1.) arg = rotmat[n*3+2];
        if(arg < -1.) arg=-1.;    
        angles[n*2] = acos(arg)*AR_RADDEG;
        angles[n*2+1] = 0.;
        if(rotmat[n*3+1] != 0.) {
          angles[n*2+1] = atan2(rotmat[n*3+2],rotmat[n*3+1])*AR_RADDEG;
          if(angles[2*n+1] < 0.) angles[2*n+1] = angles[2*n+1]+360.0;
        }
      }

      arteMsg << "crrotm - Parameters of matrix no. " <<irot<<" changed";
      arteMsg.warn();    
      char textbuf[64]; 
      sprintf(textbuf," %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f",
	      *(pjx+11),*(pjx+12),*(pjx+13),*(pjx+14),*(pjx+15),*(pjx+16));
      arteMsg << "old values" << textbuf;   
      arteMsg.warn();

//          put back the angles in place
      memcpy(pjx+11,angles,24);

      sprintf(textbuf," %8.3f  %8.3f  %8.3f  %8.3f  %8.3f  %8.3f",
	      *(pjx+11),*(pjx+12),*(pjx+13),*(pjx+14),*(pjx+15),*(pjx+16));
      arteMsg << "new values" << textbuf;
      arteMsg.warn();
  }
//       orthonormalization ended

//    set flag 
      int ip[3];   
      for(int n=0; n<3; n++) {
        ip[n]  = 3;

        for (int i = 0; i<3; i++) {
          if(fabs(*(pjx+n*3+i+1)) < 0.99999) continue;

          ip[n]  = i + 4;
          if(*(pjx+n*3+i+1) >= 0.) break;

          ip[n]  = 2 - i;
          break;

        }
      }
      *(pjx + 10) = float(ip[0] + 10* ip[1] + 100* ip[2]);
}
