        subroutine otrgeo
******************************************************************
*
* Reads definition of OTR superlayer from text files
* and declare related Arte tables and Geant volumes.
*
* Called from geomain
*
* Author: Alexander Lanyov     Aug 1999
*                      updated Oct 2000 (elimination of ZGOTR.fi)
******************************************************************
*
* Some superlayer values already filled in kagesl.F are just overwritten.
*
************************************************************************
        implicit none

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gcnum.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGDETV.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
#include "auto/CON.fi"

        integer maxsl
        parameter (maxsl=14)

        integer maxwor
        parameter (maxwor=20)

        integer isl

#include "gean/otrgeo.fi"


        integer i,l,ic,nlast,ll,nstereo
        integer it,ltyp_old,ltyp_new,ltyp_geo
        real NaN,angle_old,rotzmax,zmiddle
        real covthick,cov_th,cov_d1,cov_d2,foipin

        integer nwords,lw(maxwor)
        character words(maxwor)*80

        character side*5,name*4,cnamfil*120,dir*120

        logical lfirst,lsldec
        save lfirst

c Function types
        integer lenocc,iread
        logical otrcomc,otrcomr

c required for icdeci (M432)
        integer nd,ne,nf,ng,num
        real dummy
        common /slate/ nd,ne,nf,ng,num(2),dummy(34)

        data lfirst/.true./

        logical mod_exist
        common/otr_mod_exist/mod_exist

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/CON.ff"

        isl = currsl

c-------------------------------- initialization
        if (lfirst) then

           if (geomvers.eq.97.) then
              c78='OTRGEO: For 97 run geometry use ARTE version 1'
              call meerro(c78)
           else if (geomvers.ge.20. .and. geomvers.lt.97.) then
              c78='OTRGEO: Illegal geometry version'
              call ch78bi(int(geomvers))
              call meerro(c78)
           end if

           declare_g10 = 1
           ydg10 = 1.0
           dyg10 = 1.5          ! was 1.0   ! width of G10 strip

           call otrmed
           call otrvol

           lfirst=.false.
        end if
c-------------------------------- End of initialization

        call uhtoc(iq(hqnesl_name(isl)),4,slname,4)
        if (lenocc(slname).le.0)
     +         call meerro('OTRGEO: SLNAME not initialized!')

c------------------------- Install aging boxes at PC01 in 98-99 run
        if (slname.eq.'PC01' .and. abs(fnesl_y1(isl)).lt.100.) then
           c78 = 'Installed aging boxes at PC01 position'
           call me78wa
           call otrage(isl)
           return
        end if

        xmin_all = 1.e10 ! big positive number
        xmax_all =-xmin_all
        ymin_all = xmin_all
        ymax_all = xmax_all
        zmin_all = xmin_all
        zmax_all = xmax_all

        xmin_sl = fnesl_x1(isl)
        xmax_sl = fnesl_x2(isl)
        ymin_sl = fnesl_y1(isl)
        ymax_sl = fnesl_y2(isl)
        zmin_sl = fnesl_z1(isl)
        zmax_sl = fnesl_z2(isl)

        otdisd = 0.
        otdisds = 0.
        foipin = 7.  ! not used?

        covthick = 0.
        ping_st5  = 9.
        ping_st10 = 9.5
        ping_mc5  = 7.3
        ping_sp5  = 4.8
        ping_sp10 = 5.3

        nsl = INESL_NSL(ISL)
        if (nsl.le.0 .or. nsl .gt. maxsl) then
           c78 = 'Illegal SL number ='
           call ch78bi(nsl)
           call meerro(c78)
        endif

        cnamfil=slname(:lenocc(slname))//'.txt'
        call cutol(cnamfil)


        call zgetfil(dir,ll)
        call otropen(lread,cnamfil,dir)

        if (cnamfil(:1).eq.' ') then
           c78 = 'File'
           call ch78bc(cnamfil(:lenocc(cnamfil)))
           call ch78bc('was not opened')
           call meerro(c78)
           return
        end if

        lsldec = .false.
        lhadec = .false.

c To use e.g. NaN = 7FFFFFFF ?
        NaN = -1.e35            ! number which should give an error if used
        angle = NaN

        do i=1,6
           do iside=-1,1
              zstereo(iside,i) = 1500.
           end do
        end do
        iside = 0

        rotzwt(nsl) = 0.
        rotzmax = 0.
        dalign(1) = 0.
        dalign(2) = 0.
        dalign(3) = 0.
        zoffset = 0.
        xref = 0.
        yref = 0.
        zref = 0.
        call otrg10_ini

        ltyp_geo = inesl_ltyp(isl)      ! layer types/sequences
        iq(iqnesl_ltyp(isl)) = 0        ! layer types/sequences
        mod_exist = .false.             ! at this point, no modules exist

c================================================ begin of reading loop
 100    continue
        read (lread,'(a)',end=991) buf

c        buf = spaces(buf,1)  ! does not work for long lines
        call parse(buf,nwords,words,lw,nlast)
        buf = words(1)
        do i=2,nwords
           buf(lenocc(buf)+2:) = words(i)
        end do

        nlast = lenocc(buf)
        call cutol(buf(:nlast))
        if (nlast.le.0) goto 100
        if (buf(:1).eq.'*') goto 100


c-------------------------------- Read alignment corrections (N.B. unit is cm!)
        if (otrcomr('dx',dalign(1),1)) then
        else if (otrcomr('dy',dalign(2),1)) then
        else if (otrcomr('dz',dalign(3),1)) then
        else if (otrcomr('dphiz',rotzwt(nsl),1)) then
        else if (otrcomr('xref',xref,1)) then
        else if (otrcomr('yref',yref,1)) then
        else if (otrcomr('zref',zref,1)) then

        else if (otrcomr('zmiddle',zmiddle,1)) then
           zoffset = 0.5*(fnesl_z1(isl) + fnesl_z2(isl)) - zmiddle

c-------------------------------- Reading of stereo layers position (zstereo)
        else if (otrcomc('z-position backside basisplate 1.module',
     @           cparam,-10)) then

c cparam should contain:
c old   '1.stereo layer at y=0, mm:' zstereo
c new   '1.stereo layer at y=0, -x/+x half of sl' zstereo_left  zstereo_right
c word:  1        2     3  4    5     6    7  8

           if (ichar(cparam(1)(1:1)).lt.ichar('0')
     @    .or. ichar(cparam(1)(1:1)).gt.ichar('9')) then
              call meerro('Illegal stereo layer '//cparam(1)(1:1))
           end if

           nstereo = ichar(cparam(1)(1:1)) - ichar('0')
           if (cparam(5)(:5).eq.'-x/+x') then ! new command - for MC
              if (lenocc(cparam(9)).gt.0) then
                 read (cparam(9),*,err=99) zstereo(-1,nstereo)
              else
                 zstereo(-1,nstereo) = 0.
              end if
              if (lenocc(cparam(10)).gt.0) then
                 read (cparam(10),*,err=99) zstereo(+1,nstereo)
              else
                 zstereo(+1,nstereo) = 0.
              end if
              zstereo(-1,nstereo) = 0.1 * zstereo(-1,nstereo)
              zstereo(+1,nstereo) = 0.1 * zstereo(+1,nstereo)
              if (zstereo(+1,nstereo) .eq. 0.)
     @           zstereo(+1,nstereo) = zstereo(-1,nstereo)

           else if (cparam(5)(:3).eq.'mm:') then ! old command - for PC and TC
              read (cparam( 6),*,err=99) zstereo(-1,nstereo)
              zstereo(-1,nstereo) = 0.1 * zstereo(-1,nstereo)
              zstereo(+1,nstereo) = zstereo(-1,nstereo)
           else
              call meerro('Illegal format for stereo layer '//
     @                    cparam(5)(:lenocc(cparam(5))))
           end if

           zstereo(-1,nstereo) = zstereo(-1,nstereo) + dalign(3)+zoffset
           zstereo(+1,nstereo) = zstereo(+1,nstereo) + dalign(3)+zoffset


c-------------------------------- Some distances for whole superlayer
        else if (otrcomr('distance between pin-hole centers'//
     @              ' for full length modules, mm',otdisd,1)) then
           otdisd = 0.1 * otdisd

        else if (otrcomr('distance between pin-hole centers'//
     @          ' for shorter modules (+x side), mm',otdisds,1)) then
           otdisds = 0.1 * otdisds

        else if (otrcomr('thickness of cap material, mm',cov_th,-1))then
           cov_th = 0.1 * cov_th
        else if (otrcomr('gap between cap and 1.module in z, mm',
     @                                          cov_d1,1))then
           cov_d1 = 0.1 * cov_d1
        else if (otrcomr('gap between last module and cap in z, mm',
     @                                          cov_d2,1))then
           cov_d2 = 0.1 * cov_d2

c-- dummy at the moment
      else if (otrcomr('gap between cap and 1.module in z, mm, double',
     @                                          dummy,1))then
      else if (otrcomr('gap between cap and 1.module in z, mm, single',
     @                                          dummy,1))then

        else if (otrcomr('distance between foil end'//
     @            ' and pin-hole center, mm',foipin,1))then
           foipin = 0.1 * foipin

c distance between pin-hole center and strip end (standard 5mm modules) , mm            90
c distance between pin-hole center and strip end (all mc5 ""sp5""  5mm modules) , mm    73
c distance between pin-hole center and strip end (""sp1"" mc3,4,6,8  5mm modules) , mm  48
c distance between pin-hole center and strip end (standard 10mm modules) , mm           95
c distance between pin-hole center and strip end (""sp1"" mc3,4,6   10mm modules) , mm  53

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end (standard 5mm modules) , mm',ping_st5,1))then
           ping_st5 = 0.1 * ping_st5

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end (standard 10mm modules) , mm',ping_st10,1))then
           ping_st10 = 0.1 * ping_st10

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end (all mc5 sp5 5mm modules) , mm',ping_mc5,1))then
           ping_mc5 = 0.1 * ping_mc5

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end ( sp1 mc3,4,6,8 5mm modules) , mm',ping_sp5,1))then
           ping_sp5 = 0.1 * ping_sp5

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end ( sp1 mc3,4,6 10mm modules) , mm',ping_sp10,1))then
           ping_sp10 = 0.1 * ping_sp10

        else if (otrcomr('thickness of cover material in z, mm',
     @           covthick,1)) then
           covthick = 0.1 * covthick


c----------------------------------------------------------- command SL
        else if (otrcomc('name',cparam,1)
     @      .or. otrcomc('superlayer',cparam,-5)) then
c parameters 2-5 are ignored (nsl, zmin, zmax, n_channels_total)
           name = cparam(1)
           if (name(4:4).eq.' ') then
              name(4:4) = name(3:3)
              name(3:3) = '0'
           end if
           call cltou(name)
           if(slname.ne.name) call meerro(
     @            'Encountered '//name//' while expecting : '//slname)

           if (lsldec) then
              c78 = 'SL has been already declared in file "'
              call ch78ac(cnamfil(:lenocc(cnamfil)))
              call ch78ac('"')
              call meerro(c78)
              goto 991
           end if

           istereo = 0
           nmodule = 0

c These are the coordinates of the whole SL in Lab system.
        else if (otrcomr('xmin',xmin_sl,1)) then

c give warning - also for other coordinates
           if (fnesl_x1(isl).ne.0. .and. xmin_sl.lt.fnesl_x1(isl)) then
              c78 = 'New xmin is less than previous'
              call ch78bf(fnesl_x1(isl),-2)
              call me78wa
           end if

        else if (otrcomr('xmax',xmax_sl,1)) then
           continue
        else if (otrcomr('ymin',ymin_sl,1)) then
           continue
        else if (otrcomr('ymax',ymax_sl,1)) then
           continue

c
c S.Nowak: superlayer Z-sizes should be defined only in a special SL file.
c So ignore this information.
c

       else if (otrcomr('zmin',dummy,1)
     @     .or. otrcomr('minimal z-coord. of sl',dummy,1)) then
           continue

       else if (otrcomr('zmax',dummy,1)
     @     .or. otrcomr('maximum z-coord. of sl',dummy,1)) then
           continue


c----------------------------------------------------------- command Half
        else if (otrcomc('half',buf1,-7)) then

c parameters 4-7 are ignored (n_mod, n_mod_5mm, n_mod_10mm, n_channels_total)

           angle_old = angle

           read (buf1(1),*,err=99) angle
           angle = angle/1000. * 180. / 3.14159      ! mrad --> degrees

           angle = -angle

           iside = iread(buf1(3))
           if (iside.eq.-1) then
              side = 'left'
           else if (iside.eq.1) then
              side = 'right'
           else
              c78 = 'Wrong side in HALF command (98 geometry)'
              call meerro (c78)
           end if

           if (abs(angle-angle_old) .gt. 1.e-5) then
c In principle, it is better to check also the layer side (iside).
              istereo = istereo + 1

              if (abs(angle) .le. 1.1*rotzmax+0.001) then ! angle - in degrees
                 it=0
              else if (angle.gt.0.) then
                 it=1
              else if (angle.lt.0.) then
                 it=2
              end if

c layer types/sequences
c We will add 3 if later we find that this is double layer!
              iq(iqnesl_ltyp(isl)) = iq(iqnesl_ltyp(isl))*10 + it

              nmodule = 0
           end if

c laynam = '-t2-3' for left half of TC02, stereolayer 3
           if (iside.eq.-1) then
              laynam(1:1) = '-'
           else
              laynam(1:1) = '+'
           end if
           laynam(2:2) = slname(1:1)
           call cutol(laynam(2:2))
           laynam(3:3) = slname(4:4)
           laynam(4:4) = '-'
           write (laynam(5:),'(i1)') istereo


           rotz = rotzwt(nsl)*raddeg  ! correction of SL angles from alignment
           rotzmax = max(rotzmax, abs(rotz))
           angle = angle + rotz


c--------- find rotation matrix (create one if does not exist)
        call crrotm(irot,90.,-angle,90.,90.-angle,0.,0.)
        jrot = lq(jrotm-irot)
        if (jrot.eq.0) call meerro ('OTRGEO: illegal rotation matrix')
        ir=irot
        if (angle.eq.0.) ir=0
        currot = ir


        cosang = cos(angle * 0.01745329)
        sinang = sin(angle * 0.01745329)

c--------- Fill NESL table with new information
           if (.not. lsldec) then               ! Book new SL

              q(qnesl_x1(isl)) = xmin_sl ! Already filled in KAGESL?
              q(qnesl_x2(isl)) = xmax_sl ! Already filled in KAGESL?
              q(qnesl_y1(isl)) = ymin_sl ! Already filled in KAGESL?
              q(qnesl_y2(isl)) = ymax_sl ! Already filled in KAGESL?
              q(qnesl_z1(isl)) = zmin_sl ! Already filled in KAGESL?
              q(qnesl_z2(isl)) = zmax_sl ! Already filled in KAGESL?

c           if (xmin_sl.lt.fnesl_x1(isl)) then
c              c78 = 'xmin changed'
c              call ch78bf(fnesl_x1(isl),-2)
c              call ch78bf(xmin_sl,-2)
c              call me78wa
c              q(qnesl_x1(isl)) = xmin_sl
c           end if
c           if (xmax_sl.gt.fnesl_x2(isl)) then
c              c78 = 'xmax changed'
c              call ch78bf(fnesl_x2(isl),-2)
c              call ch78bf(xmax_sl,-2)
c              call me78wa
c              q(qnesl_x2(isl)) = xmax_sl
c           end if
c           if (ymin_sl.lt.fnesl_y1(isl)) then
c              c78 = 'ymin changed'
c              call ch78bf(fnesl_y1(isl),-2)
c              call ch78bf(ymin_sl,-2)
c              call me78wa
c              q(qnesl_y1(isl)) = ymin_sl
c           end if
c           if (ymax_sl.gt.fnesl_y2(isl)) then
c              c78 = 'ymax changed'
c              call ch78bf(fnesl_y2(isl),-2)
c              call ch78bf(ymax_sl,-2)
c              call me78wa
c              q(qnesl_y2(isl)) = ymax_sl
c           end if
c           if (zmin_sl.lt.fnesl_z1(isl)) then
c              c78 = 'zmin changed'
c              call ch78bf(fnesl_z1(isl),-2)
c              call ch78bf(zmin_sl,-2)
c              call me78wa
c              q(qnesl_z1(isl)) = zmin_sl
c           end if
c           if (zmax_sl.lt.fnesl_z2(isl)) then
c              c78 = 'zmax changed'
c              call ch78bf(fnesl_z2(isl),-2)
c              call ch78bf(zmax_sl,-2)
c              call me78wa
c              q(qnesl_z2(isl)) = zmax_sl
c           end if

              iq(iqnesl_lay(isl)) = 0   ! number of layers

              imflag = inesl_mfl(isl)
              if (imflag.eq.0) then
                 call cggesl(slname,'MOTH',0)
              else
                 call cggesl(slname,'MAGN',0)
              end if

              xp_sl = 0.5*(xmax_sl + xmin_sl)
              yp_sl = 0.5*(ymax_sl + ymin_sl)
              zp_sl = 0.5*(zmax_sl + zmin_sl)

              lsldec = .true.
           end if                               ! End of definition of new SL

        lhadec = .true.

c----------------------------------------------------------- command Module
        else if (otrcomc('module',buf1,8)
     @    .or. buf(1:4).eq.laynam(2:5)
     @    .or. (buf(1:1).eq.'s'.and.ichar(buf(2:2)).ge.ichar('1')
     @                         .and.ichar(buf(2:2)).le.ichar('3'))) then
           mod_exist = .true.        ! Some modules exist in this superlayer
           call otrchm

c------------------------------------------------ Command was not found above.
        else
           c78 = 'Unknown command: '//buf(:nlast)
           call meerro(c78)
        end if
        goto 100

99      call meerro
     @     ('Error reading parameters in command: '//buf(:nlast))


c================================================ end of reading loop

c------------------- Here at end of file: Perform final checks, etc.

991     continue
        call ioclos(lread)
992     continue

c------------------------ Create walls (and assign them to the last NEDE bank)
        call otwall(covthick)
        if (declare_g10.eq.2) call otrg10_end

        iq(iqnede_nwl(inede1)) = nnwal() - iq(iqnede_fwl(inede1)) + 1
        iq(iqnesl_nwl(isl)) = nnwal() - iq(iqnesl_fwl(isl)) + 1

c Comparison of coordinates
        if (xmin_sl.gt.xmin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' xmin_sl = ')
           call ch78af(xmin_sl,-4)
           call ch78ac(' xmin_all = ')
           call ch78af(xmin_all,-4)
           call me78wa
        end if
        if (xmax_sl.lt.xmax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' xmax_sl = ')
           call ch78af(xmax_sl,-4)
           call ch78ac(' xmax_all = ')
           call ch78af(xmax_all,-4)
           call me78wa
        end if

        if (ymin_sl.gt.ymin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' ymin_sl = ')
           call ch78af(ymin_sl,-4)
           call ch78ac(' ymin_all = ')
           call ch78af(ymin_all,-4)
           call me78wa
        end if
        if (ymax_sl.lt.ymax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' ymax_sl = ')
           call ch78af(ymax_sl,-4)
           call ch78ac(' ymax_all = ')
           call ch78af(ymax_all,-4)
           call me78wa
        end if

        if (zmin_sl.gt.zmin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' zmin_sl = ')
           call ch78af(zmin_sl,-4)
           call ch78ac(' xmin_all = ')
           call ch78af(zmin_all,-4)
           call me78wa
        end if
        if (zmax_sl.lt.zmax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' zmax_sl = ')
           call ch78af(zmax_sl,-4)
           call ch78ac(' zmax_all = ')
           call ch78af(zmax_all,-4)
           call me78wa
        end if

        iq(iqnesl_lay(isl)) = istereo ! number of layers

c reverse digits in a decimal number
        ltyp_new = 0
        ltyp_old = inesl_ltyp(isl)
        do i=1,istereo
           l = mod(ltyp_old,10)
           ltyp_old = ltyp_old / 10
           ltyp_new = 10*ltyp_new + l
        end do
        iq(iqnesl_ltyp(isl)) = ltyp_new


c OTGL table should be filled starting from ARTE-03-01:
c otgl_gede I    0     'number of rows in GEDE related to OT(=2485)'
c otgl_gede I    0     'number of rows in GEDE related to OT(=2485)'
c otgl_mchn I    0     'maximal number of channel(wire)/gede row (=128)'
c otgl_medb I    0     'method of storage of time-space relations in DB'
c                      '=0, coefficients of splines (default)'
c otgl_meth I    0     'method of storage of time-space relations in OTTS'
c                      '=0, tabulated form ,TDC=0,1,2, ... 256(default)'
c                      '=1,                , uniform binning of TDC '
c                      '=2,                , arbitrary
c otgl_knot I    0     'number of knots for tabulation(=129) '
c otgl_wvel F    0     'signal propagation velocity on wire (=28.2 cm/ns)'
c otgl_vd   F    2     '[0] average drift velocity for 5mm  (=0.01 cm/ns)'
c                      '[1]                            10mm '
c otgl_vdmf F    2     '[0]                            5mm  in magnet'
c                      '[1]                            10mm in magnet'
c otgl_fele F    0     'electronic resolution in ns (=0.8)'
c otgl_ntdc F    0     'ns per one TDC count (=0.39 ns/tdc count)'
c otgl_fatt F    0     'fraction of electrons that reach the wire (=0.25)'
c otgl_damp F    0     '            charge seen by the amplifier (=0.2)'
c otgl_db   F    0     'parameter of Polya distribution (=1.222)'
c otgl_dqe  F    0     'elementary charge (=1.602e-19)'
c otgl_dlam F    0     'doubled primary ionization density (=102. 1/cm)'
c otgl_xtcl F    0     'probability of x-talk through cell '
c otgl_xtre F    0     '                              readout circuit'

        return
        end
