      subroutine  otwall(covthick)
************************************************************************      
*
* Defines walls, plates (for MC04, MC05, MC06, MC08), rails (for MC05)
* and caps for OTR.
*
* Called from: otrgeo.F
*
* Arguments  in/out       Description
*
* covthick   in/out       cover plate total thickness in cm
*                         (may be overwritten for run 99)
*
* Created: Alexander Lanyov     Mar 1999
* Updated: Oxana Barsoukova     Sep 2001 (added plates, rails and caps)
*
************************************************************************      
      implicit none

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/otrgeo.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"

#include "arte/ArteFlag.fi"
      integer npar,loop
      integer mf,dir
      real z1,z2,dist,fiberthick,covthick
      character*4 csl,comb,fiber
      real xp,yp,zp
      real par(6),zstereo_minus(3),zstereo_plus(3)

      real zwal_mc04
      save zwal_mc04
      
      character dir_*120
      character*40 fname
      integer ll,lenocc,ii,kk,jj,ic
      real arr_dat,z5,z6,ang
      logical flfil
      
      character*4 matter
      integer nwords,lw(100),nlast
      character words(100)*80

      COMMON/A1/flfil,arr_dat(9,8)/A2/ang

      logical mod_exist
      common/otr_mod_exist/mod_exist
      
#include "arte/Arte.ff"
#include "auto/GEO.ff"
      
c--   install the superlayer volume
      call uhtoc(hnesl_name(currsl),4,csl,4)
      nsl = inesl_nsl(currsl)
      mf = inesl_mfl(currsl) + 1
      
      if (covthick.le.0.) then
         c78 = 'Illegal cover thickness:'
         call ch78bf(covthick,-2)
         call ch78bc(' cm')
         call meerro (c78)
      end if
      
      if (csl(:2).eq.'MC'
     @     .and. ichar(csl(4:4)).le.ichar('5')) then ! MC01-MC05
         if (mf.eq.1) then
            comb = 'WTCF'       ! carbon fiber
         else
            comb = 'WTCI'
         end if
         dist = 0.5             ! 0.5 ~ for mc05 exactly, not for others
         fiberthick = 0.
      else if (csl(:2).eq.'MC') then ! MC06, MC08
         if (mf.eq.1) then      ! no magnet field
            comb = 'WTPH'       ! paper honeycomb
         else
            comb = 'WTPI'
         end if
         dist = 0.5         
         fiberthick = 0.05
      else if (csl(:2).eq.'PC') then ! PC
         comb = 'WTPH'
         dist = 0.5
         fiberthick = 0.1
      else if (csl(:2).eq.'TC') then ! TC
         comb = 'WTAH'
         dist = 0.5
         fiberthick = 0.1
      else
         call meerro('OTWALL: Wrong superlayer name: '//csl)
      end if
      
      if (csl.eq.'TC02' .and. xmax_all.lt.-20 ) then ! mini-TC in run 99
         covthick = 0.4
         comb = 'WTAL'
         dist = 0.5 * (8. - zmax_all + zmin_all)
         fiberthick = 0.
      end if
      
      if (mf.eq.1) then
         fiber = 'WTGF'
      else
         fiber = 'WTGL'
      end if
      
c--   install the front/back plates of the gas box
      if (mod_exist) then
         do dir=-1,1,2
            if (dir.eq.-1) then
               z1 =  zmin_all - dist ! dir=-1: front plate
            else
               z1 =  zmax_all + dist ! dir=+1; back plate
            end if
            z2 = z1 + dir*fiberthick
            if (fiberthick.ne.0.) call otrcov(fiber,z1,z2)
            
            z1 = z2
            z2 = z1 + dir*(covthick - 2.*fiberthick)
            call otrcov(comb,z1,z2)
            
            z1 = z2
            z2 = z1 + dir*fiberthick
            if (fiberthick.ne.0.) call otrcov(fiber,z1,z2)
            
            if (csl.eq.'MC04') zwal_mc04 = z2

         end do
      end if
      
      npar = 6
      
c--   definition of plates (and rails for MC05)

      if (csl.eq.'MC04' .or. csl.eq.'MC05' 
     @     .or. csl.eq.'MC06' .or. csl.eq.'MC08') then
         
         zstereo_minus(1) = zstereo(-1,1)
         zstereo_minus(2) = zstereo(-1,2)
         zstereo_minus(3) = zstereo(-1,3)
         zstereo_plus(1) = zstereo(+1,1)
         zstereo_plus(2) = zstereo(+1,2)
         zstereo_plus(3) = zstereo(+1,3)

         call otplates(csl,zstereo_minus,zstereo_plus,
     @        zmin_all,zmax_all,zwal_mc04,mod_exist)
         
      end if


      if (mod_exist .and. csl.eq.'MC05') then   !MC05 inner frame
 
c--   'CARBONFIBRE' caps for MC05
 
         xp = 12.2 - xp_sl
         yp = 0.5*((ymax_all+7) + ymin_all) - yp_sl
         zp = 0.5*(zmin_all+zmax_all) - zp_sl
         par(1) = 0.05
         par(2) = 0.5*((ymax_all+7) - ymin_all)
         par(3) = 0.5*((zmax_all+dist)-(zmin_all-dist))
         par(4) = 0.
         par(5) = 300.
         par(6) = 0.
         call gsposp('WTCO',
     @        nnwal()+1,csl,xp,yp,zp,0,'ONLY',par,npar)
         call crgwal('WTCO',nnwal()+1)
 
         xp = 11.5 - xp_sl
         yp = 0.5*((ymax_all+7) + 13.55) - yp_sl
         par(1) = 0.05
         par(2) = 0.5*((ymax_all+7) - 13.55)
         par(3) = 0.5*((zmax_all+dist)-(zmin_all-dist))
         par(4) = 0.
         par(5) = 300.
         par(6) = 0.
         do loop = 1,2
            call gsposp('WTCO',
     @           nnwal()+1,csl,xp,yp,zp,0,'ONLY',par,npar)
            call crgwal('WTCO',nnwal()+1)
            yp = 0.5*((ymin_all-7) - 13.55) - yp_sl
         end do
 
      end if
      
c--   Procedure of reading data from caps_over_pipe.dat file
c--   running only once
c--   Data stored to the arr_dat array

      ii=0
      fname = 'caps_over_pipe.dat'
      
      if (.not. flfil) then
         call zgetfil(dir_,ll)
         call otropen(lread1,fname(:lenocc(fname)),dir_)
         
         if (fname(:1).eq.' ') then
            c78='Ignore this mess if your geovers is older than 1.0701'
            call mepric(67281944,1,c78)
            return
         end if
         
 100     continue
         read(lread1,'(a)',end=991) buf
         
         if (lenocc(buf).le.0) goto 100
         if (buf(:1).eq.'*') goto 100
         
         if (buf(:5).eq.'angle') then
            call parse(buf,nwords,words,lw,nlast)
            read(words(2),*) ang
            goto 100
         end if

         if (buf(:8).eq.'material') then
            call parse(buf,nwords,words,lw,nlast)
            matter = words(2)
            
c--   after "parse" matter is in lowercase letters;
c--   following cycle capitalize letters
            
            do jj=1,len(matter)
               ic=ichar(matter(jj:jj))
               if (ic.ge.ichar('a') .and. ic.le.ichar('z'))
     @              matter(jj:jj)=char(ic+ichar('A')-ichar('a'))
            end do
            goto 100
         end if
         
         ii=ii+1
         read(buf,*) arr_dat(ii,1),arr_dat(ii,2),
     @        arr_dat(ii,3),arr_dat(ii,4),arr_dat(ii,5),
     @        arr_dat(ii,6),arr_dat(ii,7),arr_dat(ii,8)
         
         goto 100
         
 991     continue
         call ioclos(lread1)
         flfil=.true.
      end if
      
c--   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      
c--   'CARBONFIBRE' caps for TC01 and TC02
      
      if (csl(:2).eq.'TC')
     @     call capstc(csl,xp_sl,yp_sl,zp_sl,istereo)
      
      
c--   'CARBONFIBRE' caps for PC01 and PC04
      
      if (csl.eq.'PC01' .or. csl.eq.'PC04')
     @     call capspc14(csl,xp_sl,yp_sl,zp_sl,istereo)
      
      
c--   'CARBONFIBRE' caps for PC02 and PC03
      
      if (csl.eq.'PC02' .or. csl.eq.'PC03')
     @     call capspc23(csl,xp_sl,yp_sl,zp_sl,istereo)
      
      
c--   'CARBONFIBRE' caps for MC
      
      if (mod_exist .and. csl(:2).eq.'MC'
     @     .and. ichar(csl(4:4)).ne.ichar('5'))
     @     call capsmc(csl,xp_sl,yp_sl,zp_sl,istereo)
      
      
c--   _________caps on the outside proton beam pipe____________
c--   creates caps over pipe for each MC
      
      if (mod_exist .and. csl(:2).eq.'MC') then
         do kk=1,9
            if (arr_dat(kk,7) .eq. 0.) then
               z5=arr_dat(kk,5)+zmin_all
            else
               z5=arr_dat(kk,5)+zmax_all
            end if
            
            if (arr_dat(kk,8) .eq. 0.) then
               z6=arr_dat(kk,6)+zmin_all
            else
               z6=arr_dat(kk,6)+zmax_all
            end if
            
            call crbox(matter,csl,arr_dat(kk,1),arr_dat(kk,2),
     @           arr_dat(kk,3),arr_dat(kk,4),z5,z6)
         end do
      end if

      return
      end
