#ifndef GROVER_WITHOUT_ARTE
#include "arte/arte.hh"
#endif

#include "grover/grover.hh"

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// ------------------------------------------------------------------------
//
// 01-05-10  Michael typo 16465 -> 16475 in switch fixed

int FakeWires(GroverWireList* wires, const int run_number)
{
  /////////////////////////////////////////////////////////////////////////////////
  //
  // Dirty hack to replace a bad wire list with a good one. Only for runs
  // where the contents of DTAR (=GTAR) are incorrect because of some target DB
  // problem during data taking. GTAR must exist and have 8 wires in the correct
  // order, otherwise no action is taken. The wires which were used, and their
  // approximate positions in global coordinates, must be put here by hand for
  // every bad run!
  //
  /////////////////////////////////////////////////////////////////////////////////

  int err = FAILURE;
#ifndef GROVER_WITHOUT_ARTE  


  // exit quietly if run is not known to be bad
  if (!(
	run_number    ==-99999 // etc... (add more here as required)	
	|| run_number == 15063
	|| run_number == 16529
	|| run_number == 16475
	))
    return SUCCESS;



  const int  NT = 8;
  bool used[NT] = {false};
  

  // make static list of wires from GTAR and check for consistency
  static GroverWireList w;
  static bool firstcall = true;
  static int ctr = 0;
  if (firstcall) {
    for (ArteTable<GTAR>::iterator igt = ArteTable<GTAR>::begin();
	 igt != ArteTable<GTAR>::end(); ++igt) {
      ++ctr;
      if ((*igt)->wnr != ctr) return FAILURE;
      else w.push_back(GroverWire(*igt));
    }
    firstcall = false;
  }
  if (w.size() != 8) return FAILURE;




  ////////////////////////////////////////////////////////////////////////////////////////
  //
  //
  //   The positions are set here by hand. Positions in cm please (global coordinates)!
  //   For reference, the z positions in global coordinates are listed here:
  //   GTAR.wnr = 1     2     3     4     5     6     7    8
  //   i =        0     1     2     3     4     5     6    7
  //   z =       -1.35 -1.44 -0.88 -1.06 -5.44 -5.3 -4.88 -4.94;
  //   Note that the wires are numbered 1 to 8 in GTAR but the 
  //   indexing below ("i") is from 0 to 7 inclusive!
  //
  int i = 0;
  switch (run_number) {

  case 15063:
    i=6; used[i]=true; w[i].SetX(0.46); w[i].SetY(0.07); w[i].SetZ(-4.88); 
    break;

  case 16475:
    i=6; used[i]=true; w[i].SetX(0.32); w[i].SetY(0.13); w[i].SetZ(-4.88); 
    break;

  case 16529:
    i=6; used[i]=true; w[i].SetX(0.32); w[i].SetY(0.13); w[i].SetZ(-4.88); 
    break;

  case -99999: // example of how to add another bad run
    i=0; used[i]=true; w[i].SetX(9.99); w[i].SetY(9.99); w[i].SetZ(9.99); 
    i=1; used[i]=true; w[i].SetX(8.88); w[i].SetY(8.88); w[i].SetZ(8.88); 
    break;

  default:
    return FAILURE; 
    break;
  }
  //
  //
  ////////////////////////////////////////////////////////////////////////////////////////




  // Final consistency check before we erase the contents of the 
  // user's list!
  bool quit = true;
  for (int j = 0; j < NT; ++j)  if (used[j]) quit = false;
  if (quit) return FAILURE;




  // clear user's list and overwrite with corrected wires
  wires->clear();
  for (int j = 0; j < NT; ++j)
    if (used[j]) wires->push_back(w[j]);
  err = (wires->empty()) ?  FAILURE : SUCCESS;



#endif  
  return err;
}
