#ifndef GROVER_WITHOUT_ARTE
#include "arte/arte.hh"
#endif

#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
// 01-03-13  Max      Added preprocessor directives
//

int GroverCorrectWires(GroverWireList* gwl)
{
  //////////////////////////////////////////////////////////////////
  //
  // GroverCorrectWires
  // 
  // Max 06 Dec 00 (numbers from Ivan)
  // 
  // Applies shifts to wires from GTAR to bring them closer to what 
  // we observe in the vertexing. Use at your own risk! Does nothing
  // to Monte Carlo inside Arte.
  // 
  //////////////////////////////////////////////////////////////////

#ifndef GROVER_WITHOUT_ARTE
  if (ArteTable<MCIN>::booked()) return SUCCESS;
#endif

  static const float offset[9][3] = {
    {0.0, 0.0, 0.0},
    //     
    // These are the numbers which must be added to GTAR to
    // get the numbers we see in the vertex distributions.
    //
    // dx(cm)   dy(cm)    dz(cm)
    // -------------------------
    {  0.0000, -0.0141,  0.0000  }, // wire 1 = above I
    {  0.0000, -0.0342,  0.0000  }, // wire 2 = below I
    {  0.0010,  0.0000,  0.0000  }, // wire 3 = inner I
    {  0.0201,  0.0000,  0.0000  }, // wire 4 = outer I
    {  0.0000, -0.0965,  0.0000  }, // wire 5 = above II
    {  0.0000, -0.0259,  0.0000  }, // wire 6 = below II
    {  0.0710,  0.0000,  0.0000  }, // wire 7 = inner II
    {  0.0347,  0.0000,  0.0000  }  // wire 8 = outer II
  };


  // beam spot in 2000
  static const float beamx = ggBeamspotX, beamy = ggBeamspotY;
  
  for (GroverWireList::iterator iwire = gwl->begin();
       iwire != gwl->end(); ++iwire) {
    bool waslocal = iwire->InLocal();
    if (waslocal) iwire->LocalToGlobal();
    int wnr = iwire->Number();
    if (wnr < 1 || wnr > 8) continue;
    iwire->SetX(iwire->X() + offset[wnr][0]);
    iwire->SetY(iwire->Y() + offset[wnr][1]);
    iwire->SetZ(iwire->Z() + offset[wnr][2]);
    // Correction for beam spot
    if (iwire->Horizontal())    iwire->SetX(beamx);
    else if (iwire->Vertical()) iwire->SetY(beamy);
    // back to original coordinate system
    if (waslocal) iwire->GlobalToLocal();
  }

  return SUCCESS;
}
