#ifndef GROVER_WITHOUT_ARTE
#include "arte/arte.hh"
#endif

#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
// 00-12-07  Max      Tidied up, moved hardwired corrections to new file
// 01-05-09  Michael  use only wires with effi > 0. to reject non activ wires
//                    in MC
//

int GroverFillFromGTAR(GroverWireList* gwl)
{
  ////////////////////////////////////////////////////////////////////////
  //
  // Fills a GroverWireList from GTAR
  // 1) Ignores wires which are at x,y = 0,0 
  // 2) corrects bug in MC wire 2
  // 3) Ignores vertical wires with |x|>1cm, horizontal wires with |y|>1cm
  // 4) corrects wire positions by hand to better match observed positions
  //
  // returns 0 if OK, -1 for error
  //
  ////////////////////////////////////////////////////////////////////////

  int errorflag = SUCCESS;

#ifndef GROVER_WITHOUT_ARTE
  for (ArteTable<GTAR>::iterator igtar = ArteTable<GTAR>::begin();
       igtar != ArteTable<GTAR>::end(); ++igtar) {
    if (igtar->eff <=0.) continue;
    GroverWire temp(igtar);
    // special treatment of a wrong wire in simulated data
    // (x,y,z) is actually (y,z,x) for this wire!
    if (ArteTable<MCIN>::booked() && (temp.Number() == 2)){ 
      float x = temp.Y(); float y = temp.Z(); float z = temp.X();
      temp.SetX(x);       temp.SetY(y);       temp.SetZ(z);
    } 
    // ignore wires at (0,0)
    // and wires more than 1cm radially from (0,0)
    if ((fabs(temp.X())<0.001 && fabs(temp.Y())<0.001)  ||
	((temp.Horizontal() && fabs(temp.Y()) > 1.0)|| 
         (temp.Vertical()   && fabs(temp.X()) > 1.0)  )  ) continue;
    // keep this wire
    else gwl->push_back(temp); 
  }


  // Correct the wires according to some hard-wired numbers
  if (gwl->empty()) errorflag = FAILURE;
  else errorflag = GroverCorrectWires(gwl);
#endif

  return errorflag;

}

