//**********************************************************************/
// Created: 23.09.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcLimit class
//
//**********************************************************************/
#include "marple/marple.hh"
#include <iostream.h>
#include <iomanip.h>
#include <math.h>

void RcLimit::set( const RcPoint & rp, int icheck )
{
  RcMatchFlag tmpFlag( icheck) ;
  if( icheck < 0 ) tmpFlag = flag;

  if( tmpFlag.ifmarked_Z() )
    {
      if( rp.z()  < z_min  ) { z_min  = rp.z();  }
      if( rp.z()  > z_max  ) { z_max  = rp.z();  }
    }
  if( tmpFlag.ifmarked_X() )
    {
      if( rp.x()  < x_min  ) { x_min  = rp.x();  }
      if( rp.x()  > x_max  ) { x_max  = rp.x();  }
    }
  if( tmpFlag.ifmarked_Y() )
    {
      if( rp.y()  < y_min  ) { y_min  = rp.y();  }
      if( rp.y()  > y_max  ) { y_max  = rp.y();  }
    }
  if( tmpFlag.ifmarked_TX() )
    {
      if( rp.tx() < tx_min ) { tx_min = rp.tx(); }
      if( rp.tx() > tx_max ) { tx_max = rp.tx(); }
    }
  if( tmpFlag.ifmarked_TY() )
    {
      if( rp.ty() < ty_min ) { ty_min = rp.ty(); }
      if( rp.ty() > ty_max ) { ty_max = rp.ty(); }
      icheck -=(int)floor(double(icheck+1)/10)*10;
    }
  if( tmpFlag.ifmarked_PE() )
    {
      if( rp.pe() < pe_min ) { pe_min = rp.pe(); }
      if( rp.pe() > pe_max ) { pe_max = rp.pe(); }
    }
}

//-------------------------------------------------------------
 int RcLimit::valid( const RcPoint & rp, int icheck ) const
{
  RcMatchFlag tmpFlag( icheck) ;
  if( icheck < 0 ) tmpFlag = flag;

  if( tmpFlag.ifmarked_Z() )
    if( rp.z()  < z_min  || rp.z()  > z_max  ) 
      { return 0;  }
    
  if( tmpFlag.ifmarked_X() )
    if( rp.x()  < x_min  || rp.x()  > x_max  ) 
      { return 0;  }

  if( tmpFlag.ifmarked_Y() )
    if( rp.y()  < y_min  || rp.y()  > y_max  ) 
      { return 0;  }

  if( tmpFlag.ifmarked_TX() )
    if( rp.tx() < tx_min || rp.tx() > tx_max ) 
      { return 0;  }

  if( tmpFlag.ifmarked_TY() )
      if( rp.ty() < ty_min || rp.ty() > ty_max ) 
	{ return 0;  }

  if( tmpFlag.ifmarked_PE() )
      if( rp.pe() < pe_min || rp.pe() > pe_max ) 
	{ return 0;  }

  return 1;
}

ostream & operator << (ostream & os, const RcLimit & rcl)
{
  os << "===== Limits used for selection (MARPLE) ======" << endl;
  os << " the point is accepted if the parameters {" << endl;
  
  if( rcl.flag.ifmarked_Z() )
    os << rcl.z_min  << " < z  < " << rcl.z_max  << endl;

  if( rcl.flag.ifmarked_X() )
    os << rcl.x_min  << " < x  < " << rcl.x_max  << endl;

  if( rcl.flag.ifmarked_Y() )
    os << rcl.y_min  << " < y  < " << rcl.y_max  << endl;
  
  if( rcl.flag.ifmarked_TX() )
    os << rcl.tx_min << " < tx < " << rcl.tx_max << endl;
  
  if( rcl.flag.ifmarked_TY() )
    os << rcl.ty_min << " < ty < " << rcl.ty_max << endl;
  
  if( rcl.flag.ifmarked_PE() )
    os << rcl.pe_min << " < pe < " << rcl.pe_max << endl;

  return os;
}


