#ifndef _RCSEGRPNTBUILDER_HH_
#define _RCSEGRPNTBUILDER_HH_
//---------------------------------------------------------------------
// Created: 26.07.2000 by  O.Igonkina, W.Hulsbergen & P.Conde

// MARPLE builder class for match Segments and RPNTs

//---------------------------------------------------------------------


#include <vector>
#include <iterator>

#include "marple/RcSegPoint.hh"
#include "marple/RcRpntPoint.hh"
#include "marple/RcMatchBuilder.hh"


class RcSegRpntBuilder: public RcMatchBuilder<RcSegPoint, RcRpntPoint>
{
public:
  ///              default construtor
  RcSegRpntBuilder(); 

  ///              make match of two vectors with rcConst constants
  RcSegRpntBuilder( std::vector<RcSegPoint  *> & vecLeft,
		    std::vector<RcRpntPoint *> & vecRight,
		    RcMatchConst & rcConst);

  ///               destructor
  ~RcSegRpntBuilder();

  // MatchPoint.mark() = 0 standard match
  //                   - 1 matching with segment momentum estimation

  typedef  int (RcSegRpntBuilder::* MatchFunction) ( RcSegPoint  * segment, 
						     RcRpntPoint * rpnt,
						     RcMatchPoint* &result);

  void doMatch();
  void doMatch(  MatchFunction matchFunction );

  //void matchWithMomentum();
  void matchInTheMiddle();


  void saveRSEG(int & iErr ) const;
  void saveRelation(int & iErr) const;



protected:
  /*
  int matchPairWithMomentum( RcSegPoint  * segment, 
			     RcRpntPoint * rpnt,
			     RcMatchPoint* &result);
  */
  int matchPairInTheMiddle( RcSegPoint  * segment, 
			    RcRpntPoint * rpnt,
			    RcMatchPoint* &result);

  /// In this case the relation between RSEG and another RSEG related to RPNT is set
  void saveRelation(const RcSegPoint * orig1, 
                    const RcRpntPoint * orig2, 
                    int & iErr) const;

  void saveRelationWithMatch(const RcSegPoint  * orig1, 
                             const RcRpntPoint * orig2, 
                             ArtePointer<RSEG> rseg,
                             int & iErr) const;

private:
  // same as in RcMatchBuilder
};

#endif
