//--------------------------------------------------------------------
// Created: 02.03.98 by   Olya Igonkina
// see http://www-hera-b.desy.de/subgroup/software/arte/MARPLE/    

/* MARPLE function for matching SVD segments and ECAL clusters

   FORTRAN_ROUTINE(matchsvdecal)( float * threshold_cut ) 
   calls matchsvdecalt with request = 1

   FORTRAN_ROUTINE(matchsvdecalt)( float * threshold_cut, int request)

   if request =  0 threshold cut is NOT considered
   if         = -1 cluster parameter should be LESS then threshold
   if         =  1 cluster parameter should be MORE then threshold

   if threshold_cut > 200 - it is Ktrig cut, otherwise it is pt_cut  


   1. search of pair segment-cluster corresponded to electrons/positrons 
      hypothesis is performed. Two algorithms are available
      a) segment momentum estimated by its prolongation 
         to the cluster point coordinate.
      b) momentum is supposed to be equal a energy of the cluster with  
         2 charge hypothesis
	 
   2. search of bremstrahlung is performed for segments satisfied charge 
      hypothesis ( linear prolongation of segment to the ECAL frontplate, 
      chi2 value is calculated). The energy of bremstrahlung cluster is added
      to charge hypothesis's one.

 */
//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcSegClusBuilder.hh"
#include "arte/RSEG_cmp.hh"

#include "marple/RcMcFunction.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchSvdEcal = 0;
#endif

int MARPLE_SE_SVDNORTRA = 0;
int MARPLE_SE_ECALNORTRA = 0;


//definition of constants for SUSI-CARE matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
//use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2

// match with Ecluster
RcMatchConst SCmatchEConst = { 
  11000,     1330.,   -30.,  1500.,      1,          1,            2,
  2,       7.,      2.,   1.e+10,  1.e+10,   200.,   3 //1.5
};


// match with P estimation
RcMatchConst SCmatchPConst = { 
 11000,     1330.,   -30.,  1500.,      0,          2,            2,
 //   2,       .002,      3.,   1.e+10,  1.e+10,   0.05,   3.
    1,       1.,      2.5,   1.e+10,  1.e+10,   1. ,   35.
};

// constant for bremstrahlung search
RcMatchConst BremConst = { 
 11000,     1330.,   -30.,  1500.,      1,          1,            1,
     2,       6.,      6.,   1.e+10,  1.e+10,   1.e+10,   6.
};
RcMatchConst BremConstLow = { 
 11000,     1330.,   -30.,  1500.,      1,          1,            1,
     1,       6.,      6.,   1.e+10,  1.e+10,   1.e+10,   1.e+10
};

RcMatchConst BremConstSoft = { 
 11000,     1330.,   -30.,  1500.,      0,          1,            1,
     1,       10.,      10.,   1.e+10,  1.e+10,   1.e+10,  1.e+10
};

RcMatchConst SCmatchSoftConst = { 
  11000,     1330.,   -30.,  1500.,      0,          2,            2,
  1,       3.,      5.,   1.e+10,  1.e+10,   1.,   15000
};

extern "C"  
void FORTRAN_ROUTINE(matchsvdecalt)( float * threshold_cut, int * request );

extern "C"  
void FORTRAN_ROUTINE(matchsvdecal)( float * threshold_cut )
{
  int request = 1;
  matchsvdecalt_(threshold_cut, &request);
} 

extern "C"  
void FORTRAN_ROUTINE(matchsvdecalt)( float * threshold_cut, int * request ) 
{
# ifdef MARPLEPERFORMANCES
  if(chMatchSvdEcal == 0){
    ChronoManager CM;
    chMatchSvdEcal = CM.createChrono("match Svd Ecal ");
  }
  chMatchSvdEcal->start();
# endif

  maPresentMatch =MARPLEIDVDSECAL;



  static int matchWithMomentum = 1; // choose algorithm to match
  static int matchBrem = 0;

  static const float brem_threshold = 1; // don't consider clusters with E< 1GeV for bremstrahlung search

  // ----------------------------------------------------
  //  if magnet is OFF call more trivial algorithm!
  if( !maMagnetON() ) 
    {      
      matchsenomag_( threshold_cut);
# ifdef MARPLEPERFORMANCES
      chMatchSvdEcal->stop();
# endif
      maPresentMatch = MARPLENOMATCH;
      return;
    }

  // ----------------------------------------------------

  RcSegClusBuilder gISvdEcalMatch;  // builder for charge track search
  RcSegClusBuilder gISvdEcalBrem;   // builder for bremstrahlung search


  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------

  if( MARPLE_SOFT )
    {
      gISvdEcalMatch.setConst(     SCmatchSoftConst);
      gISvdEcalMatch.setBremConst( BremConstLow);
      gISvdEcalBrem.setConst(     SCmatchSoftConst);
      //      gISvdEcalBrem.setBremConst( BremConstLow);
      gISvdEcalBrem.setBremConst( BremConstSoft);
    }else{ 
      if( matchWithMomentum )
	gISvdEcalMatch.setConst(     SCmatchPConst);
      else
	gISvdEcalMatch.setConst(     SCmatchEConst);

      gISvdEcalMatch.setBremConst( BremConstSoft);
      
      // work with low energetic clusters for bremstrahlung search
      gISvdEcalBrem.setConst(     SCmatchEConst);
      gISvdEcalBrem.setBremConst( BremConstSoft);
      //gISvdEcalBrem.setBremConst( BremConstLow);
    }      
  //--------------------------------------------------------------------
  // create vector of segments from SUSI
  //--------------------------------------------------------------------

  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if( (*irseg)->fit != 0 ) continue;
      if( ( (*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( (*irseg)->cmp != Rsegc::vxd  && 
	  (*irseg)->cmp != 65537) continue;
      if( MARPLE_SE_SVDNORTRA )
	{
	  int rtraFound = 0;
	  for(ArteRelationIterator<RTRA> irt = (*irseg)->rtra.begin();
	      !rtraFound && irt != (*irseg)->rtra.end(); ++irt )
	    if( ((*irt)->cmp & Rsegc::bitsign) == 0 ) rtraFound = 1;
	  if( rtraFound ) continue;
	}
      if( gISvdEcalMatch.ptToMatchConst()->
	  geomAccept(HepDouble((*irseg)->ze)) )
	{ 	  
	  RcSegPoint * ptToPoint = new RcSegPoint( *irseg , 1); // endPoint
	  gISvdEcalMatch.addPointLeft( *ptToPoint);

	  //ptToPoint = new RcSegPoint( irseg , 1); // endPoint
	  // gISvdEcalBrem.addPointLeft( *ptToPoint);

	  ptToPoint = 0;
	}
    }


  // no candidate found --- no reason go through clusters
  if( gISvdEcalMatch.vectorLeft().size() == 0 ) 
    {
# ifdef MARPLEPERFORMANCES
      chMatchSvdEcal->stop();
# endif
      maPresentMatch = MARPLENOMATCH;
      return;
    }

  //--------------------------------------------------------------------
  // create vector of clusters from CARE
  //--------------------------------------------------------------------

  static std::vector<RcClusPoint *> nextClusterSet;
  static std::vector<RcClusPoint *> nextBremSet;
      
  if( MARPLE_BKGR )
    {
      // clusters from previous event
      for( unsigned int clus= 0; clus < nextClusterSet.size(); clus++ )
	gISvdEcalMatch.addPointRight( *(nextClusterSet[clus]) );
      
      nextClusterSet.erase(nextClusterSet.begin(), nextClusterSet.end());
      
      for( unsigned int clus= 0; clus < nextBremSet.size(); clus++ )
	gISvdEcalBrem.addPointRight( *(nextBremSet[clus]) );    
      
      nextBremSet.erase(nextBremSet.begin(), nextBremSet.end());
    }
  

  int iErr  = 0;
  for( ArteTable<RCCL>::iterator irccl = ArteTable<RCCL>::begin();
       irccl != ArteTable<RCCL>::end(); ++irccl )
    {
      if( (*irccl)->flag < 0 ) continue;
      if( (*irccl)->esum < 1 ) continue;
      if( MARPLE_SE_ECALNORTRA )
	{
	  int rtraFound = 0;
	  for(ArteRelationIterator<RTRA> irt = (*irccl)->rtra.begin();
	      !rtraFound && irt != (*irccl)->rtra.end(); ++irt )
	    if( ((*irt)->cmp & Rsegc::bitsign) == 0 ) rtraFound = 1;
	  if( rtraFound ) continue;
	}

      RcClusPoint * ptPoint = 0;
      int greaterThenThreshold =  maTakeRccl(*threshold_cut, *irccl);
	
      if( *request == 0 ||                             // everything
	  ( *request > 0.5 && greaterThenThreshold) || // high energy
	  ( *request < 0.5 && !greaterThenThreshold))  // low energy
	{
	  ptPoint = new RcClusPoint( (*irccl));
	  if( MARPLE_BKGR )
	    nextClusterSet.push_back(ptPoint);
	  else
	    gISvdEcalMatch.addPointRight( *ptPoint);
	  ptPoint = 0;
	}
      if( matchBrem )
	{
	  if( (*irccl)->esum > brem_threshold )
	    {
	      ptPoint = new RcClusPoint( (*irccl));
	      if( MARPLE_BKGR )
		nextBremSet.push_back(ptPoint);
	      else
		gISvdEcalBrem.addPointRight( *ptPoint);
	    }
	}
      ptPoint = 0;
    }  

  //--------------------------------------------------------------------
  // do Matching 
  //--------------------------------------------------------------------

  //--------------------------------------------------------------------
  // I step : search for electron/positron candidates
  //--------------------------------------------------------------------

  if( matchWithMomentum )
    gISvdEcalMatch.matchWithMomentum();
  else
    gISvdEcalMatch.matchWithEcluster();


  //--------------------------------------------------------------------
  // Ia step : analysis of charge pair, candidates for bremstrahlung
  //--------------------------------------------------------------------

  unsigned int charge = gISvdEcalMatch.matchList().size();
  if( charge == 0 )
    {
      //cout << " no charge candidate is found " << endl;
      //gISvdEcalMatch.stat(2);
      //gISvdEcalMatch.stat(MARPLE_DEBUG,0);
      gISvdEcalMatch.deleteAll();  
      gISvdEcalBrem.deleteAll();  
# ifdef MARPLEPERFORMANCES
      chMatchSvdEcal->stop();
# endif
      maPresentMatch = MARPLENOMATCH;
      return ;
    }
  
  if( matchBrem )
    {

      // interesting entry for bremstrahlung search
      for( unsigned int iMatch = 0;  iMatch<charge; iMatch++)
	{
	  //  if( gISvdEcalMatch.matchList()[iMatch]->quality() > 0.99 ) continue;
	  if( matchWithMomentum )// check that 5GeV < p < 400 GeV
	    {
	      if( gISvdEcalMatch.leftOrigin(iMatch)->pe() > .2 ) continue; 
	      if( gISvdEcalMatch.leftOrigin(iMatch)->pe() < .0025 )continue; 
	    }
	  int found = 0;
	  std::vector< RcSegPoint *> vecTmp =  gISvdEcalBrem.vectorLeft();
	  for( unsigned int iBremSi = 0;  
	       !found && iBremSi < vecTmp.size(); iBremSi++ )
	    {
	      if(  gISvdEcalMatch.leftOrigin(iMatch)->link() == 
		   vecTmp[iBremSi]->link() ) found = 1;
	    }
	  
	  if( !found ) 
	    {
	      RcSegPoint * ptToPoint = new 
		RcSegPoint(      * gISvdEcalMatch.leftOrigin(iMatch));
	      //ptToPoint->lineswim(gISvdEcalBrem.ptToMatchConst()->z_plane);
	      ptToPoint->prolongate(gISvdEcalBrem.ptToMatchConst()->z_plane,
				    iErr,
				    1); // linear prolongation
	      
	      gISvdEcalBrem.addPointLeft( *ptToPoint );
	      ptToPoint = 0;	
	    }
	}
    
  //cout << " number of electrons" << electronVec.size() 
  //   << " " << positronVec.size() << endl;
  
  //--------------------------------------------------------------------
  // II step : search for bremstralung
  //--------------------------------------------------------------------
  
      gISvdEcalBrem.matchBrem();

    }
  //--------------------------------------------------------------------
  // outprint
  //--------------------------------------------------------------------


  if( MARPLE_DEBUG >= 20 )
    {
      gISvdEcalMatch.fillMatch(8116);
     if( matchBrem) gISvdEcalBrem.fillDeltas(8217,8317);
      gISvdEcalMatch.fillMatch(8116);
     if( matchBrem) gISvdEcalBrem.fillDeltas(8217,8317);
    }else  if( MARPLE_DEBUG >= 10 ){
      gISvdEcalMatch.fillDeltas(8216); // deltas, pulses
    if( matchBrem)  gISvdEcalBrem.fillDeltas(8217); //, 8317);
    }

  gISvdEcalMatch.stat();
  if( matchBrem) gISvdEcalBrem.stat();
    
  
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) 
    {
      gISvdEcalMatch.saveRSEG(iErr);
      gISvdEcalBrem.saveRSEG(iErr);
    }

  if( !MARPLE_BKGR )
    {
      gISvdEcalMatch.saveRelation(iErr);
      if( iErr ){ cout << "MARPLE: some relations were not saved " << endl;}
      gISvdEcalBrem.saveRelation(iErr);
      if( iErr ){ cout << "MARPLE: some relations were not saved 1 " << endl;}
    }
  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  gISvdEcalMatch.deleteAll();  
  gISvdEcalBrem.deleteAll();  

  maPresentMatch = MARPLENOMATCH;

# ifdef MARPLEPERFORMANCES
    chMatchSvdEcal->stop();
# endif
}
