/* given function replaces the rcmesm.F routine which returns average 
   MS-angle squared for the given superlayer (GESL).

   the previous version calculate average according to the volume covered
   while this look into the table built from 99.0811 geometry

   The rule to calculate parameter is : 
   MS_angle/P = 13.6e-3 [GeV]*sqrt(Xrad)*(1+0.038*ln(Xrad))

   MS2 = (MS_angle/P)*(MS_angle/P)

   created by Olya Igonkina, 22.07.2000
*/

#include <math.h>
#include "arte/arte.hh"
#include "marple/marple.hh"

double maMs2FromRad(float radThickness)
{
  if( radThickness<0.1e-10 ) return 0;
  double ms = 0.0136*sqrt(radThickness)*(1+0.038*log(radThickness));
  return ms*ms;
}

double maEnergyLossFromRad(float radThickness)
{
  if( radThickness<0.1e-10 ) return 0;
  return 0.048*radThickness; // coefficient from ranger (Node.C)
}

extern "C"
{
  void FORTRAN_ROUTINE(rcmesm)(int * iagesl, double * MS2, double * eLoss)
  {
    if( *iagesl <= 0 ||  *iagesl > (int)ArteTable<GESL>::size() )
      {
	cout << " RCMESM : superlayer " << *iagesl << " does not exist\n";
	*MS2 = 0;
	*eLoss = 0;
	return;
      }

    ArtePointer<GESL> gesl(*iagesl);
    char name[5];
    strncpy(name, gesl->name,4);
    name[4] = '\0';

    double rad;
    if(      !strncmp(gesl->name,"SI01",4) ){
      rad =  0.9375661E-02;
    }else if( !strncmp(gesl->name,"SI02",4) ){
      rad =  0.1299026E-01  ;
    }else if( !strncmp(gesl->name,"SI03",4) ){
      rad =  0.1337214E-01  ;
    }else if( !strncmp(gesl->name,"SI04",4) ){
      rad =   0.1631384E-01   ;
    }else if( !strncmp(gesl->name,"SI05",4) ){
      rad =  0.1773358E-01 ;
    }else if( !strncmp(gesl->name,"SI06",4) ){
      rad =  0.1760886E-01  ;
    }else if( !strncmp(gesl->name,"SI07",4) ){
      rad =  0.1824865E-01 ;
    }else if( !strncmp(gesl->name,"SI08",4) ){
      rad =  0.6003235E-02 ;

    }else if( !strncmp(gesl->name,"MC01",4) ){
      rad =  0.9644634E-02 ;
    }else if( !strncmp(gesl->name,"MC02",4) ){
      rad =  0.7701279E-02  ;
    }else if( !strncmp(gesl->name,"MC03",4) ){
      rad =  0.2242219E-01;
    }else if( !strncmp(gesl->name,"MC04",4) ){
      rad =  0.2187796E-01 ;
    }else if( !strncmp(gesl->name,"MC05",4) ){
      rad =  0.1694955E-01 ;
    }else if( !strncmp(gesl->name,"MC06",4) ){
      rad = 0.1953933E-01 ;
    }else if( !strncmp(gesl->name,"MC08",4) ){
      rad =  0.2231555E-01 ;
      
    }else if( !strncmp(gesl->name,"PC01",4) ){
      rad =  0.4045570E-01 ;
    }else if( !strncmp(gesl->name,"PC02",4) ){
      rad =  0.3602982E-01 ;
    }else if( !strncmp(gesl->name,"PC03",4) ){
      rad =  0.3595422E-01 ;
    }else if( !strncmp(gesl->name,"PC04",4) ){
      rad =  0.3989334E-01 ;

    }else if( !strncmp(gesl->name,"TC01",4) ){
      rad =  0.3470742E-01 ;
    }else if( !strncmp(gesl->name,"TC02",4) ){
      rad =  0.3464771E-01 ;

    }else if( !strncmp(gesl->name,"MS01",4) ){
       rad =  0.3571812E-01 ;
    }else if( !strncmp(gesl->name,"MS03",4) ){
       rad =  0.2375770E-01 ;
    }else if( !strncmp(gesl->name,"MS05",4) ){
      rad =  0.2365420E-01 ;
    }else if( !strncmp(gesl->name,"MS06",4) ){
      rad =  0.2414777E-01 ;
    }else if( !strncmp(gesl->name,"MS10",4) ){
      rad =  0.2250406E-01 ;
    }else if( !strncmp(gesl->name,"MS11",4) ){
       rad =  0.2096947E-01 ;
    }else if( !strncmp(gesl->name,"MS12",4) ){
       rad =  0.2089524E-01 ;
    }else if( !strncmp(gesl->name,"MS13",4) ){
      rad =  0.2219165E-01 ;
    }else if( !strncmp(gesl->name,"MS14",4) ){
      rad =  0.2257400E-01 ;
    }else if( !strncmp(gesl->name,"MS15",4) ){
      rad =  0.2323382E-01 ;
    
    }else if( !strncmp(gesl->name,"PTL1",4) ){
      rad =  0.8368089E-02 ;
    }else if( !strncmp(gesl->name,"PTL2",4) ){
      rad =  0.5258630E-02 ;
    }else if( !strncmp(gesl->name,"PTL3",4) ){
      rad =  0.8274696E-02 ;

    }else if( !strncmp(gesl->name,"RICH",4) ){
      rad =  0.1418770 ;
    }else if( !strncmp(gesl->name,"TR01",4) ){
      rad =  0.4420100E-01/2. ;
    }else if( !strncmp(gesl->name,"TR02",4) ){
      rad =  0.4420100E-01/2. ;
      
    }else if( !strncmp(gesl->name,"CALW",4) ){
      rad =  20 ;
    }else{      
       if( strncmp(gesl->name,"MOTH",4) && 
	   strncmp(gesl->name,"MAGN",4) && 
	   strncmp(gesl->name,"FELD",4) && 
	   strncmp(gesl->name,"TRSL",4) && 
	   strncmp(gesl->name,"MUON",4) )	   
	 cout << " RCMESM skip superlayer " << name << endl;
       rad = 0;
    }


    *MS2 = maMs2FromRad( rad);
    *eLoss = maEnergyLossFromRad( rad);
    
    return;
  }
}
