/************************************************************************
*
* MUDbAll.C
* 
* first author : Jing-Ge Shiu 
* purpose      : C++/SLT code to get the muon cable configuration from Boris's MuDb routines.
* called from  : anyone which needs this routine
* includ file  : MUDbAll.h (contains MuDbTubeArte.h, MuDbPadArte.h, MuDbPixelArte.h)
* start date   : forgot!!  Sometime before 2000 April.
*
*-----------------------------------------------------------------------
*    2000, 0627  physjg
*                add getFEDid(...) to extract the FEDid for a chamber readout 
* 
*    2000, 1014  physjg
*                add the return values from MuDbxxx::Ini and print out warning message 
*                if they are not zero.
* 
*    2000, 1017  physjg  
*                first release 
* 
***************************************************************************/
#include <stdio.h>
#include <iomanip.h>
#include <stdlib.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MUDbAll.h"


MUDbAll::MUDbAll( ) 
         : MuDbTubeArte(),MuDbPadArte() ,
           MuDbPixelArte(),MuPixChamber() {
	     
  char* DBname = "/MUON_DB"; 
  setDBname(DBname);  
  setKTstatus(-3);
  setRun(0);
  getEndian();
  initialized = 0;

}

MUDbAll::MUDbAll(int run, char* DBname ) 
         : MuDbTubeArte(),MuDbPadArte() ,
           MuDbPixelArte(),MuPixChamber() {
    
  setDBname(DBname);
  setKTstatus(-2);
  setMinor(0);
  setRun(run);
  getEndian();
  initialized = 0;

}

MUDbAll::MUDbAll(int keyMajor, int keyMinor, char* DBname ) 
         : MuDbTubeArte(),MuDbPadArte() ,
           MuDbPixelArte(),MuPixChamber() {
    
  setDBname(DBname);
  setKTstatus(2);
  setMinor(keyMinor);
  getEndian();
  initialized = 0;

}


MUDbAll::~MUDbAll( ) {

}


void MUDbAll::setDBname(char* DBname) {
  MuDbTubeArte::setDBname(DBname);
  MuDbPadArte::setDBname(DBname);
  MuDbPixelArte::setDBname(DBname);
return;
}

void MUDbAll::setKTstatus(int KTstatus) {
  MuDbTubeArte::setKTstatus(KTstatus); 
  MuDbPadArte::setKTstatus(KTstatus);
  MuDbPixelArte::setKTstatus(KTstatus);
return;
};

void MUDbAll::setMinor(int KTminor)  {
  MuDbTubeArte::setMinor(KTminor); 
  MuDbPadArte::setMinor(KTminor);
  MuDbPixelArte::setMinor(KTminor);
return;
};


void MUDbAll::setRun(int Run) {
  MuDbTubeArte::setRun(Run); 
  MuDbPadArte::setRun(Run);
  MuDbPixelArte::setRun(Run);
return;
};


int  MUDbAll::ini() {
  int rett, retp, retx;

  if ( (rett = MuDbTubeArte::Ini())!= 0 )   { arteMsg << "MUDbAll : MuDbTube is not initialized"  << endl; arteMsg.warn(); }
  if ( (retp = MuDbPadArte::Ini()) != 0 )   { arteMsg << "MUDbAll : MuDbPad is not initialized"   << endl; arteMsg.warn(); }
  if ( (retx = MuDbPixelArte::Ini()) != 0 ) { arteMsg << "MUDbAll : MuDbPixel is not initialized" << endl; arteMsg.warn(); }
  initialized = 1;

return rett|retp|retx;

}



Word MUDbAll::getFED(Word partid, Word fedID[]) {
 
  Word fedNO=0;
  fedno[0] = fedno[1] = fedno[2] = 0;
 
  if (initialized==1) {
    if (partid&0x01) {
        for ( Word sl=0; sl<MuDbTubeArte::TubeGeoSl; sl++ ) {
           for ( Word lay=0; lay<MuDbTubeArte::TubeGeoLay; lay++) {
              for (Word ud=0; ud<MuDbTubeArte::TubeGeoUpDown; ud++) {  
   	         for (Word nx=0;nx<MuDbTubeArte::NchambT[sl][lay][ud]; nx++) {  
		    fedID[fedNO] = MuDbTubeArte::CrateT[sl][lay][ud][nx];
                    Word fn;
                    for (fn=0; fn<fedNO ; fn++) {
                       if (fedID[fedNO] == fedID[fn]) break;
                    }
                    if (fn>=fedNO) {
                       fedNO++; 
                    }   
                 }
	      }
           }
        };
        for ( Word sl=0; sl<MuDbPadArte::PadGeoSl; sl++ ) {
           for (Word ud=0; ud<MuDbPadArte::PadGeoUpDown; ud++) {  
   	      for (Word nx=0;nx<MuDbPadArte::NchambP[sl][ud]; nx++) {  
   		 fedID[fedNO] = MuDbPadArte::CrateP[sl][ud][nx];
                 Word fn;
                 for (fn=0; fn<fedNO ; fn++) {
                    if (fedID[fedNO] == fedID[fn]) break;
                 }
                 if (fn>=fedNO) {
                    fedNO++; 
                 }
	      }
           }
        };
        fedno[0] = (fedNO>MAXMUFED)?MAXMUFED:fedNO;
        for (int i=0;i<fedNO;i++) {
            fedid[0][i] = fedID[i];
        }
     };  
    
     if (partid&0x02) {
        for ( Word sl=0; sl<MuDbPadArte::PadGeoSl; sl++ ) {
           for (Word ud=0; ud<MuDbPadArte::PadGeoUpDown; ud++) {  
   	      for (Word nx=0;nx<MuDbPadArte::NchambP[sl][ud]; nx++) {  
		 fedID[fedNO] = MuDbPadArte::CratePP[sl][ud][nx];
                 Word fn;
                 for (fn=0; fn<fedNO ; fn++) {
                    if (fedID[fedNO] == fedID[fn]) break;
                 }
                 if (fn>=fedNO) {
                    fedNO++; 
                 }  
	      }
           }
        };
        fedno[1] = fedNO - fedno[0];
        for (int i=0;i<fedno[1];i++)
            fedid[1][i]=fedID[fedno[0]+i];

     }

     if (partid&0x04) {
        for ( Word sl=0; sl<MuDbPixelArte::PixGeoSl; sl++ ) {
           for ( Word qu=0; qu<MuDbPixelArte::PixGeoQu; qu++) {
   	      for (Word nc=0;nc<MuDbPixelArte::PixNcon; nc++) {
		 if ( MuDbPixelArte::MaskX[sl][qu][nc]>0 ) {  
		    fedID[fedNO] = MuDbPixelArte::CrateX[sl][qu][nc];
                    Word fn;
                    for (fn=0; fn<fedNO ; fn++) {
                       if (fedID[fedNO] == fedID[fn]) break;
                    }
                    if (fn>=fedNO) {
                       fedNO++; 
                    }
                 }   
              }
           }
        };
        fedno[2] = fedNO - fedno[0]  - fedno[1];
        for (int i=0;i<fedno[2];i++)        
            fedid[2][i]=fedID[fedno[0]+fedno[1]+i];

      }
  } else {
    cout << "NOT INITIALIZED YET!!!" << endl;
  };

 return fedNO;

}


Word MUDbAll::fedID(Word fedpick, Word fedID[]) {

  Word fedNO=0;

  if (fedpick&0x01) {
     for (int i=0;i<fedno[0];i++)
        fedID[i]=fedid[0][i];
     fedNO+=fedno[0];
  }
  if (fedpick&0x02) {
     for (int i=0;i<fedno[1];i++)
        fedID[i+fedNO]=fedid[1][i];
     fedNO+=fedno[1];
  }
  if (fedpick&0x04) {
     for (int i=0;i<fedno[2];i++)
        fedID[i+fedNO]=fedid[2][i];
     fedNO+=fedno[2];
  }

  return fedNO;

}


int MUDbAll::getEndian() {

 sysendian = MuDbTubeArte::getEndian();
 return sysendian;

}

void MUDbAll::print(Word partid) {

  if (partid&0x01) MuDbTubeArte::Print();
  if (partid&0x02) MuDbPadArte::Print();
  if (partid&0x04) MuDbPixelArte::Print();

 return ;

}
