      subroutine diotr
************************************************************************
* This routine shows number of passes of OTR single layers
* for a straight track coming from target (x=0,y=0,z=0).
* Counting is done only for a given superlayer.
*
* Function otr2hit is used.
*
* Parameters read from KUIP:
*   SL        - superlayer name (can be generic, e.g. PC or ALL)
*   lay       - stereolayer     (use 0 for all stereolayers)
*   Usemask   - non-zero if channel mask should be used (default is non-zero)
*               (=1 - count good cells)
*               (=2 - count bad cells)
*   Countcell - non-zero if cells should be counted (default is single layers)
*
* If SL='ALL' then all OTR superlayer are selected.
* If SL='PC' then all PC superlayer are selected.
*
* Author:   Alexander Lanyov      April 2001
*
************************************************************************
      implicit none

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/GEO.fi"
#include "pris/zgraf.fi"
#include "gean/ZGESL.fi"

      integer lay,l,i,isl,nx,ny,ix,iy,n,nmax,nmin
      integer pl1,pl2,plmin,plmax,usemask,countcell

       parameter (nx=100,ny=100)
c      parameter (nx=300,ny=300)
c      parameter (nx=1000,ny=1000)

      real x1,x2,y1,y2,z1,z2,v(nx,ny),z0,tx,ty,x,y,z
      character slnam*4,name*4,buf*100
      logical first
      data first /.true./

      integer otr2hit,lenocc

#include "arte/Arte.ff"
#include "auto/GEO.ff"

      call kugetc(slnam,l)
      call kugeti(lay)

      call kugeti(usemask)
      call kugeti(countcell)

c------------------------------------------------------ Find superlayer
      if (slnam(:3).eq.'ALL') then
         write (*,*) 'Matching all OTR superlayers'
         plmin = 1
         plmax = ngede()
         goto 10
      end if

      plmin = 100000
      plmax = -1
      do isl=1,ngesl()
         call uhtoc(iq(hqgesl_name(isl)),4,name,4)
         if (slnam(:lenocc(slnam)) .eq. name(:lenocc(slnam))) then
            write (*,*) 'Matched superlayer ',name
            pl1 = igesl_fpl(isl)
            pl2 = pl1 + igesl_npl(isl) - 1
            plmin = min(pl1, plmin)
            plmax = max(pl2, plmax)
         end if
      end do

      if (plmin.gt.plmax) then
         write (*,*) 'Superlayer ',slnam,' is not found!'
         return
      end if

 10   continue

c------------------------------------------------------ Loop on planes
      call iselnt(nt_detzg)

      z1 =  10000.
      z2 = -10000.

      do i=plmin,plmax
         if (igede_cmp(i).ne.lgesl_wch) goto 99
         if (igede_sec(i).eq.0) goto 99
         if ((lay.ne.0 .and. igede_lay(i).ne.lay)) goto 99
         z1 = min(z1,fgede_zmin(i))
         z2 = max(z2,fgede_zmax(i))
 99      continue
      end do

      x1 = x1zg
      x2 = x2zg
      y1 = y1zg
      y2 = y2zg

      z0 = 0.5*(z1+z2)
      write (*,22) z1,z2,z0
 22   format(' Matched Z coordinate range:',f7.1,' -',f7.1,
     @  ', mean value',f7.1,' cm')

      nmax = 0.
      nmin = 2**30
      do ix=1,nx
         do iy=1,ny
            x = x1 + (ix-0.5)*(x2-x1)/nx
            y = y1 + (iy-0.5)*(y2-y1)/nx
            tx = x/z0
            ty = y/z0
            n = otr2hit(0.,0.,tx,ty,z1,z2,usemask,countcell)
            v(ix,iy) = n
            nmax = max(nmax,n)
            nmin = min(nmin,n)
c            if (n.ne.0) write (*,*) 'x=',x,' y=',y,' n=',n
         end do
      end do

      if (countcell.eq.0) then
         write (*,*) 'Maximum value of single layers is',nmax
      else
         write (*,*) 'Maximum value of cells is',nmax
      end if

      if (nmin.ne.0) then
         write (*,*) '                  min =',nmin
      end if 
      call igset('PLCI', 1.)    ! line color
      call igtabl(nx,ny,v,0,0,'COL,Z')

      call iselnt(nt_pictzg)
      call igset('PLCI', 1.)    ! line color
      call igset('TXFP',-62.)   ! text font
      call igset('CHHE', 1.2)   ! Character height
      call igset('TXAL', 13.)   ! alignment: left mid
      do iy=0,8  
         write (buf,'(f6.1)') real(iy/8.)*real(nmax)
*         call itx(105.,21.+55.5/8.*iy,buf(:lenocc(buf)))
         call itx(105.,21.+8.*iy,buf(:lenocc(buf)))
      end do
c-----------------------------------------------------
 
*     **** header text
      if (lheaderzg.and.index(choptzg,'H').eq.0
     +     .and.index(choptzg,'A').eq.0) call dihead()
 
*     **** complete frame
      if (lframezg.and.index(choptzg,'A').eq.0) call diedge(choptzg)
c-----------------------------------------------------
 
      call iselnt(nt_framzg)         ! switch to Prism NT
      call iuwk(0,1)
      return
      end
