// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Slight modification in inclusion of header files
//                                      14-Aug-00 Wouter Hulsbergen
//
// - Introduced new data members float xomin, yomin, xomax, yomax that
// represent box around volume in ACS coorindates. These coordinates
// are used to check whether volume `migh be hit' (withinBox routin)
// before rotating it. The rangerVolume::withinBox routine is an
// inline call which saves much in execution speed. For the moment the
// xy-box is only implemented for VolBox.
//
//                                      16-July-2001 Wouter Hulsbergen
// class rangerVolume
// base class for detector volume

#ifndef rangerVolume_HH
#define rangerVolume_HH

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef rangerGeWa_HH
#include "rang/GeWa.hh"
#endif
#ifndef rangerState_HH
#include "rang/State.hh"
#endif

class rangerNode;
class rangerArrow;

class rangerVolume {

public:
  rangerVolume();                 // default constructor
  rangerVolume(rangerGeWa* gewa); // constructor from gewa
  virtual ~rangerVolume();        // virtual destructor
  const int plNumber() const {return Gewa->plNumber();}; // mangled id
  const ArtePointer<GEDE> getGede() const {return Gewa->getGede();}; // 
  const ArtePointer<GWAL> getGwal() const {return Gewa->getGwal();}; // 
  const rangerGeWa* getGeWa() const {return Gewa;}; // return GeWa
  
  float Cos() const {return rotTot[0][0];}; // total cos of 
  //projected angle
  float Sin() const {return rotTot[1][0];}; // total sin of 
  //projected angle
  float ms() const {return msValue;}  // multiple scattering
  void setMs(float ms) { msValue = ms ; }

  // return the reference z value in the ACS
  float zMean() const { return zA ; }
                                           
  // fast routine which checks if this volume is `almost' hit
  // works only for VolBox (!)
  inline bool withinBox(float z, float x, float y, float tx, float ty) const {
    return xA1 <= x+tx*(zA-z) && x+tx*(zA-z) <= xA2 && 
      yA1 <= y+ty*(zA-z) && y+ty*(zA-z) <= yA2 ; }
  
  inline float sLengthFast(float z, float x, float y, float tx, float ty) const {
    return withinBox(z,x,y,tx,ty) ? sLength(z,x,y,tx,ty) : 0 ; }
  
  inline float sLengthFast(const rangerState& s) const {
    return sLengthFast(s.z(),s.x(),s.y(),s.tx(),s.ty()) ; }
  
  static rangerVolume* makeVolume( rangerGeWa* gewa);

  // calculate length of segment through volume
  virtual float sLength(float z, float x, float y, float tx, float ty) const ;
  
protected:
  // rotate a ranger arrow from ACS to PCS (+1) or vise versa (-1)
  void rotateArrow(rangerArrow& a,int direction=+1) const ;

  // pointer to GeWa
  rangerGeWa* Gewa;  
  float zA ;    // z-coordinate in ACS
  float xA1 ;   // smallest box that contains volume in ACS
  float yA1 ;
  float xA2 ;
  float yA2 ;

private:
  float rotTot[3][3]; // total rotation matrix
  float msValue;      // mean theta-proj**2 * p**2
  int rotated;        // true if rot() ne 1
  int zTilted;        // true if also z axis is tilted
  int totRotated;     // true if rotTot() ne 1
};

#endif
