// 
//  Author:  Rainer Mankel                     Created:  25-Jan-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// rginit_ 
// Standard self-initialization of ranger, which initializes the basic
// constants (zfpar,zfmode,zfout,zfconst). Must be called prior to any 
// pattern recognition or track fitting operation. Action only on first
// invocation, further calls have no effect.

// Note: 
// - many user-callable tracking machines call rginit themselves
//   so that initialization is then ensured.
// - to set/change parameters in zfpar,zfmode,zfout,zfconst, make sure
//   rginit has been called before so that constants are initialized.
//   Otherwise, automatic initialization is likely to overwrite your settings.
 
#include <iostream.h>

extern "C" {
  void tf1init_();
  void rgextini1_();
}


extern "C" {
  void rginit_() {

    // self-initialization routine of ranger

    // action occurs only in the first event

    static int init = 0;

    if (! init) {
      //cout << "rginit: ranger self-initialization" << endl;
      tf1init_();
      rgextini1_();
      init = 1;
    }
    //else {
    //  cout << "rginit: ranger is already initialized" << endl;
    //} 
  }
}
