
// *****************************************************************************
//
// source:    rcClassJPSIEE.C
//
// type:      source code
//
// created:   20 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcClassJPSIEE
//
// called by: rcClassify
//
// purpose:   check for J/Psi -> e+e- event
//
// hints:     do NOT set flagdst here!
//
// changes:
// 20 Jun 2000 (AG) creation
// 20 Jul 2000 (AG) add ARTE_mark()
// 14 Nov 2000 (AG) added global massJPSIEE
// 30 Nov 2000 (AG) copied from MV
// 01 Dec 2000 (AG) removed initialization bug of mass
// 14 Dec 2000 (MV) Selection made on fit, fid & lee fields of RTRA
// 15 Dec 2000 (AG) added debug printout for mass
// 03 Jul 2001 (MV) Added selection on dielectron identified
//
// *****************************************************************************

// standard includes

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// externals
extern int   FADEB;
extern float cutJPSIEE;
extern float massJPSIEE;
extern int   rcSelectTracksStatus;
extern float massDIELEID;
//
// function code
//
int
rcClassJPSIEE()
{
  ARTE_mark( "rcClassJPSIEE 0", 0 );

  // init
  massJPSIEE = 0.0; // reset mass
  massDIELEID= 0.0;

  // check track selection (Dorothea Samtleben)
  int rc = 0;  
  if( rcSelectTracksStatus ){
    
    // loop over tracks
    for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
	 rtra != ArteTable<RTRA>::end();
	 ++rtra ){
      if( (*rtra)->fit==2 && ((*rtra)->fid==443 || (*rtra)->fid==1443) ){
	 if( (*rtra)->lee >massJPSIEE ) massJPSIEE = (*rtra)->lee;
	 if( (*rtra)->lemi >massDIELEID ) massDIELEID = (*rtra)->lemi;
      }
    }
    if( massJPSIEE>cutJPSIEE ) rc = 1; // found!
  }

  // print
  if( FADEB ) cout << "JPSIEE: mass=" << massJPSIEE << endl;
  
  ARTE_mark( "rcClassJPSIEE 1", 0 );
  return rc;
}
