// ********************************************************************
//
//  Routine:      rcDoMatchStep.C
//  Author:       O.Igonkina
//
//  Description:  replacement of rcMatch() routine : make Match in 2 steps
//                here the calls to fit98 is removed
//
//  Modification:
//  1.11.2001 if MATCH SVD-ECAL is required
//            the high Et clusters are matched to SVD independent on RTRA (OI)
//
// ********************************************************************
#include <iostream.h>
#include "marple/marple.hh"

#include "arte/arte.hh"
#include "reco/rcflags.hh"

extern "C" void rgdownstr_();
extern float MARPLE_SEET; // default value is set in marple package
extern int MARPLE_SE_SVDNORTRA ;
extern int MARPLE_SE_ECALNORTRA;
extern "C" void matchrtrarich_();   
extern int matchMARPLERtraRich ;
//
// function
//
int
rcDoMatchStep(int step)
{

//-----------------------------------------------------------------
//  step I : call matches for SVD-OTR
//-----------------------------------------------------------------
  if ( step == 1 || step == 0) 
    {
      if ( matchMARPLEVdsOtr  )  matchsvdot_();
      if ( matchMARPLEVdsPc   )  matchsvdpc_();
      //-----------------------------------------------------------------
      //  propogation of VDS tracks downstream 
      rgdownstr_();
    }
//-----------------------------------------------------------------
//  step II : call all other matches except SVD-OTR
//-----------------------------------------------------------------
  if ( step == 2 || step == 0) 
    {
     
      if ( matchMARPLEVdsEcal )  
	{
	  matchsvdecal_(& MARPLE_SEET); 

	  // check in addition that all High Et cluster was matched to all SVD
	  // to recover all J/psi. 
	  // notice that it might create double relations SVD-ECAL

	  if( MARPLE_SE_SVDNORTRA != 0 || MARPLE_SE_ECALNORTRA != 0  ) 
	    {
	      int oldSNORTRA = MARPLE_SE_SVDNORTRA;
	      int oldENORTRA = MARPLE_SE_ECALNORTRA;
	      MARPLE_SE_SVDNORTRA = 0;
	      MARPLE_SE_ECALNORTRA= 0;
	      float Etcut = 0.9;
	      matchsvdecal_(&Etcut); 	      
	      MARPLE_SE_SVDNORTRA = oldSNORTRA;
	      MARPLE_SE_ECALNORTRA= oldENORTRA;	      
	    }

	}
      if ( matchMARPLEVdsRich )  matchsvdrich_();
      if ( matchMARPLEOtrRich )  matchotrrich_();
      if ( matchMARPLEOtrEcal )  matchotrecal_();
      if ( matchMARPLEEcalRich ){
	static float Ktrig = 0.0;
	static float RichSb = 0.0;
	matchrichecal_(& Ktrig, & RichSb);
      };
    }
//-----------------------------------------------------------------
//  step III : call RTRA-RICH match
//-----------------------------------------------------------------
  if ( step == 3 )
    {
      if ( matchMARPLERtraRich )  matchrtrarich_();
    }

  //-----------------------------------------------------------------
  // wrong parameter of step
  //-----------------------------------------------------------------
  if( step != 0 && step != 1 && step != 2 && step !=3 ) 
    {
      cout << "ERROR : rcDoMatchStep : step "<< step << " is not implemented\n";
    }

return 0;
};
