#include <reco/rcHitPattern.hh>

namespace rcHitPattern
{
  unsigned int nnonzero(unsigned int bitpattern) {
    // Returns the number of nonzero bits in int.
    unsigned int rc=0;
    for( int bit=0; bit<32; bit++)
      if( bitpattern & (1<<bit) ) ++rc ;
    return rc ;
  }
  
  unsigned int nplanevds(ArtePointer<RTRA> rtra) {
    // Returns the number of vds planes traversd by rtra.
    return nnonzero(rtra->bse) ;
  }
  
  unsigned int nhitplanevds(ArtePointer<RTRA> rtra) {
    // Returns the number of trversed vds planes with reconstructed hit.
    return nnonzero(rtra->bse&rtra->bsi) ;
  }

  unsigned int nplanepc(ArtePointer<RTRA> rtra) {
    // Returns the number of pc planes traversd by rtra.
    ArtePointer<RTRE> rtre(rtra.index()) ;
    return nnonzero(rtre->pche) ;
  }
  
  unsigned int nhitplanepc(ArtePointer<RTRA> rtra) {
    // Returns the number of trversed pc planes with close hit.
    ArtePointer<RTRE> rtre(rtra.index()) ;
    return nnonzero(rtre->pche&rtre->pchi) ;
  }

  unsigned int nplanetc(ArtePointer<RTRA> rtra) {
    // Returns the number of tc planes traversd by rtra.
    ArtePointer<RTRE> rtre(rtra.index()) ;
    return nnonzero(rtre->tche) ;
  }
  
  unsigned int nhitplanetc(ArtePointer<RTRA> rtra) {
    // Returns the number of trversed tc planes with close hit.
    ArtePointer<RTRE> rtre(rtra.index()) ;
    return nnonzero(rtre->tche&rtre->tchi) ;
  }

  unsigned int nplanemc(ArtePointer<RTRA> rtra) {
    // Returns the number of mc planes traversd by rtra.
    ArtePointer<RTRE> rtre(rtra.index()) ;
    return nnonzero(rtre->mche) ;
  }
  
  unsigned int nhitplanemc(ArtePointer<RTRA> rtra) {
    // Returns the number of trversed mc planes with close hit.
    ArtePointer<RTRE> rtre(rtra.index()) ;
    return nnonzero(rtre->mche&rtre->mchi) ;
  }
}
