//--*-C++-*--
//
//  Created: May 11 2001
//
//  Modifications:
// 
//  2001/05/15   DD  Bug fix in setting probTypeGlobal
//  2001/05/17   DD  Printout added
//  2001/05/23   DD  okL added
//  27 Jun 2001 (DD) alternative track selection added
//  09 Jul 2001  DD  tuney added for signal yield tuning
//  14 Jul 2001  dd  seed sigma enhanement added
//  14 Jul 2001  DD  defaults changed
//  09 Aug 2001  IR  change RAttribute::deltaR 0.007 to 0.0035 (temporarily)
//  
#include <iostream.h>
#include "rise/RAttribute.h"


int   RAttribute::verb      = 0;
float RAttribute::deltaR    = 0.0035;
bool  RAttribute::fitR      = true;
bool  RAttribute::fitX      = true;
bool  RAttribute::fitY      = true;
bool  RAttribute::normL     = true;
int   RAttribute::probType       = 2;
int   RAttribute::probTypeGlobal = 1;
bool  RAttribute::globalFit = true;
int   RAttribute::maxIter   = 15;
float RAttribute::smearx    = 0;
float RAttribute::smeary    = 0;
float RAttribute::maxR      = .060;
float RAttribute::minL      = -5.0;
float RAttribute::okL       = 2.5;
int   RAttribute::minROI    = 4;
int   RAttribute::track     = 0;
float RAttribute::tuney     = 0;
float RAttribute::enhtx     = 20;
float RAttribute::enhty     = 10;

void RAttribute::print() {
  cout<<"--------------------------------------------"<<endl;
  cout<<"      ..                     .  .           "<<endl;
  cout<<"    .    .   ...           .      .         "<<endl;
  cout<<"    .    .  .   .    ...   .       .        "<<endl;
  cout<<"     . .     ...   .     .  .     .         "<<endl;
  cout<<"                  .       .   . .           "<<endl;
  cout<<"   RING fitting    .     .  for RingSearch  "<<endl;
  cout<<"                     . .                    "<<endl;
  cout<<"                                            "<<endl;
  cout<<" verbosity .......................... "<<RAttribute::verb<<endl;
  cout<<" ROI radius ......................... "<<RAttribute::deltaR<<endl;
  cout<<" normalize .......................... "<<RAttribute::normL<<endl;
  cout<<" local fit type ..................... "<<RAttribute::probType<<endl;
  cout<<" global fit ......................... "<<RAttribute::globalFit<<endl;
  cout<<" global fit type .................... "<<RAttribute::probTypeGlobal<<endl;
  cout<<" fit radius ......................... "<<RAttribute::fitR<<endl;
  cout<<" fit horiz. angle ................... "<<RAttribute::fitX<<endl;
  cout<<" fit vert. angle .................... "<<RAttribute::fitY<<endl;
  cout<<" max. iterations .................... "<<RAttribute::maxIter<<endl;
  cout<<" min. photons in ROI ................ "<<RAttribute::minROI<<endl;
  cout<<" min. likelihood .................... "<<RAttribute::minL<<endl;
  cout<<" positive id likelihood ............. "<<RAttribute::okL<<endl;
  cout<<" track/seed selection ............... "<<RAttribute::track<<endl;
  cout<<" smearing tx (MC) ................... "<<RAttribute::smearx<<endl;
  cout<<" smearing ty (MC) ................... "<<RAttribute::smeary<<endl;
  cout<<" track selection type ............... "<<RAttribute::track<<endl;
  cout<<" yield tuning factor ................ "<<RAttribute::tuney<<endl;
  cout<<" sigma enhancement for tx/ty ........ "<<RAttribute::enhtx<<"/"<<RAttribute::enhty<<endl;
  cout<<"                                            "<<endl;
  cout<<"--------------------------------------------"<<endl;
}



extern "C" {
  void kugetr_(float &par);
  void kugeti_(int   &par);
  
  void ringpar_() {
    int   verb;   kugeti_(verb);
    float drad;   kugetr_(drad);
    int   normL;  kugeti_(normL);
    int   ptype;  kugeti_(ptype);
    int   ptypeg; kugeti_(ptypeg);
    int   global; kugeti_(global);
    int   fitR;   kugeti_(fitR);
    int   fitX;   kugeti_(fitX);
    int   fitY;   kugeti_(fitY);
    int   maxIt;  kugeti_(maxIt);
    float smearx; kugetr_(smearx);
    float smeary; kugetr_(smeary);
    float maxR;   kugetr_(maxR);
    float minL;   kugetr_(minL);
    float okL;    kugetr_(okL);
    int   minROI; kugeti_(minROI);
    int   track;  kugeti_(track);
    float tuney;  kugetr_(tuney);
    float enhtx;  kugetr_(enhtx);
    float enhty;  kugetr_(enhty);

    if (verb>-1)   { RAttribute::verb=verb;  cout<<"---> RING: Verbosity level set to "<<verb<<endl; }
    if (drad>0)    { RAttribute::deltaR=drad;cout<<"---> RING: Ring fit ROI set to "<<drad<<" rad"<<endl; }
    if (normL>-1)  { RAttribute::normL=normL;cout<<"---> RING: Normalization set to "<<normL<<endl; }
    if (ptype>-1)  { 
      RAttribute::probType=ptype;
      switch (ptype) {
      case 0: cout<<"---> RING: Radius only used in probability calculation"<<endl; break;
      case 1: cout<<"---> RING: Radius and momentum used in probability calculation"<<endl; break;
      case 2: cout<<"---> RING: Fitted L/Lb used in probablity calculation"<<endl; break;
      case 3: cout<<"---> RING: Chi2 used in probablity calculation"<<endl; break;
      default:cout<<"---> RING: Invalid code for probability calculation"<<endl;
      }
    }
    if (ptypeg>-1)  { 
      RAttribute::probTypeGlobal=ptypeg;
      switch (ptypeg) {
      case 0: cout<<"---> RING: Radius only used in probability calculation"<<endl; break;
      case 1: cout<<"---> RING: Radius and momentum used in global probability calculation"<<endl; break;
      default:cout<<"---> RING: Invalid code for probability calculation "<<ptypeg<<endl;
      }
    }
    if (global>-1) { RAttribute::globalFit=global; cout<<"---> RING: Global ring fit switched to "<<global<<endl; }
    if (fitR>-1)   { RAttribute::fitR=fitR;        cout<<"---> RING: Switch radius fitting: "<<fitR<<endl; }
    if (fitX>-1)   { RAttribute::fitX=fitX;        cout<<"---> RING: Switch X fitting: "<<fitX<<endl; }
    if (fitY>-1)   { RAttribute::fitY=fitY;        cout<<"---> RING: Switch Y fitting: "<<fitY<<endl; }
    if (maxIt>-1)  { RAttribute::maxIter=maxIt;    cout<<"---> RING: Maximum number of iterations: "<<maxIt<<endl; }
    if (smearx>-1) { RAttribute::smearx=smearx;    cout<<"---> RING: Tx track smearing: "<<smearx<<endl; }
    if (smeary>-1) { RAttribute::smeary=smeary;    cout<<"---> RING: Ty track smearing: "<<smeary<<endl; }
    if (maxR>-1)   { RAttribute::maxR=maxR;        cout<<"---> RING: Maximum radius: "<<maxR<<endl; }
    if (minL>-100) { RAttribute::minL=minL;        cout<<"---> RING: Minimum likelihood for accepting ring: "<<minL<<endl; }
    if (okL>-100)  { RAttribute::okL=okL;          cout<<"---> RING: Minimum likelihood for positive ID: "<<okL<<endl; }
    if (minROI>-1) { RAttribute::minROI=minROI;    cout<<"---> RING: Minimum hits in ROI: "<<minROI<<endl; }
    if (track>-1)  { RAttribute::track=track;      cout<<"---> RING: Track/seed selection: "<<track<<endl; }
    if (tuney>-1)  { RAttribute::tuney=tuney;      cout<<"---> RING: Ring yield tuning factor: "<<tuney<<endl; }
    if (enhtx>-1)  { RAttribute::enhtx=enhtx;      cout<<"---> RING: Seed sigma tx enhancement: "<<enhtx<<endl; }
    if (enhty>-1)  { RAttribute::enhty=enhty;      cout<<"---> RING: Seed sigma ty enhancement: "<<enhty<<endl; }
  }

}
