//-----------------------------------------------------------------------//
//
//  RingSearchDB: DB interface for stand-alone ring reconstruction
//
//  Author: Denis Dujmic
//
//  Created: May-99
//
//  Modifications:
//    May 27, 2000 (DD) RITH table used only in ARTE
//    Jun 09, 2000 (DD) parameters can be overwriten by kuip
//    Dec 13, 2000 (DD) DB update not possible in case of DQ test runs 
//                      (run number=-123456)
//
//-----------------------------------------------------------------------//
#include <stdio.h>
#include "rise/RingSearchDB.h"
#include "rich/ricutil.h"


int RingSearchDB::distribute=0;

void RingSearchDB::getSearchParameters(int run_no, int rev_no, int verb) {

#ifdef USE_ARTE
  // -- if RITH filled use stored parameters
  if (ArteTable<RITH>::size()) {
    ArtePointer<RITH> rith(1);
    if (RingSearchAttrib::overwriteMinSearchRadius<-100)
      RingSearchAttrib::minSearchRadius=rith->par[0];
    else
      RingSearchAttrib::minSearchRadius=RingSearchAttrib::overwriteMinSearchRadius;

    if (RingSearchAttrib::overwriteMaxSearchRadius<-100)
      RingSearchAttrib::maxSearchRadius=rith->par[1];
    else
      RingSearchAttrib::maxSearchRadius=RingSearchAttrib::overwriteMaxSearchRadius;

    if (RingSearchAttrib::overwriteSearchStep<-100)
      RingSearchAttrib::searchStep     =rith->par[2];
    else
      RingSearchAttrib::searchStep     =RingSearchAttrib::overwriteSearchStep;

    if (RingSearchAttrib::overwriteOccupancyCut<-100)
      RingSearchAttrib::occupancyCut   =int(rith->par[3]);
    else
      RingSearchAttrib::occupancyCut   =int(RingSearchAttrib::overwriteOccupancyCut);

    if (RingSearchAttrib::overwritePeakThreshold<-100)
      RingSearchAttrib::peakThreshold  =int(rith->par[4]);
    else
      RingSearchAttrib::peakThreshold  =int(RingSearchAttrib::overwritePeakThreshold);
      
    return;
  }
#endif


#ifndef USE_ARTE
  // -- check for testing of DQ software, do not update access DB in this case
  if (run_no==-123456) return;
#endif

  // --  OLD method if RITH not filled
  static int oldRun_no=0;
  static int oldRev_no=-1;
  if (oldRun_no==run_no && oldRev_no==rev_no) return;

  verb=1;////////////////////////////////////////////////////////////////////////////////////

  if (verb>0) cout<<" $$$ RISE [Inform]: Database will be accesed "<<endl;

  char dbsite[256];
  dbsite[0]=0;
  char *getdbsite=getenv("RICH_RECODB");
  if (!getdbsite) {
    if (verb>0) {
      fprintf(stderr," $$$ RISE Inform: RICH_RECODB not defined, using default values for RingSearch\n");
      fprintf(stderr," $$$ RISE Inform: %f %f %f %d %d\n",
	      RingSearchAttrib::minSearchRadius,RingSearchAttrib::maxSearchRadius,
	      RingSearchAttrib::searchStep,RingSearchAttrib::peakThreshold,RingSearchAttrib::occupancyCut);
    }
    return;
  }
  strcpy(dbsite,getdbsite);

  Tables x=OpenTables(dbsite,0);
  if (!x) { fprintf(stderr," $$$ getSearchParameters: Cannot open a table for run %d/table not found\n",run_no); Exit(1); }  

  int tx=BookTable(x,"RICH_RADIUS",0,0);
  if (!tx) { fprintf(stderr," $$$ getSearchParameters: Cannot book a table for run %d/table not found\n",run_no); Exit(2); }

  TableStamp(x,run_no,rev_no);
  if (!FetchTable(x,tx,TABLEle)) { fprintf(stderr," $$$ getSearchParameters: Cannot fatch a table for run %d/table not found\n",run_no); Exit(3); }

  SkipTableField(x,1);
  int   major=TableMajor(x);
  //int   minor=TableMinor(x);
  float radius,temperature,pressure,time;
  GetTableField(x,TABLEfloat,10000,&pressure);
  GetTableField(x,TABLEfloat,10000,&temperature);
  GetTableField(x,TABLEfloat,10000,&radius);
  GetTableField(x,TABLEfloat,10000,&time);
  GetTableField(x,TABLEfloat,10000,&RingSearchAttrib::minSearchRadius);
  GetTableField(x,TABLEfloat,10000,&RingSearchAttrib::maxSearchRadius);
  GetTableField(x,TABLEfloat,10000,&RingSearchAttrib::searchStep);
  GetTableField(x,  TABLEint,10000,&RingSearchAttrib::occupancyCut);
  GetTableField(x,  TABLEint,10000,&RingSearchAttrib::peakThreshold);

  //printf(" major minor x TABLEint %d %d %d %d \n",major,minor,(int)x,TABLEint);
  //printf("RUN %d:   P=%f  T=%f  r=%f  t=%f : %f %f %f : %d %d\n",
  //run_no,pressure,temperature,radius,time,MinSearchRadius,MaxSearchRadius,SearchStep,OccupancyCut,Threshold);

  if (verb>0) {
    fprintf(stdout," $$$ RISE Inform: RingSearch parameters changed (new reference run %d):\n",major); 
    fprintf(stdout," $$$ RISE Inform: lower radius=%f  upper radius=%f search step=%f threshold=%d occupancy cut=%d\n",
	    RingSearchAttrib::minSearchRadius,
	    RingSearchAttrib::maxSearchRadius,
	    RingSearchAttrib::searchStep,
	    RingSearchAttrib::peakThreshold,
	    RingSearchAttrib::occupancyCut);
  }

  if (verb>1) PrintTables(x,1);

  CloseTables(x);

  oldRun_no=run_no;
  oldRev_no=rev_no;
}



void RingSearchDB::fillSearchParameters(int run_no, float *parameters,int verb) {

#ifndef USE_ARTE
  // -- check for testing of DQ software, do not update anything in this case
  if (run_no==-123456) return;
#endif


  char dbsite[256];
  dbsite[0]=0;
  char *getdbsite=getenv("RICH_RECODB");
  if (!getdbsite) {
    if (verb>0) {
      fprintf(stderr," $$$ RingSearchDB [Warning]: RICH_RECODB not defined, using default values for RingSearch\n");
      fprintf(stderr," $$$ RingSearchDB [Warning]: %f %f %f %d %d\n",
	      RingSearchAttrib::minSearchRadius,RingSearchAttrib::maxSearchRadius,
	      RingSearchAttrib::searchStep,RingSearchAttrib::peakThreshold,RingSearchAttrib::occupancyCut);
    }
    return;
  }
  strcpy(dbsite,getdbsite);

  Tables x=OpenTables(dbsite,0);
  if (!x) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(1); }  

  if (distribute) {
    if (verb>0) cout<<" $$$ RingSearchDB [Inform]: >>>>>>> Distributing table <<<<<<<<"<<endl;
    DistributeTables(x);
  }

  int tx=BookTable(x,"RICH_RADIUS",0,0);
  if (!tx) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(2); }

  TableStamp(x,run_no,TABLEmaxrev);
  if (!FetchTable(x,tx,TABLEle)) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(3); }

  SkipTableField(x,1);
  int   major=TableMajor(x);
  int   minor=TableMinor(x);  
  int   run_ver=0;
  if (major==run_no) run_ver=minor+1;

  TableStamp(x,run_no,run_ver);
  SkipTableField(x,1);

  if (verb>1) {
    fprintf(stderr," $$$ RISE Inform: New parameters for run %d (minor=%d)\n",run_no,run_ver);
    if (major==run_no)
      fprintf(stderr," $$$ RISE Inform: Old version run %d (minor=%d)\n",major,minor);
  }

  float pressure   =parameters[0];
  float temperature=parameters[1];
  float radius     =parameters[2];
  float time       =parameters[3];

  RingSearchAttrib::minSearchRadius=parameters[4];
  RingSearchAttrib::maxSearchRadius=parameters[5];
  RingSearchAttrib::searchStep     =parameters[6];
  RingSearchAttrib::occupancyCut   =int(parameters[7]);
  RingSearchAttrib::peakThreshold  =int(parameters[8]);

  PutTableField(x,TABLEfloat,1,&pressure);
  PutTableField(x,TABLEfloat,1,&temperature);
  PutTableField(x,TABLEfloat,1,&radius);
  PutTableField(x,TABLEfloat,1,&time);
  PutTableField(x,TABLEfloat,1,&RingSearchAttrib::minSearchRadius);
  PutTableField(x,TABLEfloat,1,&RingSearchAttrib::maxSearchRadius);
  PutTableField(x,TABLEfloat,1,&RingSearchAttrib::searchStep);
  PutTableField(x,  TABLEint,1,&RingSearchAttrib::occupancyCut);
  PutTableField(x,  TABLEint,1,&RingSearchAttrib::peakThreshold);

  StoreTable(x,tx,0);
  if (verb>1) PrintTables(x,1);
  CloseTables(x);
}

extern "C"
int UpdateRadiusDatabase(int major, float *parameters,int verb) {
  RingSearchDB db;
  db.fillSearchParameters(major, parameters, verb);
  return 0;
}
