//----------------------------------------------------------------------------
//
//  Subroutines to fill HITB Table from marvin's structures
//  ========================================================
// 
//----------------------------------------------------------------------------

// include files
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <math.h>

// Arte banks,  C++ version
#include "arte/arte.hh"
#include "silarte/silarte.hh"
#include "silarte/EVinfo.hh"

#include "aframelib.h"
#include "vdsleda.h"
#include "marvin.h"
#include "grr.h"

// ---------------------------------------------------------------------------
//  Fill HITB table from Marvin_Sevt_t structure 
//  ============================================
// 
//  Author: Rainer Wanke / 3-Oct-1998
//
//  Returns:   GRR_OK    in case of no problems
//             GRR_FISHY otherwise
// 
// ---------------------------------------------------------------------------
int fillHITBfromSPARSEVT()
{
  char *routine = "fillHITBfromSPARSEVT";

  ArtePointer<GEDE> oldgede = 0;

  static bool  firsttime = true;
  static bool afirsttime = true;
  static bool bfirsttime = true;
  static bool cfirsttime = true;
  static bool xfirsttime = true;

  //--------------------------------------------------------------------------


  // First of all get event from marvin
  Marvin_Sevt_t *sevt = marvin_fetch_sevt();
  if ( !sevt ) {

    if(firsttime == true) {
      aframe_mess(routine,GRR_SERROR,
		  "NULL pointer to sparsified information!");
      firsttime = false;
    }

    return GRR_FISHY;
  }

  // check reasonable number of clusters in format 3
  Marvin_Header_t *header = marvin_sevtHeader( (char*) sevt->data );
  if ( header->format == 3 && header->ncl > 0x7FFF / 8 ) {

    if(afirsttime == true) {
      aframe_mess(routine,GRR_SERROR,
		  "Too many clusters (%d) in event --> skipping it!", header->ncl );
      afirsttime = false;
    }

    return GRR_FISHY;
  }

  // -------------------------------
  //  fill HITB and update GEDE bank
  // -------------------------------

  if ( silarte_iol.debug_mode.value )
    cout << "MC flag = " << CheckMC() << " -> filling HITB bank" << endl;

  // loop over clusters to fill HITB bank
  Marvin_Cluster_t *cluster = marvin_sevtCluster( (char*) sevt->data, -1 );
  while ( cluster ) {

    // print cluster if in debug
    if ( silarte_iol.debug_mode.value )
      cout << " marvin cluster " << cluster->side
	   << ": side = "  << cluster->side
	   << ", x = "     << cluster->cog
	   << ", width = " << cluster->width
	   << ", q = "     << cluster->charge
	   << ", var = "   << cluster->variance
	   << endl;
    

    // get GEDE id from database
    LedaPointer<class VDS_Side> side = VDS_Side->get->key( cluster->side );
    if ( !side ) {

      if(xfirsttime == true) {
	aframe_mess(routine,GRR_SERROR,
		    "SVD side %d not in database", cluster->side );
	xfirsttime = false;
      }

      cluster = marvin_sevtCluster(NULL, -1);
      continue;
    }
    int gederow = silarte_returnGEDE( side->slname, (int) side->lay, (int) side->sec );
    ArtePointer<GEDE> gede = ArtePointer<GEDE>( gederow );
    if ( gede == 0 ) {

      if(bfirsttime == true) {
	aframe_mess(routine,GRR_SERROR,
		    "GEDE row %d for side %d (SL=%s,lay=%d,sec=%d) is non-existent",
		    gederow, (int) side->id, side->slname, (int) side->lay, (int) side->sec );
	bfirsttime = false;
      }

      cluster = marvin_sevtCluster(NULL, -1);
      continue;
    }
    if ( gede->cmp != 1 ) {

      if(cfirsttime == true) {
	aframe_mess(routine,GRR_SERROR,
		    "GEDE row %d (side %d) is not an SVD GEDE table",
		    gederow, (int) side->id );
	cfirsttime = false;
      }

      cluster = marvin_sevtCluster(NULL, -1);
      continue;
    }


    // Keep right sign hits only (usually p=-, n=+, but taken from database)
    if ( (float) ((signed char) side->polarity) * cluster->charge < 0. ) {
      cluster = marvin_sevtCluster(NULL, -1);
      continue;
    }

    // cluster coordinates
    HITB hit;
    hit.z    = gede->zfw[0];
    hit.x    = cluster->cog * gede->pit + gede->xfw[0];
    hit.y    = cluster->sn;
    hit.q    = cluster->charge;
    hit.r    = silarte_iol.hit_resolution.value;
    hit.gede = gede;

    // append hit to HITB table
    ArtePointer<HITB> hitb = ArteTable<HITB>::push_back( hit );

    // update GEDE bank: hit1 only , if GEDEnr has changed,
    //                   hitl always (every additional hit overwrites it)
    //   !!! We rely on already ordered hits !!!
    if ( hitb->gede != oldgede ) {
      gede->hit1 = hitb;
      oldgede = hitb->gede;
    }
    gede->hitl = hitb;

    // get next cluster
    cluster = marvin_sevtCluster(NULL, -1);

  }

  return GRR_OK;
}
