#ifndef _L3_PARAMETER_HH_
#define _L3_PARAMETER_HH_
// *****************************************************************************
//
// source:    L3_Parameter.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************

#include "L3_Defines.hh"
#include <vector>

extern "C" void tltparam_(void);

class L3_Parameter {
public:
  L3_Parameter();
  L3_Parameter(const L3_Parameter&);  
  static void init(int argc,char** argv);  //initialize tlt parameters
  static int readFile(char *filename); //read parameters from file
  static int saveFile(char *filename); //write current settings to file
  static void showParameter();         //print current settings to screen

  //return various parameters
  inline static float StereoResol(int sl0=0);
  inline static float MinMomentumOfRefTracks();
  inline static float margin();
  inline static int run(); 
  inline static int maxCommonHits(); 
  inline static float maxProbDiff(); 
  inline static float minProb(); 
  inline static float Offset_TargetBox_zmax(); 
  inline static float Offset_TargetBox_zmin();
  inline static float TargetBox_Width();
  inline static float SearchWindow_HalfMinimalWidth(); // 0.5 * Minimal Width of SearchWindow (old name: dmin)
  inline static float SearchWindow_HalfMaximalWidth(); // 0.5 * Maximal Width of SearchWindow (old name: dmax)
  inline static float SearchWindow_HalfOpeningAngle(); // 0.5 * Opening Angle of SearchWindow (old name: tm)
  inline static int Debug();                           // return debug level 
  inline static int Trace();                           // return debug level 
  inline static int Info();                            // write out info
  inline static float SearchBandSize();                // width of search band around wire
                                                       //for primary vertex
  inline static int Ana(); //kind of analysis to carry out
  inline static float chi2cut(int ndof); 
  //returns the chi2-cut for different numbers of degrees of freedom (ndof)
  inline static float sigmaxy();
  inline static float sigmaz();
  inline static int setDebug(int newvalue);
  inline static float meanMomentum();
  inline static float multipleScattering();
  static float setMultipleScattering(float newvalue); //returns old value
  inline static float K();
private:
//Minimum momentum for a track to be a reference track (only makes sense for MC-Events) 
  static float the_MinMomentumOfRefTracks;
//run chooses specific reconstruction codes for different detector setups
  static int   the_run;
//Spacepoint reconstruction parameters
  static float the_margin;
  static float the_StereoResol[7];
//Clone/Ghost killer parameters  
  static int   the_maxCommonHits; //  maximum number of shared hits of two tracks
  static float the_maxProbDiff;   //  maximal probability loss allowed for additional hits
  static float the_minProb;       //  minimal probability to accept a track
//Track reconstruction parameters
  static float the_SearchWindow_HalfMinimalWidth; // 0.5 * Minimal Width of SearchWindow 
  static float the_SearchWindow_HalfMaximalWidth; // 0.5 * Maximal Width of SearchWindow 
  static float the_SearchWindow_HalfOpeningAngle; // 0.5 * Opening Angle of SearchWindow
//Target constraint parameters
  static float the_Offset_TargetBox_zmax; // Absolut z-distance (FirstTarget-FrontSideOfTargetBox)
  static float the_Offset_TargetBox_zmin; // Absolut z-distance (LastTarget-BackSideOfTargetBox)
  static float the_TargetBox_Width;       // Absolut width of quadratic (x-y plane) TargetBox
  static vector<float> chi2cuts;
  static int tlt_debug;
  static float the_SearchBandSize;
  static int the_Ana;
  static float the_sigmaxy;
  static float the_sigmaz;
  static float the_meanp;
  static float the_msfak;
  static float the_k;

  static const int minNdof = 1;
  static const int maxNdof = 7*4-4;  //(7 SL) * (4 Hits/SL) - (4 track parameter) 

  static void setminProb(float prob);
  static float calcChi2cut(int ndof);  
#ifdef L3_ARTE_FRAME
  //needed for kuip interface
  friend void tltparam_(void);
  static int setParameter(char *name,float value);  //returns 0 if name is no valid parameter
  static void helpARTE();
#endif
  static void user_init();
  static void compileInfo();
};

#include "L3_Parameter.icc"

#endif
